/*
----------------------------------------------------------------------
	
	Alain Chebrou

	Spartan RT kernel

	bsp.c	target specific "C" routines

	When		Who	What
	30/12/13	ac	Creation

	For intrinsics compile with: /Oi
----------------------------------------------------------------------
*/
/*
	Windows specific BSP for debug and unit tests
*/

#define _WIN32_WINNT 0x0501		// For CreateWaitableTimer
#include	<windows.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<process.h>
#include	<conio.h>

// To declare an interlocked function for use as an intrinsic,
// include intrin.h and put the function in a #pragma intrinsic statement.
#include <intrin.h>
#pragma intrinsic (_InterlockedIncrement)
#pragma intrinsic (_InterlockedDecrement)

#include	"aa.h"
#include	"aakernel.h"
#include	"aaprintf.h"
#include	"aalogmes.h"

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//	Windows Visual C Simulator
//--------------------------------------------------------------------------------
#define	BSP_THREAD_STACK_SIZE		8192

#define	BSP_MAX_INTR		8			// Count of interrupts in interruptRegister
#define	INTR_TICK_INDEX		0			// Intex of tick interrupt

static	HANDLE				hInterruptEvent ;
static	int32_t				interruptRegister ;
static	CRITICAL_SECTION	interruptMutex ;
static	void (*intHandlers [BSP_MAX_INTR]) (void) ;	// Array of pointer to handlers functions

static	uint32_t			bspDisableInterrupt ;	// Count of aaCriticalEnter : Int enabled if 0
static	uint32_t			bspNeedTaskSwitch ;		// Task switch request to interrupt controller

static	void	interruptManagerThread	(void) ;
static	void	tickIntSimulatorThread	(void) ;
static	void	tickIntHandler			(void) ;	// Tick interrupt handler

//	Heap simulation
uint8_t		bspHeap [BSP_HEAP_SIZE] ;


//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
#include <intrin.h>

#pragma intrinsic(_BitScanReverse)
#pragma intrinsic(_BitScanForward)

//--------------------------------------------------------------------------------

uint32_t	bspGetSysClock 		(void)
{
	return 72000000 ;	// Any frequency
}

//--------------------------------------------------------------------------------
//	CPU usage measurement

static	int64_t		tickTimerFrequency ; 		// MHz
static	int64_t		tickPrev ;

static	void	tickTimerInit (void) 
{
	LARGE_INTEGER	tickFreq, currTicks ;

	QueryPerformanceFrequency (& tickFreq);
	tickTimerFrequency = tickFreq.QuadPart / 1000000 ;

	QueryPerformanceCounter (& currTicks);
	tickPrev = currTicks.QuadPart ;
}

uint32_t	bspGetTimerDelta (void)
{
	LARGE_INTEGER	currTicks ;
	int64_t			now ;
	uint32_t		delta ;

	QueryPerformanceCounter (& currTicks);
	now = currTicks.QuadPart ;

	delta = (uint32_t) ((now - tickPrev) / tickTimerFrequency) ;
	tickPrev = now ;
	return delta ;
}

//--------------------------------------------------------------------------------

int	main ()
{
	aaMain () ;		// Never return
	return 1 ;
}

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//	Console simulator

void	bspPutChar	(char cc)
{
	aaCriticalEnter () ;	// To avoid thread suspended in Windows
	putc (cc, stdout) ;
	aaCriticalExit () ;
}

char	bspGetChar	(void)
{
	char cc ;

	aaCriticalEnter () ;	// To avoid thread suspended in Windows
	while (_kbhit () == 0)
	{
		;
	}
	cc = _getch () ;
	aaCriticalExit () ;
	return cc ;
}

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
static	char	assertStr [256] ;

void bspAssertFailed (uint8_t * fileName, uint32_t line)
{
	sprintf (assertStr, "Assert failed: file %s, line %d\n", fileName, line) ;
	OutputDebugString (assertStr) ;
	while (1)
	{
	}
}

//--------------------------------------------------------------------------------
// Perform task switch from scheduler

void	bspTaskSwitch (void)
{
	// Simulate a SVC call (software interrupt)
/*
	bspNeedTaskSwitch = 1 ;
	aaCriticalExit () ;		// Enable interrupt and SetEvent
	AA_ASSERT (bspDisableInterrupt == 0) ;
	while (bspNeedTaskSwitch == 1)
	{
		Yield () ;	// Do nothing
	}
*/
	bspNeedTaskSwitch = 1 ;
	if (bspDisableInterrupt == 1)
	{
		aaCriticalExit () ;		// Enable interrupt and SetEvent
		AA_ASSERT (bspDisableInterrupt == 0) ;
		while (bspNeedTaskSwitch == 1)
		{
			Yield () ;	// Do nothing
		}
	}
	else
	{
		aaCriticalExit () ;		// decrement and no setEvent
	}
}

// Perform task switch at end of ISR
void	bspTaskSwitchFromIsr (void)
{
	// Simulate a SVC call (software interrupt)
	// Win32 simulator: done at end of interrupt management
}

//--------------------------------------------------------------------------------
// Start kernel, task to start is pAaNextTcb

void	bspStart ()
{
	tickTimerInit () ;

	// Create tick simulator thread
	CreateThread (NULL, BSP_THREAD_STACK_SIZE, (LPTHREAD_START_ROUTINE ) tickIntSimulatorThread, (void *) NULL, 0, NULL );

	// Start first task
	pAaCurrentTcb = pAaNextTcb ;
	ResumeThread ((HANDLE) (* pAaCurrentTcb->pTopOfStack)) ;

	// In real port this thread goes to the idle task.
	SuspendThread (GetCurrentThread()) ;
}

//--------------------------------------------------------------------------------
// Initialize basic hardware

void	bspInit ()
{
	int32_t	ii ;

	aaSetStdOut (bspPutChar) ;
	aaSetStdIn   (bspGetChar) ;
	aaLogMesSetPutChar (bspPutChar) ;

	// Init simulation environment
	InitializeCriticalSection (& interruptMutex) ;
	hInterruptEvent = CreateEvent (NULL, TRUE, FALSE, NULL) ; // lpEventAttributes, bManualReset, bInitialState, lpName
	interruptRegister = 0 ;
	bspNeedTaskSwitch = 0 ;

	// Init interrupt handlers
	for (ii = 0 ; ii < BSP_MAX_INTR ; ii++)
	{
		intHandlers[ii] = NULL ;
	}
	intHandlers[INTR_TICK_INDEX] = tickIntHandler ;


	// Init BSP environment
	bspDisableInterrupt = 0 ;

	// Create interrupt manager thread
	CreateThread (NULL, BSP_THREAD_STACK_SIZE, (LPTHREAD_START_ROUTINE ) interruptManagerThread, (void *) NULL, 0, NULL );
}

//--------------------------------------------------------------------------------
//	Create the environment to start a task

// This is the real entry point of the task.
// This function call the user entry point
// and when the user entry function returns, the task is deleted by aaTaskEnd()
void	bspTaskStart (uint32_t * pArg)
{
	(* (aaTaskFunction) ((uintptr_t)(pArg [-1]))) ((uintptr_t)pArg [-2]) ;
	aaTaskEnd () ;
}

void	bspTaskInit (aaTcb_t * pTcb, aaTaskFunction pEntry, uintptr_t arg)
{
	// Simulate high to low address stack grow
	pTcb->pTopOfStack = pTcb->pStack + (pTcb->stackSize - 1) ;

	// The highest stack word contains the windows thread HANDLE of the task
	pTcb->pTopOfStack [0] = (uintptr_t) CreateThread (NULL, BSP_THREAD_STACK_SIZE, (LPTHREAD_START_ROUTINE ) bspTaskStart, (void *) pTcb->pTopOfStack, CREATE_SUSPENDED, NULL );

	// One word below contains the task entry function argument
	pTcb->pTopOfStack[-2] = (uintptr_t) arg ;

	// Two word below contains the task entry function address
	pTcb->pTopOfStack[-1] = (uintptr_t) pEntry ;
}

//--------------------------------------------------------------------------------

void	aaCriticalEnter (void)
{
	// If RTK is not running, ignore
	if (aaIsRunning != 0)
	{
		_InterlockedIncrement (& bspDisableInterrupt) ;
		aaInCriticalCounter++ ;

		//#pragma omp atomic
//		bspCriticalNesting ++ ;
	}
}

//--------------------------------------------------------------------------------

void	aaCriticalExit (void)
{
	if (aaIsRunning != 0)
	{
		aaInCriticalCounter-- ;
		if (bspDisableInterrupt == 1)
		{
			_InterlockedDecrement (& bspDisableInterrupt) ;
			SetEvent (hInterruptEvent) ;
		}
		else
		{
			_InterlockedDecrement (& bspDisableInterrupt) ;
		}
	}
}

//--------------------------------------------------------------------------------

uint32_t	bspGetTickRate		(void)
{
	return BSP_TICK_RATE ;
}

//--------------------------------------------------------------------------------

uint8_t	bspSetTickRate		(uint32_t tickRateHz)
{
	// TODO
	return FALSE ;
}

//--------------------------------------------------------------------------------
//	Hardware interrupt controller simulation thread

static	void	interruptManagerThread (void)
{
	int32_t		ii ;
	int32_t		bDoSchedule ;
	CONTEXT		threadContext;

	while (1)
	{
		WaitForSingleObject (hInterruptEvent, INFINITE) ;

		SuspendThread ((HANDLE) (* pAaCurrentTcb->pTopOfStack)) ;
		threadContext.ContextFlags = CONTEXT_INTEGER ;
		GetThreadContext ((HANDLE)(* pAaCurrentTcb->pTopOfStack), & threadContext) ; // Block until suspended ?
		bDoSchedule = 0 ;

		// Reset hInterruptEvent event
		ResetEvent (hInterruptEvent) ;

		if (bspDisableInterrupt != 0)
		{
			// Interrupt locked: no schedule, no interrupt management
			ResumeThread ((HANDLE) (* pAaCurrentTcb->pTopOfStack)) ;
			continue ;
		}

		// Check if task switch is requested
		if (bspNeedTaskSwitch != 0)
		{
			// pAaCurrentTcb is already suspended
			// Wake up next thread
			bspNeedTaskSwitch = 0 ;
			bDoSchedule = 1 ;
		}

		// Simulator: avoid new interrupt while looping on interrupt register
		EnterCriticalSection (& interruptMutex) ;

		// If nothing to do: resume current thread ;
		if (bDoSchedule == 0  &&  interruptRegister == 0)
		{
			LeaveCriticalSection (& interruptMutex) ;
			ResumeThread ((HANDLE) (* pAaCurrentTcb->pTopOfStack)) ;
			continue ;
		}

		aaIntEnter () ;

		// Manage pending interrupts
		if (interruptRegister != 0)
		{
			// Call handler for interrupts signaled in interruptRegister
			for (ii = 0 ; ii < BSP_MAX_INTR ; ii++)
			{
				if (interruptRegister & (1 << ii))
				{
					if (intHandlers[ii] != NULL)
					{
						(* intHandlers[ii]) ();
					}
				}
				interruptRegister &= ~(1 << ii);	// Clear interrupt bit
			}
		}

		aaIntExit () ; 

		// Schedule higher priority thread
		if (pAaCurrentTcb != pAaNextTcb)
		{
			aaTaskSwitch () ;
		}
		ResumeThread ((HANDLE) (* pAaCurrentTcb->pTopOfStack)) ;

		LeaveCriticalSection (& interruptMutex) ;
	}
}

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//	Tick interrupt handler
static	void	tickIntHandler (void)
{
	aaTick (); 
}

//--------------------------------------------------------------------------------
// Hardware Timer simulator thread

static	void	tickIntSimulatorThread (void)
{
	uint32_t		cnt = 0 ;
	DWORD			retVal = 0 ;
	HANDLE			hTimer ;
	LARGE_INTEGER	dueTime; 

	hTimer = CreateWaitableTimer (NULL, FALSE, NULL) ;
	if (hTimer == NULL)
	{
		DWORD err = GetLastError (); 
		while (1)
		{
		}
	}

	dueTime.QuadPart = -10000000 / BSP_TICK_RATE ;
	if (! SetWaitableTimer (	hTimer,
						& dueTime,				// Due time 100ns, neg value => relative time
						1000 / BSP_TICK_RATE,	// Period ms
						NULL,					// pfnCompletionRoutine
						NULL,					// lpArgToCompletionRoutine
						FALSE))					// fResume
	{
		printf ("SetWaitableTimer failed\n") ;
		while (1) ;
	}

	while (1)
	{
		retVal = WaitForSingleObject (hTimer, INFINITE) ;
		cnt++ ;

		EnterCriticalSection (& interruptMutex) ;
		interruptRegister |= 1 << INTR_TICK_INDEX ;
		SetEvent (hInterruptEvent) ;
		LeaveCriticalSection (& interruptMutex) ;
	}

	CloseHandle (hTimer) ;
}

//--------------------------------------------------------------------------------
