/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	Devices vector table for GCC based toolchain:

	STM32F103 is a very complicated family:
		Define STM32F103x6 for : Low Density		F103x4  F103x6
		Define STM32F103xB for : Medium Density		F103x8  F103xB
		Define STM32F103xE for : High Density		F103xC  F103xD  F103xE
		Define STM32F103xG for : XL Density			F103xF  F103xG

	Exception vectors and default handlers for Cortex-M3, Cortex-M4, Cortex-M7
	Interrupts vector and default handlers

	After Reset the Cortex-M processor is in Thread mode,
	priority is Privileged, and the stack is set to Main.

	When		Who	What
	28/04/17	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2020 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/
/*
	__ARM_ARCH_6M__		Set to 1 when generating code for Armv6-M (Cortex-M0, Cortex-M0+, Cortex-M1)
	__ARM_ARCH_7M__		Set to 1 when generating code for Armv7-M (Cortex-M3)
	__ARM_ARCH_7EM__	Set to 1 when generating code for Armv7-M (Cortex-M4, Cortex-M7) with FPU
*/
// One of STM32F103x6 STM32F103xB STM32F103xE STM32F103xG must be defined
#include <stm32f1xx.h>

#if defined (DEBUG)
	#if defined(__CC_ARM)
		#define __DEBUG_BKPT()	__breakpoint(0)
	#else
		#define __DEBUG_BKPT()	__ASM volatile ("bkpt 0")
	#endif
#else
	#define __DEBUG_BKPT()	while (1) { }
#endif

extern void __attribute__((noreturn)) start_ (void);	// The entry point in bsp.c

// ----------------------------------------------------------------------------
// Exception Stack Frame of the Cortex-M0-M0+-M3-M4-M7 processors.

typedef struct
{
	uint32_t	r0 ;
	uint32_t	r1 ;
	uint32_t	r2 ;
	uint32_t	r3 ;
	uint32_t	r12 ;
	uint32_t	lr ;
	uint32_t	pc ;
	uint32_t	psr ;
	#if  defined(__ARM_ARCH_7EM__)
		uint32_t s[16];				// Floating point registers
	#endif

} exceptionStackFrame ;

void	  HardFault_Handler_C (exceptionStackFrame * frame, uint32_t lr) ;

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

	void	  UsageFault_Handler_C	(exceptionStackFrame * frame, uint32_t lr) ;
	void	  BusFault_Handler_C	(exceptionStackFrame * frame, uint32_t lr) ;

#endif // defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

// ----------------------------------------------------------------------------
// Forward declarations of exception handlers
// There is specific default handlers for these exception at the end of the file

extern void		Reset_Handler		(void) ;
extern void		NMI_Handler			(void) ;
extern void		HardFault_Handler	(void) ;

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)
	extern void	MemManage_Handler	(void) ;
	extern void	BusFault_Handler	(void) ;
	extern void	UsageFault_Handler	(void) ;
	extern void	DebugMon_Handler	(void) ;
#endif

extern void		SVC_Handler			(void) ;
extern void		PendSV_Handler		(void) ;
extern void		SysTick_Handler		(void) ;

// Generic default handler for interruptions
void __attribute__((weak)) Default_Handler (void) ;

// ----------------------------------------------------------------------------
// Forward declaration of the IRQ handlers with weak definitions.
// The application can defines a handler with the same name and take precedence

void __attribute__ ((weak, alias ("Default_Handler"))) WWDG_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) PVD_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TAMPER_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) RTC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) FLASH_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) RCC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI0_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Channel1_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Channel2_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Channel3_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Channel4_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Channel5_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Channel6_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA1_Channel7_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) ADC1_2_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USB_HP_CAN1_TX_IRQHandler	(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USB_LP_CAN1_RX0_IRQHandler	(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN1_RX1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) CAN1_SCE_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI9_5_IRQHandler			(void) ;

#if ! defined (STM32F103xG)
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_BRK_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_UP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_TRG_COM_IRQHandler		(void) ;
#endif

void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_CC_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C1_EV_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C1_ER_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C2_EV_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) I2C2_ER_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USART1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USART2_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USART3_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) EXTI15_10_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) RTCAlarm_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) USBWakeUp_IRQHandler			(void) ;

#if defined (STM32F103xE)
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_BRK_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_UP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_TRG_COM_IRQHandler		(void) ;
#endif

#if defined (STM32F103xG)
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_BRK_TIM9_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_UP_TIM10_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM1_TRG_COM_TIM11_IRQHandler(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_BRK_TIM12_IRQHandler	(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_UP_TIM13_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_TRG_COM_TIM14_IRQHandler(void) ;
#endif

#if defined (STM32F103xE) | defined (STM32F103xG)
void __attribute__ ((weak, alias ("Default_Handler"))) TIM8_CC_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) ADC3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) FSMC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SDIO_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) SPI3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) UART4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) UART5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM6_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) TIM7_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Channel1_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Channel2_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Channel3_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler"))) DMA2_Channel4_5_IRQHandler	(void) ;
#endif

// ----------------------------------------------------------------------------

extern unsigned int _estack_ ;				// From linker script: initial stack address

typedef void (* const pHandler) (void) ;	// Type of the interruptions vector array

// ----------------------------------------------------------------------------

// The table of interrupt handlers. It has an explicit section name
// and relies on the linker script to place it at the correct location in memory.

// Can't place integer and function pointer in one array (Forbidden by ISO C)
__attribute__ ((section(".isr_stackaddress"),used)) unsigned int * isrStackAdress =
{
	& _estack_
} ;

__attribute__ ((section(".isr_vectors"),used)) pHandler isrVectors_[] =
{
	// Cortex-M Core Handlers
	//									   The initial stack pointer is in the section .isr_stackaddress
	Reset_Handler,						// The reset handler

	NMI_Handler,						// The NMI handler
	HardFault_Handler,					// The hard fault handler

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)
	MemManage_Handler,					// The MPU fault handler
	BusFault_Handler,					// The bus fault handler
	UsageFault_Handler,					// The usage fault handler
#else
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
#endif
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
	SVC_Handler,						// SVCall handler
#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)
	DebugMon_Handler,					// Debug monitor handler
#else
	0,									// Reserved
#endif
	0,									// Reserved
	PendSV_Handler,						// The PendSV handler
	SysTick_Handler,					// The SysTick handler

	// ----------------------------------------------------------------------
	// Common to all F103 devices
	WWDG_IRQHandler,					// 0	Window WatchDog
	PVD_IRQHandler,						// 1	PVD through EXTI Line detection
	TAMPER_IRQHandler,					// 2	Tamper through the EXTI line
	RTC_IRQHandler,						// 3	RTC Wakeup through the EXTI line
	FLASH_IRQHandler,					// 4	FLASH
	RCC_IRQHandler,						// 5	RCC
	EXTI0_IRQHandler,					// 6	EXTI Line0
	EXTI1_IRQHandler,					// 7	EXTI Line1
	EXTI2_IRQHandler,					// 8	EXTI Line2
	EXTI3_IRQHandler,					// 9	EXTI Line3
	EXTI4_IRQHandler,					// 10	EXTI Line4
	DMA1_Channel1_IRQHandler,			// 11	DMA1 Channel 1
	DMA1_Channel2_IRQHandler,			// 12	DMA1 Channel 2
	DMA1_Channel3_IRQHandler,			// 13	DMA1 Channel 3
	DMA1_Channel4_IRQHandler,			// 14	DMA1 Channel 4
	DMA1_Channel5_IRQHandler,			// 15	DMA1 Channel 5
	DMA1_Channel6_IRQHandler,			// 16	DMA1 Channel 6
	DMA1_Channel7_IRQHandler,			// 17	DMA1 Channel 7
	ADC1_2_IRQHandler,					// 18	ADC1, ADC2

#if defined (STM32F103x6)		// Low density devices

	USB_HP_CAN1_TX_IRQHandler,			// 19	USB High Priority or CAN1 TX
	USB_LP_CAN1_RX0_IRQHandler,			// 20	USB Low  Priority or CAN1 RX0
	CAN1_RX1_IRQHandler,				// 21	CAN1 RX1
	CAN1_SCE_IRQHandler,				// 22	CAN1 SCE
	EXTI9_5_IRQHandler,					// 23	EXTI Line 9..5
	TIM1_BRK_IRQHandler,				// 24	TIM1 Break
	TIM1_UP_IRQHandler,					// 25	TIM1 Update
	TIM1_TRG_COM_IRQHandler,			// 26	TIM1 Trigger and Commutation
	TIM1_CC_IRQHandler,					// 27	TIM1 Capture Compare
	TIM2_IRQHandler,					// 28	TIM2
	TIM3_IRQHandler,					// 29	TIM3
	0,									// 30	Reserved
	I2C1_EV_IRQHandler,					// 31	I2C1 Event
	I2C1_ER_IRQHandler,					// 32	I2C1 Error
	0,									// 33	Reserved
	0,									// 34	Reserved
	SPI1_IRQHandler,					// 35	SPI1
	0,									// 36	Reserved
	USART1_IRQHandler,					// 37	USART1
	USART2_IRQHandler,					// 38	USART2
	0,									// 39	Reserved
	EXTI15_10_IRQHandler,				// 40	EXTI Line 15..10
	RTCAlarm_IRQHandler,				// 41	RTC Alarm through EXTI Line
	USBWakeUp_IRQHandler,				// 42	USB Wakeup from suspend
	0,									// 43
	0,									// 44
	0,									// 45
	0,									// 46
	0,									// 47
	0,									// 48
	0,									// 49
	(pHandler)0xF108F85F				// @0x108. BootRAM This is for boot in RAM mode.

#elif defined (STM32F103xB)		// Medium density devices

	USB_HP_CAN1_TX_IRQHandler,			// 19	USB High Priority or CAN1 TX
	USB_LP_CAN1_RX0_IRQHandler,			// 20   USB Low  Priority or CAN1 RX0
	CAN1_RX1_IRQHandler,				// 21   CAN1 RX1
	CAN1_SCE_IRQHandler,				// 22   CAN1 SCE
	EXTI9_5_IRQHandler,					// 23   EXTI Line 9..5
	TIM1_BRK_IRQHandler,				// 24   TIM1 Break
	TIM1_UP_IRQHandler,					// 25   TIM1 Update
	TIM1_TRG_COM_IRQHandler,			// 26   TIM1 Trigger and Commutation
	TIM1_CC_IRQHandler,					// 27   TIM1 Capture Compare
	TIM2_IRQHandler,					// 28   TIM2
	TIM3_IRQHandler,					// 29   TIM3
	TIM4_IRQHandler,					// 30   TIM4
	I2C1_EV_IRQHandler,					// 31   I2C1 Event
	I2C1_ER_IRQHandler,					// 32   I2C1 Error
	I2C2_EV_IRQHandler,					// 33   I2C2 Event
	I2C2_ER_IRQHandler,					// 34   I2C2 Error
	SPI1_IRQHandler,					// 35   SPI1
	SPI2_IRQHandler,					// 36   SPI2
	USART1_IRQHandler,					// 37   USART1
	USART2_IRQHandler,					// 38   USART2
	USART3_IRQHandler,					// 39   USART3
	EXTI15_10_IRQHandler,				// 40   EXTI Line 15..10
	RTCAlarm_IRQHandler,				// 41   RTC Alarm through EXTI Line
	USBWakeUp_IRQHandler,				// 42   USB Wakeup from suspend
	0,									// 43
	0,									// 44
	0,									// 45
	0,									// 46
	0,									// 47
	0,									// 48
	0,									// 49
	(pHandler)0xF108F85F				// @0x108. BootRAM this is for boot in RAM mode.

#elif defined (STM32F103xE)		// High density devices

	USB_HP_CAN1_TX_IRQHandler,			// 19	USB High Priority or CAN1 TX
	USB_LP_CAN1_RX0_IRQHandler,			// 20	USB Low  Priority or CAN1 RX0
	CAN1_RX1_IRQHandler,				// 21	CAN1 RX1
	CAN1_SCE_IRQHandler,				// 22	CAN1 SCE
	EXTI9_5_IRQHandler,					// 23	EXTI Line 9..5
	TIM1_BRK_IRQHandler,				// 24	TIM1 Break
	TIM1_UP_IRQHandler,					// 25	TIM1 Update
	TIM1_TRG_COM_IRQHandler,			// 26	TIM1 Trigger and Commutation
	TIM1_CC_IRQHandler,					// 27	TIM1 Capture Compare
	TIM2_IRQHandler,					// 28	TIM2
	TIM3_IRQHandler,					// 29	TIM3
	TIM4_IRQHandler,					// 30	TIM4
	I2C1_EV_IRQHandler,					// 31	I2C1 Event
	I2C1_ER_IRQHandler,					// 32	I2C1 Error
	I2C2_EV_IRQHandler,					// 33	I2C2 Event
	I2C2_ER_IRQHandler,					// 34	I2C2 Error
	SPI1_IRQHandler,					// 35	SPI1
	SPI2_IRQHandler,					// 36	SPI2
	USART1_IRQHandler,					// 37	USART1
	USART2_IRQHandler,					// 38	USART2
	USART3_IRQHandler,					// 39	USART3
	EXTI15_10_IRQHandler,				// 40	EXTI Line 15..10
	RTCAlarm_IRQHandler,				// 41	RTC Alarm through EXTI Line
	USBWakeUp_IRQHandler,				// 42	USB Wakeup from suspend
	TIM8_BRK_IRQHandler,				// 43	TIM8 Break
	TIM8_UP_IRQHandler,					// 44	TIM8 Update
	TIM8_TRG_COM_IRQHandler,			// 45	TIM8 Trigger and Commutation
	TIM8_CC_IRQHandler,					// 46	TIM8 Capture Compare
	ADC3_IRQHandler,					// 47	ADC3
	FSMC_IRQHandler,					// 48	FSMC
	SDIO_IRQHandler,					// 49	SDIO
	TIM5_IRQHandler,					// 50	TIM5
	SPI3_IRQHandler,					// 51	SPI3
	UART4_IRQHandler,					// 52	UART4
	UART5_IRQHandler,					// 53	UART5
	TIM6_IRQHandler,					// 54	TIM6
	TIM7_IRQHandler,					// 55	TIM7
	DMA2_Channel1_IRQHandler,			// 56	DMA2 Channel1
	DMA2_Channel2_IRQHandler,			// 57	DMA2 Channel2
	DMA2_Channel3_IRQHandler,			// 58	DMA2 Channel3
	DMA2_Channel4_5_IRQHandler,			// 59	DMA2 Channel4 & Channel5
	0,				// 60
	0,				// 61
	0,				// 62
	0,				// 63
	0,				// 64
	0,				// 65
	0,				// 66
	0,				// 67
	0,				// 68
	0,				// 69
	0,				// 70
	0,				// 71
	0,				// 72
	0,				// 73
	0,				// 74
	0,				// 75
	0,				// 76
	0,				// 77
	0,				// 78
	0,				// 79
	0,				// 80
	0,				// 81
	0,				// 82
	0,				// 83
	0,				// 84
	0,				// 85
	0,				// 86
	0,				// 87
	0,				// 88
	0,				// 89
	0,				// 90
	0,				// 91
	0,				// 92
	0,				// 93
	0,				// 94
	0,				// 95
	0,				// 96
	0,				// 97
	0,				// 98
	0,				// 99
	0,				// 100
	0,				// 101
	0,				// 102
	0,				// 103
	(pHandler)0xF1E0F85F				// @0x1E0. BootRAM this is for boot in RAM mode.

#elif defined (STM32F103xG)		// XL devices

	USB_HP_CAN1_TX_IRQHandler,			// 19	USB High Priority or CAN1 TX
	USB_LP_CAN1_RX0_IRQHandler,			// 20	USB Low  Priority or CAN1 RX0
	CAN1_RX1_IRQHandler,				// 21	CAN1 RX1
	CAN1_SCE_IRQHandler,				// 22	CAN1 SCE
	EXTI9_5_IRQHandler,					// 23	EXTI Line 9..5
	TIM1_BRK_TIM9_IRQHandler,			// 24	TIM1 Break and TIM9
	TIM1_UP_TIM10_IRQHandler,			// 25	TIM1 Update and TIM10
	TIM1_TRG_COM_TIM11_IRQHandler,		// 26	TIM1 Trigger and Commutation and TIM11
	TIM1_CC_IRQHandler,					// 27	TIM1 Capture Compare
	TIM2_IRQHandler,					// 28	TIM2
	TIM3_IRQHandler,					// 29	TIM3
	TIM4_IRQHandler,					// 30	TIM4
	I2C1_EV_IRQHandler,					// 31	I2C1 Event
	I2C1_ER_IRQHandler,					// 32	I2C1 Error
	I2C2_EV_IRQHandler,					// 33	I2C2 Event
	I2C2_ER_IRQHandler,					// 34	I2C2 Error
	SPI1_IRQHandler,					// 35	SPI1
	SPI2_IRQHandler,					// 36	SPI2
	USART1_IRQHandler,					// 37	USART1
	USART2_IRQHandler,					// 38	USART2
	USART3_IRQHandler,					// 39	USART3
	EXTI15_10_IRQHandler,				// 40	EXTI Line 15..10
	RTCAlarm_IRQHandler,				// 41	RTC Alarm through EXTI Line
	USBWakeUp_IRQHandler,				// 42	USB Wakeup from suspend
	TIM8_BRK_TIM12_IRQHandler,			// 43	TIM8 Break and TIM12
	TIM8_UP_TIM13_IRQHandler,			// 44	TIM8 Update and TIM13
	TIM8_TRG_COM_TIM14_IRQHandler,		// 45	TIM8 Trigger and Commutation and TIM14
	TIM8_CC_IRQHandler,					// 46	TIM8 Capture Compare
	ADC3_IRQHandler,					// 47	ADC3
	FSMC_IRQHandler,					// 48	FSMC
	SDIO_IRQHandler,					// 49	SDIO
	TIM5_IRQHandler,					// 50	TIM5
	SPI3_IRQHandler,					// 51	SPI3
	UART4_IRQHandler,					// 52	UART4
	UART5_IRQHandler,					// 53	UART5
	TIM6_IRQHandler,					// 54	TIM6
	TIM7_IRQHandler,					// 55	TIM7
	DMA2_Channel1_IRQHandler,			// 56	DMA2 Channel1
	DMA2_Channel2_IRQHandler,			// 57	DMA2 Channel2
	DMA2_Channel3_IRQHandler,			// 58	DMA2 Channel3
	DMA2_Channel4_5_IRQHandler,			// 59	DMA2 Channel4 & Channel5
	0,				// 60
	0,				// 61
	0,				// 62
	0,				// 63
	0,				// 64
	0,				// 65
	0,				// 66
	0,				// 67
	0,				// 68
	0,				// 69
	0,				// 70
	0,				// 71
	0,				// 72
	0,				// 73
	0,				// 74
	0,				// 75
	0,				// 76
	0,				// 77
	0,				// 78
	0,				// 79
	0,				// 80
	0,				// 81
	0,				// 82
	0,				// 83
	0,				// 84
	0,				// 85
	0,				// 86
	0,				// 87
	0,				// 88
	0,				// 89
	0,				// 90
	0,				// 91
	0,				// 92
	0,				// 93
	0,				// 94
	0,				// 95
	0,				// 96
	0,				// 97
	0,				// 98
	0,				// 99
	0,				// 100
	0,				// 101
	0,				// 102
	0,				// 103
	(pHandler)0xF1E0F85F				// @0x1E0. BootRAM this is for boot in RAM mode.

#else
	#error "Not supported"
#endif
} ;

// ----------------------------------------------------------------------------
// Processor jumps here if an unexpected interrupt occurs or a
// specific handler is not present in the application code.
// When in DEBUG, triggers a debug exception to warn the user

void __attribute__ ((section(".isr_handlers"))) Default_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Default exception handlers. Override the ones here by defining your own
// handler routines in your application code.
// When in DEBUG, triggers a debug exception to warn the user

void __attribute__ ((section(".isr_handlers"), noreturn, naked)) Reset_Handler (void)
{
	start_ () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) NMI_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) SVC_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) PendSV_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// Hard Fault handler wrapper in assembly.
// It extracts the location of stack frame and passes it to handler in C as a pointer.
// We also pass the LR value as second parameter.
// (Based on Joseph Yiu's, The Definitive Guide to ARM Cortex-M3 and
// Cortex-M4 Processors, Third Edition, Chap. 12.8, page 402).

// naked attribute:
// Use this attribute on the ARM ports to indicate that the specified function
// does not need prologue/epilogue sequences generated by the compiler.
// It is up to the programmer to provide these sequences.
// The only statements that can be safely included in naked functions are asm statements that do not have operands.
// All other statements, including declarations of local variables, if statements, and so forth, should be avoided.
// Naked functions should be used to implement the body of an assembly function,
// while allowing the compiler to construct the requisite function declaration for the assembler.

void __attribute__ ((section(".isr_handlers"),weak,naked)) HardFault_Handler (void)
{
	__ASM volatile
	(
		" tst	lr,#4	\n"
		" ite	eq		\n"
		" mrseq	r0,msp	\n"
		" mrsne	r0,psp	\n"
		" mov	r1,lr	\n"
		" ldr	r2,=HardFault_Handler_C \n"
		" bx	r2"

		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) HardFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr  ;
	#endif

	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Exceptions specific to ARM ARCH 7M /  7EM

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

void __attribute__ ((section(".isr_handlers"),weak)) MemManage_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) DebugMon_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak,naked)) BusFault_Handler (void)
{
	__ASM volatile
	(
		" tst		lr,#4	\n"
		" ite		eq		\n"
		" mrseq		r0,msp	\n"
		" mrsne		r0,psp	\n"
		" mov		r1,lr	\n"
		" ldr		r2,=BusFault_Handler_C \n"
		" bx		r2"
		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) BusFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr  ;
	#endif

	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak,naked)) UsageFault_Handler (void)
{
	__ASM volatile
	(
		" tst		lr,#4	\n"
		" ite		eq		\n"
		" mrseq		r0,msp	\n"
		" mrsne		r0,psp	\n"
		" mov		r1,lr	\n"
		" ldr		r2,=UsageFault_Handler_C \n"
		" bx		r2"
		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) UsageFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t 				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr ;
	#endif

	__DEBUG_BKPT () ;
}

#endif	// defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

