/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	adcbasic.c	Some routine to manage basic ADC

	When		Who	What
	25/05/18	ac	Creation

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"
#include	"gpiobasic.h"
#include	"dmabasic.h"

#include	"stm32f1xx_ll_bus.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------


#if ! defined ADCBASIC_H_
#define	ADCBASIC_H_
//----------------------------------------------------------------------
#include	"stm32f1xx_ll_adc.h"

// adcInitAdc() initializes all channels sampling time to this value
// The user can later set the channels sampling time with adcInitSamplingTime()
#define	ADC_DEF_SAMPLINGTIME	LL_ADC_SAMPLINGTIME_480CYCLES

// Theses flags are memorized in adcCommonConfig
#define	ADC_ADC1			0x00000001u
#define	ADC_ADC2			0x00000002u
#define	ADC_DUAL			0x00000008u
#define	ADC_INTERLEAVED		0x00000020u
#if defined (ADC3)
	#define	ADC_ADC3		0x00000004u
	#define	ADC_TRIPLE		0x00000010u
#endif	// ADC3

#define	ADC_PRES2			0x00u
#define	ADC_PRES4			0x01u
#define	ADC_PRES6			0x10u
#define	ADC_PRES8			0x11u

// For adcInitAdc()
#define	ADC_ALIGNRIGHT		0x00000000u
#define	ADC_ALIGNLEFT		0x00000100u
#define	ADC_RESO12BITS		0x00000000u
#define	ADC_RESO8BITS		0x00000200u
#define	ADC_SCAN			0x00000400u
#define	ADC_INT_EOC			0x00001000u		// interrupt on end of sequence conversions
#define	ADC_INT_EOCS		0x00002000u		// interrupt on end of unitary conversion
#define	ADC_INT_OVR			0x00004000u		// interrupt on overrun

// For adcInitSequencer()
#define	ADC_DISCONT			0x00010000u
#define	ADC_ONESHOT			0x00020000u

// For adcDmaInit()
#define	ADC_DMA_SINGLE		0x00010000u		// No new DMA request after the last transfer
#define	ADC_DMA_CIRCULAR	0x00020000u		// DMA circular
#define	ADC_DMA_DOUBLE		0x00040000u		// DMA double buffer
#define	ADC_DMA_NOIT		0x00080000u		// No DMA interruptions

typedef enum
{
	ADC_ENONE		= 0,
	ADC_EFAIL		= 1,
	ADC_ENOTINIT	= 2,
	ADC_EARG		= 3,
	ADC_ESTATE		= 4,

} adcStatus_t ;

#define	ADC_INT_PRIORITY			(BSP_MAX_INT_PRIO+2)		//	ADCs interruptions priority

#define	ADC1_DMA_INT_PRIORITY		(BSP_MAX_INT_PRIO+2)		//	ADC1 DMA interruptions priority
#define	ADC2_DMA_INT_PRIORITY		(BSP_MAX_INT_PRIO+2)		//	ADC2 DMA interruptions priority
#if defined (ADC3)
	#define	ADC3_DMA_INT_PRIORITY	(BSP_MAX_INT_PRIO+2)		//	ADC3 DMA interruptions priority
#endif	// ADC3

// Forward declaration for opaque type
typedef	struct adcDesc_t adcDesc_t;

// Callback type for ADC interruptions handler
typedef	void (* adcCallback_t) (adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg) ;

#define	ADC_EVENT_EOC		1		// End of conversion
#define	ADC_EVENT_OVR		2		// DMA overrun

// A structure to describe channels sampling time for adcInitSamplingTime()
typedef struct
{
	uint8_t	channel ;	// Channel number: 0..15
	uint8_t	time; 		// Sampling time: LL_ADC_SAMPLINGTIME_xCYCLES

} adcSamplingTime_t ;

//----------------------------------------------------------------------
#ifdef __cplusplus
extern "C" {
#endif

adcStatus_t		adcInit				(uint32_t flags, uint32_t prescaler, uint32_t interDelay) ;
void			adcInitChannels		(uint32_t flags, uint8_t * pChannel, uint32_t count) ;
adcStatus_t		adcInitAdc			(uint32_t flags, adcDesc_t ** ppAdcDesc) ;
adcStatus_t		adcInitSequencer	(adcDesc_t * pAdcDesc, uint32_t flags, const uint8_t * pChannels, uint32_t count) ;
adcStatus_t		adcInitSamplingTime	(adcDesc_t * pAdcDesc, const adcSamplingTime_t * pSamplingTime, uint32_t count) ;
adcStatus_t		adcDmaInit			(adcDesc_t * pAdcDesc, uint32_t flags, uint32_t length) ;
adcStatus_t 	adcDmaBuffer		(adcDesc_t * pAdcDesc, const void * buffer1, const void * buffer2) ;
adcStatus_t		adcSetTrigger		(adcDesc_t * pAdcDesc, uint32_t triggerSource, uint32_t edge) ;
adcStatus_t 	adcStop				(adcDesc_t * pAdcDesc) ;
adcStatus_t 	adcStart			(adcDesc_t * pAdcDesc) ;
void 			adcSwTrigger		(adcDesc_t * pAdcDesc) ;
adcStatus_t		adcSetCallback		(adcDesc_t * pAdcDesc, adcCallback_t cb, uintptr_t cbArg) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// ADCBASIC_H_


//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	DMA descriptor for each ADC

const dmaDesc_t		adc1DmaDesc =
{
	2,										//	DMA number 1 or 2, 0 => unused descriptor
	DMA_TOSTREAMCHAN (0, 0),				//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
	LL_DMA_DIRECTION_PERIPH_TO_MEMORY,		//	Direction mandatory LL_DMA_DIRECTION_PERIPH_TO_MEMORY
	ADC1_DMA_INT_PRIORITY,					//	interruptions priority
} ;

const dmaDesc_t		adc2DmaDesc =
{
	0, // 2,										//	DMA number 1 or 2, 0 => unused descriptor
	DMA_TOSTREAMCHAN (2, 1),				//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
	LL_DMA_DIRECTION_PERIPH_TO_MEMORY,		//	Direction mandatory LL_DMA_DIRECTION_PERIPH_TO_MEMORY
	ADC2_DMA_INT_PRIORITY,					//	interruptions priority
} ;

#if defined (ADC3)

	const dmaDesc_t		adc3DmaDesc =
	{
		2,									//	DMA number 1 or 2, 0 => unused descriptor
		DMA_TOSTREAMCHAN (1, 2),			//	DMA_TOSTREAMCHAN (stream number 0..7, channel number 0..7)
		LL_DMA_DIRECTION_PERIPH_TO_MEMORY,	//	Direction mandatory LL_DMA_DIRECTION_PERIPH_TO_MEMORY
		ADC3_DMA_INT_PRIORITY,				//	interruptions priority
	} ;

#endif	// ADC3

//----------------------------------------------------------------------
//	One descriptor for each ADC

typedef	struct adcDesc_t
{
	ADC_TypeDef		* pAdc ;	// ADC registers address
	dmaDesc_t		dmaDesc ;	// DMA descriptor
	uint32_t		flags ;		// Configuration flags
	uint32_t		triggerSource ;
	adcCallback_t	cb ;		// interruptions handler callback
	uintptr_t		cbArg ;		// interruptions handler callback user argument

} adcDesc_t ;

adcDesc_t		adc1Desc ;
adcDesc_t		adc2Desc ;

#if defined (ADC3)
	adcDesc_t	adc3Desc ;
#endif	// ADC3

//----------------------------------------------------------------------
//	GPIO channel definition
//	No alternate pins for analog input

typedef struct
{
	uint8_t		port ;	// 'A', 'B', etc. or GPIO_PORTA, etc.
	uint8_t		pin ;	// 0 to 15

} adcPin_t ;

const adcPin_t	adc12PinList [16] =		// 1 and 2 have same pins
{
	{	'A',	0	},	// Chan 0
	{	'A',	1	},	// Chan 1
	{	'A',	2	},	// Chan 2
	{	'A',	3	},	// Chan 3
	{	'A',	4	},	// Chan 4
	{	'A',	5	},	// Chan 5
	{	'A',	6	},	// Chan 6
	{	'A',	7	},	// Chan 7
	{	'B',	0	},	// Chan 8
	{	'B',	1	},	// Chan 9
	{	'C',	0	},	// Chan 10
	{	'C',	1	},	// Chan 11
	{	'C',	2	},	// Chan 12
	{	'C',	3	},	// Chan 13
	{	'C',	4	},	// Chan 14
	{	'C',	5	},	// Chan 15
} ;

#if defined (ADC3)
const adcPin_t	adc3PinList [16] =
{
	{	'A',	0	},	// Chan 0
	{	'A',	1	},	// Chan 1
	{	'A',	2	},	// Chan 2
	{	'A',	3	},	// Chan 3
	{	'F',	6	},	// Chan 4
	{	'F',	7	},	// Chan 5
	{	'F',	8	},	// Chan 6
	{	'F',	9	},	// Chan 7
	{	'F',	10	},	// Chan 8
	{	'F',	3	},	// Chan 9
	{	'C',	0	},	// Chan 10
	{	'C',	1	},	// Chan 11
	{	'C',	2	},	// Chan 12
	{	'C',	3	},	// Chan 13
	{	'F',	4	},	// Chan 14
	{	'F',	5	},	// Chan 15
} ;
#endif	// ADC3

static	uint32_t	adcCommonConfig ;		// Common configuration

static	uint32_t	regRank [16] =
{
	LL_ADC_REG_RANK_1,
	LL_ADC_REG_RANK_2,
	LL_ADC_REG_RANK_3,
	LL_ADC_REG_RANK_4,
	LL_ADC_REG_RANK_5,
	LL_ADC_REG_RANK_6,
	LL_ADC_REG_RANK_7,
	LL_ADC_REG_RANK_8,
	LL_ADC_REG_RANK_9,
	LL_ADC_REG_RANK_10,
	LL_ADC_REG_RANK_11,
	LL_ADC_REG_RANK_12,
	LL_ADC_REG_RANK_13,
	LL_ADC_REG_RANK_14,
	LL_ADC_REG_RANK_15,
	LL_ADC_REG_RANK_16,
} ;

//----------------------------------------------------------------------
//	Default callback for ADC interruptions handler

void adcDefaultCallback (adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg)
{
	(void) pAdcDesc ;
	(void) event ;
	(void) arg ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// Initialize ADCs and some ADC common parameters
// interDelay is used only for dual or triple interleaved mode: 5 to 20

adcStatus_t		adcInit		(uint32_t flags, uint32_t prescaler, uint32_t interDelay)
{
	adcCommonConfig = flags ;		// Memorize configuration

	if (flags & ADC_TRIPLE)
	{
		adcCommonConfig |= ADC_ADC1 | ADC_ADC2 ;
		#if defined (ADC3)
			adcCommonConfig |= ADC_ADC3 ;
		#endif	// ADC3
	}
	if (flags & ADC_DUAL)
	{
		adcCommonConfig |= ADC_ADC1 | ADC_ADC2 ;
	}

	// Check parameters
#if defined (ADC3)
	if ((adcCommonConfig & (ADC_ADC1 | ADC_ADC2 | ADC_ADC3)) == 0)
#else
	if ((adcCommonConfig & (ADC_ADC1 | ADC_ADC2)) == 0)
#endif	// ADC3
	{
		adcCommonConfig = 0 ;
		AA_ASSERT (0) ;
		return ADC_EARG ;	// No ADC specified
	}

	if (prescaler > 3)
	{
		adcCommonConfig = 0 ;
		AA_ASSERT (0) ;
		return ADC_EARG ;	// Bad prescaler
	}

	if ((adcCommonConfig & ADC_INTERLEAVED) != 0  && (interDelay < 5  ||  interDelay > 20))
	{
		adcCommonConfig = 0 ;
		AA_ASSERT (0) ;
		return ADC_EARG ;	// Bas interDelay
	}

	// Initialize and reset ADCs
	if (adcCommonConfig & ADC_ADC1)
	{
		rccEnableClock (ADC1, RCC_APB2ENR_ADC1EN) ;
		rccResetPeriph (ADC1, RCC_APB2ENR_ADC1EN) ;
	}

	if (adcCommonConfig & ADC_ADC2)
	{
		rccEnableClock (ADC2, RCC_APB2ENR_ADC2EN) ;
		rccResetPeriph (ADC2, RCC_APB2ENR_ADC2EN) ;
	}

#if defined (ADC3)
	if (adcCommonConfig & ADC_ADC3)
	{
		rccEnableClock (ADC3, RCC_APB2ENR_ADC3EN) ;
		rccResetPeriph (ADC3, RCC_APB2ENR_ADC3EN) ;
	}
#endif	// ADC3

	// Set CCR MULTI. Default value is independent ADCs
	if (flags & ADC_DUAL)
	{
		// Dual mode for ADC1 and ADC2
		__LL_ADC_COMMON_INSTANCE (ADC1)->CCR |= LL_ADC_MULTI_DUAL_REG_SIMULT ;		// Dual: Regular simultaneous mode only
	}
#if defined (ADC3)
	else if (flags & ADC_TRIPLE)
	{
		// Triple mode
		__LL_ADC_COMMON_INSTANCE (ADC1)->CCR |= LL_ADC_MULTI_TRIPLE_REG_SIMULT ;	// Triple: Regular simultaneous mode only
	}
#endif	// ADC3
	else
	{
		// Only independent ADC. Nothing to do
	}

	// Set common clock
	LL_ADC_SetCommonClock (__LL_ADC_COMMON_INSTANCE (ADC1), prescaler << ADC_CCR_ADCPRE_Pos) ;

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	Initialize channels pins for analog use
//	Flags is ADC_ADC1 or ADC_ADC2 or ADC_ADC3
//	pChannel is a pointer to an array of values 0 to 15
//	count is the count of channels in pChannels
//	WARNING: ADC3 does not have the same pins as ADC1 and ADC2

void	adcInitChannels		(uint32_t flags, uint8_t * pChannel, uint32_t count)
{
	const adcPin_t	* pAdcPin ;
	gpioPinDesc_t	pinDesc ;
	uint32_t		ii ;

	if (flags == ADC_ADC1  ||  flags == ADC_ADC2)
	{
		pAdcPin = adc12PinList;
	}
#if defined (ADC3)
	else
	{
		pAdcPin = adc3PinList;
	}
#endif	// ADC3
	pinDesc.af    = LL_GPIO_AF_0 ;
	pinDesc.flags = GPIO_MODE_ANALOG ;

	for (ii = 0 ; ii < count ; ii++)
	{
		pinDesc.port =  pAdcPin [pChannel[ii]].port ;
		pinDesc.pin  =  pAdcPin [pChannel[ii]].pin ;
		gpioConfigurePin (& pinDesc) ;
	}
//LL_AHB1_GRP1_EnableClock(LL_AHB1_GRP1_PERIPH_GPIOA);
//LL_GPIO_SetPinMode(GPIOA, LL_GPIO_PIN_3, LL_GPIO_MODE_ANALOG);
}

//----------------------------------------------------------------------
//	Flags is one of ADC_ADC1, ADC_ADC2, ADC_ADC3
//	and: ADC_INT_EOC or ADC_INT_EOCS, ADC_INT_OVR

adcStatus_t	adcInitAdc		(uint32_t flags, adcDesc_t ** ppAdcDesc)
{
	ADC_TypeDef		* pAdc ;
	adcDesc_t 		* pAdcDesc ;
	uint32_t		reg ;

	if ((flags & ADC_ADC1) != 0)
	{
		pAdcDesc = & adc1Desc ;
		pAdcDesc->pAdc		= ADC1 ;
		pAdcDesc->dmaDesc	= adc1DmaDesc ;
	}
	else if ((flags & ADC_ADC2) != 0)
	{
		pAdcDesc = & adc2Desc ;
		pAdcDesc->pAdc		= ADC1 ;
		pAdcDesc->dmaDesc	= adc2DmaDesc ;
	}
#if defined (ADC3)
	else if ((flags & ADC_ADC3) != 0)
	{
		pAdcDesc = & adc3Desc ;
		pAdcDesc->pAdc		= ADC3 ;
		pAdcDesc->dmaDesc	= adc3DmaDesc ;
	}
#endif	// ADC3
	else
	{
		AA_ASSERT (0);
		return ADC_EARG ;
	}
	pAdcDesc->flags	= 0 ;
	pAdcDesc->triggerSource = 0 ;
	pAdcDesc->cb 	= adcDefaultCallback ;
	pAdcDesc->cbArg	= 0 ;

	pAdc = pAdcDesc->pAdc ;
	pAdc->CR1 &= ~0x07C0FFFFu ;		// Clear all bits of CR1
	pAdc->CR2 &= ~0x7F7F0F03u ;		// Clear all bits of CR2

	reg = pAdc->CR1 & ~(ADC_CR1_RES | LL_ADC_SEQ_SCAN_ENABLE) ;
	// Default resolution is 12 bits
	if ((flags & ADC_RESO8BITS) != 0)
	{
		reg |= LL_ADC_RESOLUTION_8B ;
	}
	pAdc->CR1 = reg ;

	reg = pAdc->CR2 & ~(ADC_CR2_ALIGN) ;
	// Default alignment is right
	if ((flags & ADC_ALIGNLEFT) != 0)
	{
		reg |= LL_ADC_DATA_ALIGN_LEFT ;
		pAdc->CR2 = reg ;
	}

	// Initialize interruptions if requested
	if (flags & (ADC_INT_EOC | ADC_INT_EOCS | ADC_INT_OVR))
	{
		// Some interrupt is requested
		NVIC_SetPriority (ADC_IRQn, ADC_INT_PRIORITY) ;
		NVIC_EnableIRQ   (ADC_IRQn) ;

		// End of sequence or end of conversion interrupt requested ?
		if (flags & (ADC_INT_EOC | ADC_INT_EOCS))
		{
			LL_ADC_EnableIT_EOCS (pAdc) ;
			if (flags & ADC_INT_EOCS)
			{
				// Interrupt at end of every channel conversion
				LL_ADC_REG_SetFlagEndOfConversion (pAdc, LL_ADC_REG_FLAG_EOC_UNITARY_CONV) ;
			}
		}

		// Overrun interrupt requested ?
		if (flags & ADC_INT_OVR)
		{
			LL_ADC_EnableIT_OVR (pAdc) ;
		}
	}

	// Initialize channels sampling time
	for (reg = 0 ; reg < 19 ; reg++)
	{
		LL_ADC_SetChannelSamplingTime (	pAdc,
										__LL_ADC_DECIMAL_NB_TO_CHANNEL (reg),
										ADC_DEF_SAMPLINGTIME) ;
	}

	* ppAdcDesc = pAdcDesc ;
	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	Flags: [ADC_DISCONT + count] | ADC_ONESHOT
//	pChannels: pointer to array of channel number 0..16, this is the conversion sequence
//	if count > 1 then scan mode is enabled

//	  TODO internal channels

adcStatus_t	adcInitSequencer	(adcDesc_t * pAdcDesc, uint32_t flags, const uint8_t * pChannels, uint32_t count)
{
	ADC_TypeDef		* pAdc = pAdcDesc->pAdc ;
	uint32_t		reg ;
	uint32_t		ii ;

	if (count < 1 || count > 16)
	{
		AA_ASSERT (0) ;
		return ADC_EARG ;
	}

	// Set scan and discontinuous parameters
	reg = pAdc->CR1 & ~(LL_ADC_SEQ_SCAN_ENABLE | LL_ADC_REG_SEQ_DISCONT_8RANKS) ;
	// Default is no scan (conversion of 1 channel)
	if (count > 1)
	{
		reg |= LL_ADC_SEQ_SCAN_ENABLE ;

		// Default is no discontinuous
		// Discontinuous valid only if SCAN is enabled
		if ((flags & ADC_DISCONT) != 0)
		{
			reg |= ADC_CR1_DISCEN ;
			reg |= ((flags & 0x0Fu) - 1) << ADC_CR1_DISCNUM_Pos ;
		}
	}
	pAdc->CR1 = reg ;

	// Set continuous/one shot parameter
	reg = pAdc->CR2 & ~(ADC_CR2_CONT) ;
	// Default is single conversion (one shot)
	if ((flags & ADC_ONESHOT) == 0)
	{
		reg |= ADC_CR2_CONT ;
	}
	pAdc->CR2 = reg ;

	// Set channels sequence
	for (ii = 0 ; ii < count ; ii++)
	{
		LL_ADC_REG_SetSequencerRanks (	pAdc,
										regRank [ii],
										__LL_ADC_DECIMAL_NB_TO_CHANNEL (pChannels [ii])) ;
	}

	// Set scan length
	LL_ADC_REG_SetSequencerLength (pAdc, (count-1) << ADC_SQR1_L_Pos);

	return ADC_ENONE ;
}

//----------------------------------------------------------------------

adcStatus_t		adcInitSamplingTime	(adcDesc_t * pAdcDesc, const adcSamplingTime_t * pSamplingTime, uint32_t count)
{
	uint32_t		ii ;

	for (ii = 0 ; ii < count ; ii++)
	{

		LL_ADC_SetChannelSamplingTime (	pAdcDesc->pAdc,
										__LL_ADC_DECIMAL_NB_TO_CHANNEL (pSamplingTime[ii].channel),
										pSamplingTime [ii].time) ;
	}

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	flags is one of: ADC_DMASINGLE ADC_DMACIRCULAR ADC_DMADOUBLE

adcStatus_t	adcDmaInit	(adcDesc_t * pAdcDesc, uint32_t flags, uint32_t length)
{
	ADC_TypeDef			* pAdc = pAdcDesc->pAdc;
	const dmaDesc_t		* pDmaDesc = & pAdcDesc->dmaDesc ;
	DMA_TypeDef			* pDma ;
	uint32_t			streamNum ;
	uint32_t			reg ;

	pAdcDesc->flags = flags ;			// Memorize DMA configuration for this ADC

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Set DMA transfer according to the common configuration
	if ((pAdcDesc == & adc1Desc)  &&  (adcCommonConfig & ADC_DUAL) != 0)
	{
		//	DUAL => DMA mode 2
		reg = ((flags & ADC_DMA_SINGLE) != 0 ) ? LL_ADC_MULTI_REG_DMA_LIMIT_2 : LL_ADC_MULTI_REG_DMA_UNLMT_2 ;
		LL_ADC_SetMultiDMATransfer (__LL_ADC_COMMON_INSTANCE (pAdc), reg) ;
		reg = LL_DMA_PDATAALIGN_WORD ;
	}
#if defined (ADC3)
	else if ((pAdcDesc == & adc1Desc)  &&  (adcCommonConfig & ADC_TRIPLE) != 0)
	{
		//	TRIPLE => DMA mode 1
		reg = ((flags & ADC_DMA_SINGLE) != 0 ) ? LL_ADC_MULTI_REG_DMA_LIMIT_1 : LL_ADC_MULTI_REG_DMA_UNLMT_1 ;
		LL_ADC_SetMultiDMATransfer (__LL_ADC_COMMON_INSTANCE (pAdc), reg) ;
		reg = LL_DMA_PDATAALIGN_HALFWORD ;
	}
#endif	// ADC3
	else
	{
		// Set DMA for independent ADC
		LL_ADC_REG_SetDMATransfer (pAdc, ((flags & ADC_DMA_SINGLE) != 0 ) ? LL_ADC_REG_DMA_TRANSFER_LIMITED : LL_ADC_REG_DMA_TRANSFER_UNLIMITED) ;
		reg = LL_DMA_PDATAALIGN_HALFWORD ;
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Set DMA configuration
	pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
	streamNum	= DMA_GETSTREAM (pDmaDesc) ;

	// Enable DMA clock
	rccEnableClock (pDma, 1 << ((pDma == DMA1) ? RCC_AHB1ENR_DMA1EN_Pos : RCC_AHB1ENR_DMA2EN_Pos)) ;

	// Reset stream
	dmaStreamAddress (pDma, streamNum)->CR = 0 ;

	LL_DMA_SetChannelSelection (pDma, streamNum, DMA_GETCHANNEL (pDmaDesc) << 25);

	LL_DMA_ConfigTransfer (pDma, streamNum,
							LL_DMA_DIRECTION_PERIPH_TO_MEMORY |
							(((flags & ADC_DMA_CIRCULAR) != 0) ? LL_DMA_MODE_CIRCULAR : LL_DMA_MODE_NORMAL) |
							LL_DMA_PERIPH_NOINCREMENT         |
							LL_DMA_MEMORY_INCREMENT           |
							reg								  |		// LL_DMA_PDATAALIGN_xxx
							LL_DMA_MDATAALIGN_HALFWORD        |		// direct mode so: MSIZE is PSIZE
							LL_DMA_PRIORITY_HIGH               );

	// Set double buffer if required
	if ((flags & ADC_DMA_DOUBLE) != 0)
	{
		LL_DMA_EnableDoubleBufferMode (pDma, streamNum) ;
	}

	// Set DMA buffer size
	LL_DMA_SetDataLength (pDma, streamNum, length) ;

	// Configure DMA interruptions
	if ((flags & ADC_DMA_NOIT) == 0)
	{
		//	Configure NVIC for DMA transfer complete/error interruptions
		dmaSetIrq (pDmaDesc) ;

		// Enable DMA transfer interrupt and transfer error
		LL_DMA_EnableIT_TE (pDma, streamNum) ;
		LL_DMA_EnableIT_HT (pDma, streamNum) ;
		LL_DMA_EnableIT_TC (pDma, streamNum) ;
	}
	else
	{
		// Disable DMA interruptions
		LL_DMA_DisableIT_TE (pDma, streamNum) ;
		LL_DMA_DisableIT_HT (pDma, streamNum) ;
		LL_DMA_DisableIT_TC (pDma, streamNum) ;
	}
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	Set DMA buffers. To be called before starting ADC
//	If ADC_DMA_CIRCULAR or ADC_DMA_ONESHOT only buffer 1 is used
//	If ADC_DMA_DOUBLE and called before starting AD: buffer 1 and buffer 2 are used
//	If ADC_DMA_DOUBLE and called after starting AD: only buffer 1 is used

adcStatus_t 	adcDmaBuffer	(adcDesc_t * pAdcDesc, const void * buffer1, const void * buffer2)
{
	ADC_TypeDef			* pAdc = pAdcDesc->pAdc ;
	const dmaDesc_t 	* pDmaDesc = & pAdcDesc->dmaDesc ;
	DMA_TypeDef			* pDma ;
	uint32_t			streamNum ;
	volatile uint32_t	* pDataReg ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	if ((pAdcDesc->pAdc == ADC1)  &&  (adcCommonConfig & ADC_DUAL) != 0)
	{
		// If DUAL use common data register
		pDataReg = & __LL_ADC_COMMON_INSTANCE(pAdc)->CDR ;
	}
#if defined (ADC3)
	else if ((pAdcDesc->pAdc == ADC1)  &&  (adcCommonConfig & ADC_TRIPLE) != 0)
	{
		// If TRIPLE use common data register
		pDataReg = & __LL_ADC_COMMON_INSTANCE(pAdc)->CDR ;
	}
#endif	// ADC3
	else
	{
		// Use ADC data register
		pDataReg = & pAdc->DR ;
	}

	if (pDmaDesc->dmaNum == DMA_NONE)
	{
		AA_ASSERT (0) ;
		return ADC_ESTATE ;
	}
	pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
	streamNum	= DMA_GETSTREAM (pDmaDesc) ;

	// Set DMA addresses
	if ((LL_ADC_IsEnabled (pAdc)) == 0)
	{
		// ADC not enabled: initialize DMA buffers
		LL_DMA_SetMemoryAddress (pDma, streamNum, (uint32_t) buffer1) ;
		LL_DMA_SetPeriphAddress (pDma, streamNum, (uint32_t) pDataReg) ;

		if ((pAdcDesc->flags & ADC_DMA_DOUBLE) != 0)
		{
			// Also set second buffer address
			LL_DMA_SetMemory1Address (pDma, streamNum, (uint32_t) buffer2) ;
		}
	}
	else
	{
		// ADC enabled : update memory DMA buffer not in use (double buffer only)
		if (LL_DMA_GetCurrentTargetMem (pDma, streamNum) == 0)
		{
			LL_DMA_SetMemory1Address (pDma, streamNum, (uint32_t) buffer1) ;
		}
		else
		{
			LL_DMA_SetMemoryAddress (pDma, streamNum, (uint32_t) buffer1) ;
		}
	}

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	triggerSource: one of LL_ADC_REG_TRIG_xxx
//	edge: one of LL_ADC_REG_TRIG_EXT_RISING, LL_ADC_REG_TRIG_EXT_FALLING, LL_ADC_REG_TRIG_EXT_RISINGFALLING
//	edge: ignored if triggerSource is LL_ADC_REG_TRIG_SOFTWARE


adcStatus_t	adcSetTrigger	(adcDesc_t * pAdcDesc, uint32_t triggerSource, uint32_t edge)
{
	uint32_t	reg = triggerSource ;

	if (triggerSource != LL_ADC_REG_TRIG_SOFTWARE)
	{
		// build ADC_CR2 EXTEN:EXTSEL fields
		reg = (triggerSource & ~LL_ADC_REG_TRIG_EXT_RISINGFALLING) | edge ;
	}
	pAdcDesc->triggerSource = reg ;

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	Start ADC conversion

adcStatus_t adcStart		(adcDesc_t * pAdcDesc)
{
	ADC_TypeDef			* pAdc = pAdcDesc->pAdc ;
	const dmaDesc_t 	* pDmaDesc = & pAdcDesc->dmaDesc ;
	DMA_TypeDef			* pDma ;
	uint32_t			streamNum ;

	// Enable the DMA transfer
	if (pDmaDesc->dmaNum != DMA_NONE)
	{
		pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
		streamNum	= DMA_GETSTREAM (pDmaDesc) ;
		LL_DMA_EnableStream (pDma, streamNum) ;
	}

	// Enable ADC channel
	LL_ADC_Enable (pAdc) ;

	// Enable non software trigger
	if (pAdcDesc->triggerSource != LL_ADC_REG_TRIG_SOFTWARE)
	{
		uint32_t	reg ;
		reg = pAdc->CR2 & ~(ADC_CR2_EXTEN | ADC_CR2_EXTSEL) ;
		pAdc->CR2 = reg |pAdcDesc->triggerSource ;
	}

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	Stop ADC conversion

adcStatus_t adcStop		(adcDesc_t * pAdcDesc)
{
	ADC_TypeDef			* pAdc = pAdcDesc->pAdc ;
	const dmaDesc_t 	* pDmaDesc = & pAdcDesc->dmaDesc ;

	LL_ADC_REG_StopConversionExtTrig (pAdc) ;

	LL_ADC_Disable (pAdc) ;

	if(pDmaDesc->dmaNum != DMA_NONE)
	{
		DMA_TypeDef			* pDma ;
		uint32_t			streamNum ;

		pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
		streamNum	= DMA_GETSTREAM (pDmaDesc) ;
		LL_DMA_DisableStream (pDma, streamNum) ;
	}

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
// Trigger ADC conversion by software

void 	adcSwTrigger	(adcDesc_t * pAdcDesc)
{
	LL_ADC_REG_StartConversionSWStart (pAdcDesc->pAdc) ;
}

//----------------------------------------------------------------------
//	ADC interruptions callback
//	Use dmaSetCallback() for DMA callback

adcStatus_t	adcSetCallback	(adcDesc_t * pAdcDesc, adcCallback_t cb, uintptr_t cbArg)
{
	pAdcDesc->cb = cb ;
	pAdcDesc->cbArg = cbArg ;

	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	Allows to wait for end of unitary conversion or sequence
//	Returns ADC_EFAIL if not ended
//	Returns ADC_ENONE if ended. EOCS flag is cleared

adcStatus_t	adcWaitForEoc	(adcDesc_t * pAdcDesc)
{
	if (LL_ADC_IsActiveFlag_EOCS (pAdcDesc->pAdc) == 0)
	{
		return ADC_EFAIL;
	}
	LL_ADC_ClearFlag_EOCS (pAdcDesc->pAdc) ;
	return ADC_ENONE ;
}

//----------------------------------------------------------------------
//	Allows to read unitary conversion data
//	Independent ADC only
// TODO: read DUAL or TRIPLE ?

void	adcReadUnitary	(adcDesc_t * pAdcDesc, uint16_t * pData)
{
	* pData = LL_ADC_REG_ReadConversionData12 (pAdcDesc->pAdc) ;
}

//----------------------------------------------------------------------
//	One IRQ common to all ADCs : ADC_IRQn

void	ADC_IRQHandler	()
{
//	ADC_TypeDef		* pAdc = ADC1 ;

	// Check for ADC1 interruptions	// **********************************************
	if (LL_ADC_IsActiveFlag_EOCS (ADC1) != 0)
	{
		// Clear flag ADC group regular end of unitary conversion
		LL_ADC_ClearFlag_EOCS (ADC1) ;

		// Call interrupt treatment function
		(* adc1Desc.cb) (& adc1Desc, ADC_EVENT_EOC, adc1Desc.cbArg) ;
	}

	// Check whether ADC group regular overrun caused the ADC interrupt
	if (LL_ADC_IsActiveFlag_OVR (ADC1) != 0)
	{
		// Clear flag ADC group regular overrun
		LL_ADC_ClearFlag_OVR (ADC1) ;

		// Call interrupt treatment function
		(* adc1Desc.cb) (& adc1Desc, ADC_EVENT_OVR, adc1Desc.cbArg) ;
	}

	// Check for ADC2 interruptions	// **********************************************
	if (LL_ADC_IsActiveFlag_EOCS (ADC2) != 0)
	{
		// Clear flag ADC group regular end of unitary conversion
		LL_ADC_ClearFlag_EOCS (ADC2) ;

		// Call interrupt treatment function
		(* adc2Desc.cb) (& adc2Desc, ADC_EVENT_EOC, adc2Desc.cbArg) ;
	}

	// Check whether ADC group regular overrun caused the ADC interrupt
	if (LL_ADC_IsActiveFlag_OVR (ADC2) != 0)
	{
		// Clear flag ADC group regular overrun
		LL_ADC_ClearFlag_OVR (ADC2) ;

		// Call interrupt treatment function
		(* adc2Desc.cb) (& adc2Desc, ADC_EVENT_OVR, adc2Desc.cbArg) ;
	}


#if defined (ADC3)
	// Check for ADC3 interruptions	// **********************************************
	if (LL_ADC_IsActiveFlag_EOCS (ADC3) != 0)
	{
		// Clear flag ADC group regular end of unitary conversion
		LL_ADC_ClearFlag_EOCS (ADC3) ;

		// Call interrupt treatment function
		(* adc3Desc.cb) (& adc3Desc, ADC_EVENT_EOC, adc3Desc.cbArg) ;
	}

	// Check whether ADC group regular overrun caused the ADC interrupt
	if (LL_ADC_IsActiveFlag_OVR (ADC3) != 0)
	{
		// Clear flag ADC group regular overrun
		LL_ADC_ClearFlag_OVR (ADC3) ;

		// Call interrupt treatment function
		(* adc3Desc.cb) (& adc3Desc, ADC_EVENT_OVR, adc3Desc.cbArg) ;
	}
#endif	// ADC3
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	ADC tests

#include	"aaprintf.h"
#include	"aalogmes.h"
#include	"timbasic.h"

adcDesc_t	* pAdcDesc ;

#define		TADC_CHANNEL_COUNT	1
uint8_t		adcChannels [TADC_CHANNEL_COUNT] = { 3 } ;	// PA3
//uint8_t	adcSampling [TADC_CHANNEL_COUNT] = { 7 } ;	// ADC cycles count: 7=>480

#define	ADCX		ADC_ADC1

//----------------------------------------------------------------------
//	Unitary conversion on software trigger

void	adcTest1 ()
{
	uint16_t	data16 ;

	// Configure 1 channel
	adcInit				(ADCX, ADC_PRES2, 0) ;						// ADC common parameters
	adcInitChannels 	(ADCX, adcChannels, TADC_CHANNEL_COUNT) ;	// Configure analog pins
//	adcInitAdc 			(ADCX | ADC_INT_EOCS, & pAdcDesc) ;	// Initialize ADC
	adcInitAdc 			(ADCX, & pAdcDesc) ;	// Initialize ADC
	adcInitSequencer	(pAdcDesc, ADC_ONESHOT, adcChannels, TADC_CHANNEL_COUNT) ;
	adcSetTrigger 		(pAdcDesc, LL_ADC_REG_TRIG_SOFTWARE, 0) ;
	adcStart			(pAdcDesc) ;

	while (1)
	{
		adcSwTrigger (pAdcDesc) ;
		while (adcWaitForEoc (pAdcDesc) != ADC_ENONE)
		{
		}
		adcReadUnitary	(pAdcDesc, & data16) ;
		aaPrintf ("ADC: %04X\n", data16) ;

		aaTaskDelay (100) ;
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Sequence conversion on software trigger

#define	TMAX_CONV	8

uint8_t	adcSequence [TMAX_CONV] = { 3, 3, 3, 3,  3, 3, 3, 3} ;

//----------------------------------------------------------------------
//	ADC interrupt callback
void adcTest2Callback (adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg)
{
	aaLogMes ("adcTest2Callback %d\n", event, 0, 0, 0, 0) ;
	if (event == ADC_EVENT_EOC)
	{
		aaSignalSend ((aaTaskId_t) arg, 1) ;
	}

	(void) pAdcDesc ;
}

//----------------------------------------------------------------------
//	DMA interrupt callback

void	adcDmaCallback (uint32_t event, uint32_t dir, uintptr_t arg)
{
//	aaLogMes ("adcDmaCallback %d\n", event, 0, 0, 0, 0) ;
	if (event == DMA_EV_COMPLETE)
	{
		aaSignalSend ((aaTaskId_t) arg, 1) ;
	}

	(void) dir ;
}


//----------------------------------------------------------------------

void	adcTest2 ()
{
	uint16_t	data16 [TMAX_CONV] ;

	aaLogMes ("ADC Test 2\n", 0, 0, 0, 0, 0) ;

	// Configure ADC with SCAN and software trigger
	adcInit				(ADCX, ADC_PRES2, 0) ;							// ADC common parameters
	adcInitChannels 	(ADCX, adcChannels, TADC_CHANNEL_COUNT) ;		// Configure analog pins
//	adcInitAdc 			(ADCX | ADC_SCAN | ADC_INT_EOC, & pAdcDesc) ;	// Initialize ADC
//	adcInitAdc 			(ADCX | ADC_SCAN | ADC_RESO8BITS, & pAdcDesc) ;					// Initialize ADC
	adcInitAdc 			(ADCX | ADC_SCAN, & pAdcDesc) ;					// Initialize ADC
	adcInitSequencer	(pAdcDesc, ADC_ONESHOT, adcSequence, TMAX_CONV) ;
	adcSetTrigger 		(pAdcDesc, LL_ADC_REG_TRIG_SOFTWARE, 0) ;

	// Configure DMA and callback
	adcDmaInit			(pAdcDesc, ADC_DMA_CIRCULAR, TMAX_CONV) ;
	adcDmaBuffer		(pAdcDesc, data16, NULL) ;
//	adcSetCallback		(pAdcDesc, adcTest2Callback, aaTaskSelfId ()) ;
	dmaSetCallback		(& pAdcDesc->dmaDesc, adcDmaCallback, aaTaskSelfId ()) ;

	adcStart			(pAdcDesc) ;

	while (1)
	{
		aaSignalClear (AA_SELFTASKID, 1) ;
		adcSwTrigger (pAdcDesc) ;

		aaSignalWait (1, NULL, AA_SIGNAL_OR, AA_INFINITE) ;
//		while (adcWaitForEoc (pAdcDesc) != ADC_ENONE)
//		{
//		}
		aaPrintf ("ADC: %04X %04X %04X %04X %04X %04X %04X %04X  %04X\n",
				data16[0], data16[1], data16[2], data16[3], data16[4], data16[5], data16[6], data16[7],
				(data16[0] + data16[1] + data16[2] + data16[3] + data16[4] + data16[5] + data16[6] + data16[7]) / 8) ;

		aaTaskDelay (100) ;
	}
}

//----------------------------------------------------------------------
//	Dual ADC1+ADC2

#define	ADC_TIMER		3

#define	DUALMAX_CONV	8

const uint8_t			dualChannels [2] = { 3, 7 } ;	// PA3, PA7
const adcSamplingTime_t	dualSampling [2] =
{
			{ 3, LL_ADC_SAMPLINGTIME_480CYCLES },		// ADC cycles count: 7=>480
			{ 7, LL_ADC_SAMPLINGTIME_480CYCLES}
} ;
const uint8_t			dualSeqAdc1  [DUALMAX_CONV] = { 3, 3, 3, 3, 3, 3, 3, 3 } ;
const uint8_t			dualSeqAdc2  [DUALMAX_CONV] = { 7, 7, 7, 7, 7, 7, 7, 7 } ;

adcDesc_t			* pAdcDesc1 ;
adcDesc_t			* pAdcDesc2 ;

uint16_t			dualData32 [DUALMAX_CONV] ;

void adcTest3Callback (adcDesc_t * pAdcDesc, uint32_t event, uintptr_t arg)
{
	aaLogMes ("adcTest3Callback %d\n", event, 0, 0, 0, 0) ;
	if (event == ADC_EVENT_EOC)
	{
//		aaSignalSend ((aaTaskId_t) arg, 1) ;
	}

	(void) pAdcDesc ;
	(void) arg ;
}

void	adcTest3 ()
{
	const timDesc_t		* pTimDesc ;
	aaSignal_t			sigsOut ;
	ADC_Common_TypeDef	* pCommon = ADC123_COMMON ;

	aaLogMes ("ADC Test 3\n", 0, 0, 0, 0, 0) ;

	// Configure ADC with SCAN and software trigger
	adcInit				(ADC_DUAL, ADC_PRES4, 0) ;						// ADC common parameters
	adcInitChannels 	(ADC_ADC1, adcChannels, sizeof (adcChannels) / sizeof (uint8_t)) ;	// Configure analog pins, common to ADC1 a nd ADC2
//	adcInitAdc 			(ADCX | ADC_SCAN | ADC_INT_EOC, & pAdcDesc) ;	// Initialize ADC
//	adcInitAdc 			(ADCX | ADC_SCAN | ADC_RESO8BITS, & pAdcDesc) ;					// Initialize ADC

	adcInitAdc 			(ADC_ADC1 | ADC_SCAN | ADC_INT_EOC, & pAdcDesc1) ;					// Initialize ADC1
	adcInitSequencer	(pAdcDesc1, ADC_ONESHOT, dualSeqAdc1, DUALMAX_CONV) ;
	adcInitSamplingTime	(pAdcDesc1, dualSampling, sizeof (dualSampling) / sizeof (adcSamplingTime_t)) ;
	adcSetTrigger 		(pAdcDesc1, LL_ADC_REG_TRIG_EXT_TIM3_TRGO, LL_ADC_REG_TRIG_EXT_RISING) ;

	adcInitAdc 			(ADC_ADC2 | ADC_SCAN, & pAdcDesc2) ;					// Initialize ADC2
	adcInitSequencer	(pAdcDesc2, ADC_ONESHOT, dualSeqAdc2, DUALMAX_CONV) ;
	adcInitSamplingTime	(pAdcDesc2, dualSampling, sizeof (dualSampling) / sizeof (adcSamplingTime_t)) ;

	// Configure DMA and callback
	adcDmaInit			(pAdcDesc1, ADC_DMA_CIRCULAR, TMAX_CONV) ;
	adcDmaBuffer		(pAdcDesc1, dualData32, NULL) ;
	dmaSetCallback		(& pAdcDesc1->dmaDesc, adcDmaCallback, aaTaskSelfId ()) ;
	adcSetCallback		(pAdcDesc1, adcTest3Callback, aaTaskSelfId ()) ;

	// Configure timer
	if (timGetDesc (ADC_TIMER, & pTimDesc) == TIM_EFAIL)
	{
		aaPrintf ("Timer DAC: Unknown descriptor id %u\n", ADC_TIMER) ;
		return ;
	}
	timTimeBaseInit     (pTimDesc, 0, 10, 0) ;		// 10 Hz
	timSetTriggerOutput (pTimDesc, LL_TIM_TRGO_UPDATE) ;		// Set timer TRGO

	adcStart			(pAdcDesc1) ;
	adcStart			(pAdcDesc2) ;

	aaSignalClear (AA_SELFTASKID, 1) ;
	timStart (pTimDesc) ;

	while (1)
	{
//		adcSwTrigger (pAdcDesc1) ;
//		aaSignalWait (1, NULL, AA_SIGNAL_OR, AA_INFINITE) ;
		do
		{
			aaTaskDelay (1) ;
			aaSignalWait (1, & sigsOut, AA_SIGNAL_OR, 0) ;
		} while ((sigsOut & 1) == 0) ;

		aaPrintf ("ADC: %08X %08X %08X %08X %08X %08X %08X %08X\n",
				dualData32[0], dualData32[1], dualData32[2], dualData32[3],
				dualData32[4], dualData32[5], dualData32[6], dualData32[7]) ;

//		aaTaskDelay (100) ;
	}
}

//----------------------------------------------------------------------

void	adcTest ()
{
//	adcTest1 () ;
//	adcTest2 () ;
	adcTest3 () ;
}

//----------------------------------------------------------------------
