/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dmabasic.c	DMA management

	When		Who	What
	23/04/18	ac	Creation


----------------------------------------------------------------------
*/
#include	"aa.h"
#include	"aakernel.h"

#include	"gpiobasic.h"
#include	"dmabasic.h"

//----------------------------------------------------------------------

//	Array to get interrupt number of DMA stream
static	const uint8_t	dmaStreamIrqIndex [16] =
{
	DMA1_Stream0_IRQn,
	DMA1_Stream1_IRQn,
	DMA1_Stream2_IRQn,
	DMA1_Stream3_IRQn,
	DMA1_Stream4_IRQn,
	DMA1_Stream5_IRQn,
	DMA1_Stream6_IRQn,
	DMA1_Stream7_IRQn,
	
	DMA2_Stream0_IRQn,
	DMA2_Stream1_IRQn,
	DMA2_Stream2_IRQn,
	DMA2_Stream3_IRQn,
	DMA2_Stream4_IRQn,
	DMA2_Stream5_IRQn,
	DMA2_Stream6_IRQn,
	DMA2_Stream7_IRQn,
} ;

// Position of streams flags in LISR/HISR/LIFCR/HIFCR
static const uint8_t	dmaStreamFlagOffset [4] =
{
	DMA_LISR_FEIF0_Pos,
	DMA_LISR_FEIF1_Pos,
	DMA_LISR_FEIF2_Pos,
	DMA_LISR_FEIF3_Pos,
} ;

//----------------------------------------------------------------------

// DMA callback descriptor
typedef struct
{
	dmaCallback_t	fn ;	// Function to call, NULL if no function
	uintptr_t		arg ;	// User argument

} dmaCallbackDesc_t;

static	void dmaDefCallback (uint32_t event, uint32_t dir, uintptr_t arg) ;

static	dmaCallbackDesc_t	dmaCallbackArray [16] =
{
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },

	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
	{ dmaDefCallback, 0 },
} ;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Default DMA callback

static	void dmaDefCallback (uint32_t event, uint32_t dir, uintptr_t arg)
{
	(void) event ;
	(void) dir ;
	(void) arg ;
}

//----------------------------------------------------------------------
// Configure and enable DMA interrupt

void	dmaSetIrq (const dmaDesc_t * pDmaDesc)
{
	uint32_t	irqNum = dmaStreamIrqIndex [((pDmaDesc->dmaNum - 1) << 3) + DMA_GETSTREAM (pDmaDesc)] ;
	
	NVIC_SetPriority	(irqNum, pDmaDesc->irqPrio);
	NVIC_EnableIRQ		(irqNum);
 
}

//----------------------------------------------------------------------
// Disable DMA interrupt

void	dmaDisableIrq (const dmaDesc_t * pDmaDesc)
{
	uint32_t	irqNum = dmaStreamIrqIndex [((pDmaDesc->dmaNum - 1) << 3) + DMA_GETSTREAM (pDmaDesc)] ;
	
	NVIC_DisableIRQ (irqNum);
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------

void	dmaSetCallback (const dmaDesc_t * pDmaDesc, dmaCallback_t fn, uintptr_t arg)
{
	if (fn == NULL)
	{
		// No user callback, set default callback
		dmaCallbackArray [((pDmaDesc->dmaNum - 1) << 3) + DMA_GETSTREAM (pDmaDesc)].fn  = dmaDefCallback ;
	}
	else
	{
		dmaCallbackArray [((pDmaDesc->dmaNum - 1) << 3) + DMA_GETSTREAM (pDmaDesc)].fn  = fn ;
		dmaCallbackArray [((pDmaDesc->dmaNum - 1) << 3) + DMA_GETSTREAM (pDmaDesc)].arg = arg ;
	}
}

//----------------------------------------------------------------------
//	dmaNum: 1..2
//	streamNum: 0..7

static	void	DMA_Stream_IRQHandler (uint32_t dmaNum, uint32_t streamNum)
{
	DMA_TypeDef			* pDma = DMA_GETDMA (dmaNum) ;
	dmaCallbackDesc_t	* pCb = & dmaCallbackArray [((dmaNum-1) << 3) + streamNum] ;
	volatile uint32_t	* pClear ;
	uint32_t			reg ;
	uint32_t			maskBase, mask ;
	uint32_t			dir ;
	
	// Read interrupt status register
	if (streamNum < 4)
	{
		reg = pDma->LISR ;
		pClear = & pDma->LIFCR ;
	}
	else
	{
		reg = pDma->HISR ;
		pClear = & pDma->HIFCR ;
	}

	dir = LL_DMA_GetDataTransferDirection (pDma, streamNum) ;

	// maskBase: mask of LSB interrupt flag for the stream
	// To get other flags mask: left shift maskBase using stream0 pos values
	maskBase = 1 << dmaStreamFlagOffset [streamNum & 0x03u] ;
	
	// Check transfer complete
	mask = maskBase << DMA_LISR_TCIF0_Pos ;
	if ((reg & mask) != 0)
	{
		// Clear flag
		* pClear = mask ;

		(* pCb->fn) (DMA_EV_COMPLETE, dir, pCb->arg) ;
	}

	// Check half transfer complete
	mask =  maskBase << DMA_LISR_HTIF0_Pos ;
	if ((reg & mask) != 0)
	{
		// Clear flag
		* pClear = mask ;

		(* pCb->fn) (DMA_EV_HCOMPLETE, dir, pCb->arg) ;
	}

	// Check transfer error
	mask = maskBase <<  DMA_LISR_TEIF0_Pos ;
	if ((reg & mask) != 0)
	{
		// Clear flag
		* pClear = mask ;

		(* pCb->fn) (DMA_EV_ERROR, dir, pCb->arg) ;
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	DMA1 IRQ handlers

void DMA1_Stream0_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 0) ;
	aaIntExit () ;
}

void DMA1_Stream1_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 1) ;
	aaIntExit () ;
}

void DMA1_Stream2_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 2) ;
	aaIntExit () ;
}

void DMA1_Stream3_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 3) ;
	aaIntExit () ;
}

void DMA1_Stream4_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 4) ;
	aaIntExit () ;
}

void DMA1_Stream5_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 5) ;
	aaIntExit () ;
}

void DMA1_Stream6_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 6) ;
	aaIntExit () ;
}

void DMA1_Stream7_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (1, 7) ;
	aaIntExit () ;
}

//----------------------------------------------------------------------
//	DMA2 IRQ handlers

void DMA2_Stream0_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 0) ;
	aaIntExit () ;
}

void DMA2_Stream1_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 1) ;
	aaIntExit () ;
}

void DMA2_Stream2_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 2) ;
	aaIntExit () ;
}

void DMA2_Stream3_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 3) ;
	aaIntExit () ;
}

void DMA2_Stream4_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 4) ;
	aaIntExit () ;
}

void DMA2_Stream5_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 5) ;
	aaIntExit () ;
}

void DMA2_Stream6_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 6) ;
	aaIntExit () ;
}

void DMA2_Stream7_IRQHandler (void)
{
	aaIntEnter () ;
	DMA_Stream_IRQHandler (2, 7) ;
	aaIntExit () ;
}

//----------------------------------------------------------------------
