/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dmabasic.c	DMA management for F1

	When		Who	What
	29/05/20	ac	Creation


----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"

#include	"gpiobasic.h"
#include	"dmabasic.h"

//----------------------------------------------------------------------

// DMA callback descriptor
typedef struct dmaCallbackDesc_s
{
	dmaCallback_t	fn ;	// Function to call, NULL if no function
	uintptr_t		arg ;	// User argument

} dmaCallbackDesc_t ;

static	void dmaDefCallback (uint32_t event, uint32_t source, uintptr_t arg) ;

static	dmaCallbackDesc_t	dma1CallbackArray [] =
{
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
} ;

#if defined DMA2
static	dmaCallbackDesc_t	dma2CallbackArray [] =
{
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
	{ dmaDefCallback, 0u },
} ;
#endif

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Default DMA callback, when not set by the application.

static	void dmaDefCallback (uint32_t event, uint32_t source, uintptr_t arg)
{
	(void) event ;
	(void) source ;
	(void) arg ;
}

//----------------------------------------------------------------------
//	Enable DMA peripheral clock

void		dmaEnableClock	(dma_t * pDma)
{
	#if defined DMA2
		RCC->AHBENR |= (pDma == (dma_t *) DMA1) ? RCC_AHBENR_DMA1EN : RCC_AHBENR_DMA2EN ;
	#else
		(void) pDma ;
		RCC->AHBENR |= RCC_AHBENR_DMA1EN ;
	#endif
}

//----------------------------------------------------------------------
//	Return DMA_ISR flags for chanNum right aligned
//	Return a combination of: DMA_FLAG_GIF, DMA_FLAG_TCIF, DMA_FLAG_HTIF, DMA_FLAG_TEIF

uint32_t	dmaGetFlags		(const dmaDesc_t * pDmaDesc)
{
	dma_t		* pDma	= DMA_GETDMA     (pDmaDesc->dmaNum) ;
	uint32_t	chanNum	= DMA_GETCHANNEL (pDmaDesc) ;
	uint32_t	isr ;

	isr = pDma->ISR ;
	return ((isr >> ((chanNum) << 2u)) & DMA_FLAG_ALLIF) ;
}

//----------------------------------------------------------------------
//	Allows to clear DMA_ISR flags for streamNum. Bits are right aligned
//	Use a combination of: DMA_FLAG_TEIF, DMA_FLAG_HTIF, DMA_FLAG_TCIF, DMA_FLAG_GIF

void	dmaClearFlags	(const dmaDesc_t * pDmaDesc, uint32_t flags)
{
	dma_t		* pDma	= DMA_GETDMA     (pDmaDesc->dmaNum) ;
	uint32_t	chanNum	= DMA_GETCHANNEL (pDmaDesc) ;

	pDma->IFCR = (flags & DMA_FLAG_ALLIF) << ((chanNum) << 2u) ;
}

//----------------------------------------------------------------------
// Configure and enable DMA interrupt

void	dmaSetIrq (const dmaDesc_t * pDmaDesc)
{
	IRQn_Type	irqNum ;

	#if defined (DMA2)
		if (pDmaDesc->dmaNum == DMA_DMA1)
		{
			irqNum = DMA1_Channel1_IRQn + pDmaDesc->dmaChan ;
		}
		else
		{
			if (pDmaDesc->dmaChan < 3u)
			{
				irqNum = DMA2_Channel1_IRQn + pDmaDesc->dmaChan ;
			}
			else
			{
				irqNum = DMA2_Channel4_5_IRQn ;
			}
		}
	#else
		irqNum = DMA1_Channel1_IRQn + pDmaDesc->dmaChan ;
	#endif
	
	NVIC_SetPriority	(irqNum, pDmaDesc->irqPrio);
	NVIC_EnableIRQ		(irqNum) ;
}

//----------------------------------------------------------------------
// Disable DMA interrupt

void	dmaDisableIrq (const dmaDesc_t * pDmaDesc)
{
	IRQn_Type	irqNum ;

	#if defined (DMA2)
		if (pDmaDesc->dmaNum == DMA_DMA1)
		{
			irqNum = DMA1_Channel1_IRQn + pDmaDesc->dmaChan ;
		}
		else
		{
			if (pDmaDesc->dmaChan < 3u)
			{
				irqNum = DMA2_Channel1_IRQn + pDmaDesc->dmaChan ;
			}
			else
			{
				irqNum = DMA2_Channel4_5_IRQn ;
			}
		}
	#else
		irqNum = DMA1_Channel1_IRQn + pDmaDesc->dmaChan ;
	#endif
	NVIC_DisableIRQ (irqNum) ;
}

//--------------------------------------------------------------------------------
//	Set the DMA mode
//	Mode is: LL_DMA_MODE_NORMAL or LL_DMA_MODE_CIRCULAR

void	dmaSetMode (const dmaDesc_t * pDmaDesc, uint32_t mode)
{
	dma_t			* pDma ;
	dmaChannel_t	* pChannel ;
	uint32_t		reg ;

	AA_ASSERT (pDmaDesc->dmaNum != DMA_NONE) ;
	if (pDmaDesc->dmaNum == DMA_NONE)
	{
		return ;
	}
	pDma     = DMA_GETDMA (pDmaDesc->dmaNum) ;
	pChannel = dmaChannelAddressRaw (pDma, DMA_GETCHANNEL (pDmaDesc)) ;

	reg = pChannel->CCR ;
	reg &= ~LL_DMA_MODE_CIRCULAR ;
	pChannel->CCR = reg | mode ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Set DMA callback using DMA pointer and stream number

void	dmaSetCallbackRaw (dma_t * pDma, uint32_t chanNum, dmaCallback_t fn, uintptr_t arg)
{
	#if defined (DMA2)
		if (pDma == DMA2)
		{
			if (fn == NULL)
			{
				// No user callback, set default callback
				dma2CallbackArray [chanNum].fn  = dmaDefCallback ;
			}
			else
			{
				dma2CallbackArray [chanNum].fn  = fn ;
				dma2CallbackArray [chanNum].arg = arg ;
			}
		}
		else
	#endif
		{
			if (fn == NULL)
			{
				// No user callback, set default callback
				dma1CallbackArray [chanNum].fn  = dmaDefCallback ;
			}
			else
			{
				dma1CallbackArray [chanNum].fn  = fn ;
				dma1CallbackArray [chanNum].arg = arg ;
			}
		}
	(void) pDma ;	// When only DMA1 is defined
}

//----------------------------------------------------------------------
//	Set DMA callback using DMA descriptor

void	dmaSetCallback (const dmaDesc_t * pDmaDesc, dmaCallback_t fn, uintptr_t arg)
{
	dmaSetCallbackRaw (DMA_GETDMA (pDmaDesc->dmaNum), DMA_GETCHANNEL (pDmaDesc), fn, arg) ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// Interrupt handlers. Separate DMA1 and DMA2 treatments to optimize when there is only DMA1.

//	chanNum: 0..6    WARNING: not 1..7

static	void	DMA1_Channel_IRQHandler (uint32_t chanNum)
{
	dmaCallbackDesc_t	* pCb = & dma1CallbackArray [chanNum] ;
	uint32_t			reg ;
	uint32_t			maskBase, mask ;
	dma_t				* pDma = (dma_t *) DMA1 ;
	
	aaIntEnter () ;

	// Read interrupt status register
	reg = pDma->ISR ;

	// maskBase: mask of LSB interrupt flag for the channel
	// To get other flags mask: left shift maskBase using chan0 pos values
	maskBase = 1u << (chanNum * 4) ;
	
	// Check transfer complete
	mask = maskBase << DMA_ISR_TCIF1_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		pDma->IFCR = mask ;

		(* pCb->fn) (DMA_EV_COMPLETE, (1u << 16) | chanNum, pCb->arg) ;
	}

	// Check half transfer complete
	mask =  maskBase << DMA_ISR_HTIF1_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		pDma->IFCR = mask ;

		(* pCb->fn) (DMA_EV_HCOMPLETE, (1u << 16) | chanNum, pCb->arg) ;
	}

	// Check transfer error
	mask = maskBase <<  DMA_ISR_TEIF1_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		pDma->IFCR = mask ;

		(* pCb->fn) (DMA_EV_ERROR, (1u << 16) | chanNum, pCb->arg) ;
	}

/* Who use global interrupt ?
	// Check global interrupt
	mask = maskBase << DMA_ISR_GIF1_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		pDma->IFCR = mask ;

		(* pCb->fn) (DMA_EV_GLOBAL, (1u << 16) | chanNum, pCb->arg) ;
	}
*/
	aaIntExit () ;
}

//----------------------------------------------------------------------
//	DMA1 IRQ handlers

void DMA1_Channel1_IRQHandler (void) ;
void DMA1_Channel1_IRQHandler (void)
{
	DMA1_Channel_IRQHandler (0u) ;
}

void DMA1_Channel2_IRQHandler (void) ;
void DMA1_Channel2_IRQHandler (void)
{
	DMA1_Channel_IRQHandler (1u) ;
}

void DMA1_Channel3_IRQHandler (void) ;
void DMA1_Channel3_IRQHandler (void)
{
	DMA1_Channel_IRQHandler (2u) ;
}

void DMA1_Channel4_IRQHandler (void) ;
void DMA1_Channel4_IRQHandler (void)
{
	DMA1_Channel_IRQHandler (3u) ;
}

void DMA1_Channel5_IRQHandler (void) ;
void DMA1_Channel5_IRQHandler (void)
{
	DMA1_Channel_IRQHandler (4u) ;
}

void DMA1_Channel6_IRQHandler (void) ;
void DMA1_Channel6_IRQHandler (void)
{
	DMA1_Channel_IRQHandler (5u) ;
}

void DMA1_Channel7_IRQHandler (void) ;
void DMA1_Channel7_IRQHandler (void)
{
	DMA1_Channel_IRQHandler (6u) ;
}

//----------------------------------------------------------------------
//	DMA2 IRQ handlers

#if (defined DMA2)

//	chanNum: 0..4    WARNING: not 1..5

static	void	DMA2_Channel_IRQHandler (uint32_t chanNum)
{
	dmaCallbackDesc_t	* pCb = & dma2CallbackArray [chanNum] ;
	uint32_t			reg ;
	uint32_t			maskBase, mask ;

	aaIntEnter () ;

	// Read interrupt status register
	reg = DMA2->ISR ;

	// maskBase: mask of LSB interrupt flag for the channel
	// To get other flags mask: left shift maskBase using chan0 pos values
	maskBase = 1u << (chanNum * 4) ;

	// Check transfer complete
	mask = maskBase << DMA_ISR_TCIF1_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		DMA2->IFCR = mask ;

		(* pCb->fn) (DMA_EV_COMPLETE, (2u << 16) | chanNum, pCb->arg) ;
	}

	// Check half transfer complete
	mask =  maskBase << DMA_ISR_HTIF1_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		DMA2->IFCR = mask ;

		(* pCb->fn) (DMA_EV_HCOMPLETE, (2u << 16) | chanNum, pCb->arg) ;
	}

	// Check transfer error
	mask = maskBase <<  DMA_ISR_TEIF1_Pos ;
	if ((reg & mask) != 0u)
	{
		// Clear flag
		DMA2->IFCR = mask ;

		(* pCb->fn) (DMA_EV_ERROR, (2u << 16) | chanNum, pCb->arg) ;
	}

/* Who use global interrupt ?
	// Check global interrupt
	mask = maskBase << DMA_ISR_GIF1_Pos ;
	if ((reg & mask) != 0)
	{
		// Clear flag
		DMA2->IFCR = mask ;

		(* pCb->fn) (DMA_EV_COMPLETE, (2u << 16) | chanNum, pCb->arg) ;
	}
*/
	aaIntExit () ;
}

//----------------------------------------------------------------------

void DMA2_Channel1_IRQHandler (void) ;
void DMA2_Channel1_IRQHandler (void)
{
	DMA2_Channel_IRQHandler (0) ;
}

void DMA2_Channel2_IRQHandler (void) ;
void DMA2_Channel2_IRQHandler (void)
{
	DMA2_Channel_IRQHandler (1) ;
}

void DMA2_Channel3_IRQHandler (void) ;
void DMA2_Channel3_IRQHandler (void)
{
	DMA2_Channel_IRQHandler (2) ;
}

void DMA2_Channel4_5_IRQHandler (void) ;
void DMA2_Channel4_5_IRQHandler (void)
{
	if ((DMA2->ISR & 0x0000F000u) != 0)
	{
		DMA2_Channel_IRQHandler (3) ;
	}
	else
	{
		DMA2_Channel_IRQHandler (4) ;
	}
}

#endif
//----------------------------------------------------------------------
