/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dmabasic.c	DMA management for F1 (doesn't have data cache)

	When		Who	What
	29/05/20	ac	Creation
	29/07/21	ac	Add dmaGetFlagsRaw(), dmaClearFlagsRaw()
	11/10/21	ac	Add dmaStopChannel() and dmaStopChannelRaw()
					Change dmaResetChannel() to dmaChannelResetRaw()
					Change dmaChannelAddress() to dmaChannelAddressRaw()
					Add dmaSetMode()

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined	DMABASIC_H_
#define DMABASIC_H_
//----------------------------------------------------------------------

#include	"stm32f1xx_ll_dma.h"

// In the DMA files channels are numbered 1..7. Many thank to ST for not having numbered them from 0 to 6!

// STM32F103x4/6/8/B	Only DMA1
// STM32F103xC/D/E/F/G	DMA1 and DMA2

//----------------------------------------------------------------------

// Get pointer to DMA device from DMA number
#if (defined DMA2)
	#define	DMA_GETDMA(dmaNum)		((dma_t *)(((dmaNum) == 1u) ? DMA1 : DMA2))
#else
	#define	DMA_GETDMA(dmaNum)		((dma_t *) DMA1)
#endif
#define	DMA_GETCHANNEL(pDesc)		((pDesc)->dmaChan)

#define	DMA_NONE	0u
#define	DMA_DMA1	1u
#define	DMA_DMA2	2u

typedef struct dmaDesc_s
{
	uint8_t		dmaNum ;		//	DMA number 1 or 2, DMA_NONE => unused descriptor
	uint8_t		dmaChan ;		//	Channel number 0..7
	uint8_t		irqPrio ;		//	Interrupt priority
	uint16_t	config ;		//	Direction e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY, mode, priority...

} dmaDesc_t ;

// Event of DMA IRQ callback function
#define	DMA_EV_COMPLETE		1u		// Transfer complete
#define	DMA_EV_HCOMPLETE	2u		// Transfer half complete
#define	DMA_EV_ERROR		3u		// Transfer error
#define	DMA_EV_GLOBAL		4u		// Global interrupt (commented out in IRQ handler)

// Callback parameter source is 'dmaNum << 16 | channel' :
//		High word : DMA number (1 or 2)
//		Low word  : channel number (0..6) for DMA1, (0..4) for DMA2

// Prototype of DMA IRQ callback function
typedef void (* dmaCallback_t) (uint32_t event, uint32_t source, uintptr_t arg) ;

//----------------------------------------------------------------------
//	Some definitions of DMA and BDMA structures easier to use than those of LL

typedef struct dmaChannel_s
{
	__IO uint32_t	CCR ;         	// DMA channel x configuration register
	__IO uint32_t	CNDTR ;       	// DMA channel x number of data register
	__IO uint32_t	CPAR ;        	// DMA channel x peripheral address register
	__IO uint32_t	CMAR ;        	// DMA channel x memory address register
	__IO uint32_t	reserved ;		// Not defined by ST files...

} dmaChannel_t ;

// This is the structure pointed to by DMA1 or DMA2
typedef struct dma_s
{
	__IO uint32_t	ISR ;			// DMA interrupt status register
	__IO uint32_t	IFCR ;			// DMA interrupt flag clear register
	dmaChannel_t	chan [8] ;

} dma_t ;


#ifdef __cplusplus
extern "C" {
#endif

void		dmaEnableClock		(dma_t * pDma) ;
void		dmaSetIrq			(const dmaDesc_t * pDmaDesc) ;
void		dmaDisableIrq		(const dmaDesc_t * pDmaDesc) ;
void		dmaSetMode 			(const dmaDesc_t * pDmaDesc, uint32_t mode) ;
void		dmaSetCallback		(const dmaDesc_t * pDmaDesc, dmaCallback_t fn, uintptr_t arg) ;
void		dmaSetCallbackRaw	(dma_t * pDma, uint32_t streamNum, dmaCallback_t fn, uintptr_t arg) ;

// Flags to use with dmaGetFlags(), dmaClearFlags(), dmaGetFlagsRaw(), dmaClearFlagsRaw()
#define DMA_FLAG_GIF		((uint32_t) 0x00000001u)	// Global interrupt
#define DMA_FLAG_TCIF		((uint32_t) 0x00000002u)	// Transfer complete
#define DMA_FLAG_HTIF		((uint32_t) 0x00000004u)	// Transfer half complete
#define DMA_FLAG_TEIF		((uint32_t) 0x00000008u)	// Transfer error
#define DMA_FLAG_ALLIF		((uint32_t) 0x0000000Fu)

uint32_t	dmaGetFlags		(const dmaDesc_t * pDmaDesc) ;
void		dmaClearFlags	(const dmaDesc_t * pDmaDesc, uint32_t flags) ;

//	Allows to get or clear DMA_ISR flags for chanNum. Bits are right aligned
//	Use a combination of: DMA_FLAG_TEIF, DMA_FLAG_HTIF, DMA_FLAG_TCIF

__STATIC_INLINE uint32_t	dmaGetFlagsRaw		(dma_t * pDma, uint32_t chanNum)
{
	uint32_t	isr = pDma->ISR ;
	return ((isr >> (chanNum << 2)) & 0x0Fu) ;	// Each channel have 4 flags
}

__STATIC_INLINE void		dmaClearFlagsRaw	(dma_t * pDma, uint32_t chanNum, uint32_t flags)
{
	pDma->IFCR = (flags & 0x0Fu) << (chanNum << 2u) ;
}

// Get a pointer to a channel registers
__STATIC_INLINE dmaChannel_t * dmaChannelAddressRaw (dma_t * pDma, uint32_t chanNum)
{
	return & pDma->chan [chanNum] ;
}

__STATIC_INLINE dmaChannel_t * dmaChannelAddress (const dmaDesc_t * pDmaDesc)
{
	return dmaChannelAddressRaw (DMA_GETDMA (pDmaDesc->dmaNum), DMA_GETCHANNEL (pDmaDesc)) ;
}

// Set stream to reset state
__STATIC_INLINE void		dmaChannelResetRaw	(dma_t * pDma, uint32_t chanNum)
{
	dmaChannel_t	* pChannel = dmaChannelAddressRaw (pDma, chanNum) ;

	pChannel->CCR	= 0u ;
	pChannel->CNDTR	= 0u ;
	pChannel->CPAR	= 0u ;
	pChannel->CMAR	= 0u ;
	dmaClearFlagsRaw (pDma, chanNum, DMA_FLAG_ALLIF) ;
}

// Stop a stream and reset flags
__STATIC_INLINE void		dmaChannelStopRaw	(dma_t *  pDma, uint32_t chanNum)
{
	dmaChannel_t * pChannel = dmaChannelAddressRaw (pDma, chanNum) ;

	pChannel->CCR	&= ~DMA_CCR_EN ;
	dmaClearFlagsRaw (pDma, chanNum, DMA_FLAG_ALLIF) ;
}

__STATIC_INLINE void		dmaChannelStop	(const dmaDesc_t * pDmaDesc)
{
	dmaChannelStopRaw (DMA_GETDMA (pDmaDesc), DMA_GETCHANNEL (pDmaDesc)) ;
}

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// DMABASIC_H_
