/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	i2cC24xx.c	Master I2C library for 24Cxx EEPROM
				Allows to use 24C series of EEPROM with up to 2048 bytes

	When		Who	What
	09/04/18	ac	Creation

----------------------------------------------------------------------
*/

#include	"aa.h"

#include	"i2cbasic.h"
#include	"ee24Cxx.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------

#if ! defined (NULL)
	#define	NULL	((void *) 0)
#endif

const uint32_t	ee24cPageDesc [5] =
{
	//	Device 			  Page         Page
	//	address mask      count        size
		(0x00FFu << 16) | ( 16 << 8) |  8,		// EE24C01
		(0x00FFu << 16) | ( 32 << 8) |  8,		// EE24C02
		(0x00FDu << 16) | ( 32 << 8) | 16,		// EE24C04
		(0x00F9u << 16) | ( 64 << 8) | 16,		// EE24C08
		(0x00F1u << 16) | (128 << 8) | 16,		// EE24C16
} ;

//----------------------------------------------------------------------
//	Initialize an EEPROM descriptor

i2cStatus_t	ee24cInit (ee24cDesc_t * pDesc, uint32_t i2cId, ee24cType_t type, uint32_t i2cAddress)
{
	pDesc->pI2cDesc = i2cGetDesc (i2cId) ;
	if (pDesc->pI2cDesc == NULL)
	{
		return I2C_EFAIL ;
	}

	pDesc->i2cAddress = i2cAddress ;
	pDesc->eeType = type ;
	return I2C_EOK ;
}

//----------------------------------------------------------------------
//	Write data not crossing a page boundary

static	i2cStatus_t	ee24cPageWrite (ee24cDesc_t * pDesc, uint32_t offset, uint8_t * pBuffer, uint32_t count)
{
	i2cStatus_t	status ;

	// Compute device I2C address byte
	// header [0] is device address
	// header [1] is byte offset
	pDesc->header [0] = pDesc->i2cAddress & EE24C_PAGEMASK (pDesc->eeType) ;	// Clear Px bits
	pDesc->header [0] |= (offset >> 7) & ~EE24C_PAGEMASK (pDesc->eeType) ;	// Add Px bits
	pDesc->header [1] = offset & 0x00FFu ;

	// Send header
	status = i2cMasterStart (pDesc->pI2cDesc, pDesc->header [0], I2C_TX) ;
	if (status != I2C_EOK)
	{
		i2cMasterStop (pDesc->pI2cDesc) ;
		return status ;
	}
	status = i2cMasterWrite (pDesc->pI2cDesc, & pDesc->header [1], 1) ;

	// Send bytes
	status = i2cMasterWrite (pDesc->pI2cDesc, pBuffer, count) ;
	i2cMasterStop (pDesc->pI2cDesc) ;

	// Wait 10 ms
//	aaTaskDelay (10 / (1000 / bspGetTickRate ()) + 1) ;

	// Wait until end of write
	while (1)
	{
		status = i2cMasterStart (pDesc->pI2cDesc, pDesc->header [0], I2C_TX) ;
		if (status == I2C_EOK)
		{
			i2cMasterStop (pDesc->pI2cDesc) ;
			break ;
		}
	}

	return status ;
}

//----------------------------------------------------------------------
//	Write any amount of data anywhere in the EEPROM

i2cStatus_t	ee24cWrite (ee24cDesc_t * pDesc, uint32_t offset, uint8_t * pBuffer, uint32_t count)
{
	uint8_t		* pByte = pBuffer ;
	i2cStatus_t	status = I2C_EOK ;
	uint32_t	pageOffset = offset ;
	uint32_t	size = count ;
	uint32_t	temp ;

	// Write first incomplete page
	temp = pageOffset & (EE24C_PAGESIZE (pDesc->eeType) - 1) ;
	if (temp != 0)
	{
		// Offset is not at page boundary
		if (temp > size)
		{
			temp = size ;
		}
		status = ee24cPageWrite (	pDesc,
									pageOffset,
									pByte,
									EE24C_PAGESIZE (pDesc->eeType) - temp) ;
		pageOffset += temp ;
		size -= temp ;
		pByte = & pByte [temp] ;
	}

	// Write full pages
	temp = EE24C_PAGESIZE (pDesc->eeType) ;
	while (status == I2C_EOK  &&  size >= temp)
	{
		status = ee24cPageWrite (	pDesc,
									pageOffset,
									pByte,
									temp) ;
		pageOffset += temp ;
		size -= temp ;
		pByte = & pByte [temp] ;
	}


	// Write remaining in last page
	if (status == I2C_EOK  &&  size > 0)
	{
		status = ee24cPageWrite (	pDesc,
									pageOffset,
									pByte,
									size) ;
	}

	return status ;
}

//----------------------------------------------------------------------
//	Read any amount of data anywhere in the EEPROM

i2cStatus_t	ee24cRead (ee24cDesc_t * pDesc, uint32_t offset, uint8_t * pBuffer, uint32_t count)
{
	i2cStatus_t	status = I2C_EOK ;

	// Compute device I2C address byte
	// header [0] is device address
	// header [1] is byte offset
	pDesc->header [0] = pDesc->i2cAddress & EE24C_PAGEMASK (pDesc->eeType) ;	// Clear Px bits
	pDesc->header [0] |= (offset >> 7) & ~EE24C_PAGEMASK (pDesc->eeType) ;	// Add Px bits
	pDesc->header [1] = offset & 0x00FFu ;

	// Send address
	status = i2cMasterStart (pDesc->pI2cDesc, pDesc->header [0], I2C_TX) ;
	if (status != I2C_EOK)
	{
		i2cMasterStop (pDesc->pI2cDesc) ;
	}
	else
	{
		// Send eeprom offset
		status = i2cMasterWrite (pDesc->pI2cDesc, & pDesc->header [1], 1) ;
		i2cMasterStop (pDesc->pI2cDesc) ;

		// Read bytes
		if (status == I2C_EOK)
		{
			status = i2cMasterReadStraight (pDesc->pI2cDesc, pDesc->header [0], pBuffer, count) ;
		}
	}

	return status ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	EE24CXX test

#include	"aa.h"
#include	"aaprintf.h"
#include	"aautils.h"
#include	"ee24Cxx.h"

#define	EE24CADDDR	0xA0

static	uint8_t	eeBuffer [256] ;


void	ee24cTest ()
{
	ee24cDesc_t		eeDesc ;
	i2cStatus_t		sts ;
	uint32_t		size, offset ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	//	Initialize EEPROM descriptor
	sts = ee24cInit (& eeDesc, 2, EE24C02, EE24CADDDR) ;
	if (sts != I2C_EOK)
	{
		aaPrintf ("ee24cInit error: %d\n") ;
		return ;
	}

	// Initialize I2C used by EEPROM
	i2cConfigureMaster (eeDesc.pI2cDesc, 400000) ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Write 4 bytes at the beginning of page 0
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	for (uint32_t ii = 0 ; ii < 4 ; ii++)
	{
		eeBuffer [ii] = ii ;
	}

	size   = 4 ;
	offset = 0 ;
	sts = ee24cWrite (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("Write page 0 OK sts = %d\n", sts) ;
	}
	else
	{
		aaPrintf ("Write page 0 error sts = %d\n", sts) ;
	}

	// Read page 0
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	size   = EE24C_PAGESIZE (eeDesc.eeType) ;
	offset = 0 ;
	sts = ee24cRead (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("Read page 0 OK sts = %d\n", sts) ;
		aaDump (eeBuffer, size) ;
	}
	else
	{
		aaPrintf ("Read page 0 error sts = %d\n", sts) ;
		return ;
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Write more than 1 page, from middle of page 4
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	size   = 2 * EE24C_PAGESIZE (eeDesc.eeType) ;
	offset = (EE24C_PAGESIZE (eeDesc.eeType) * 4) + (EE24C_PAGESIZE (eeDesc.eeType) / 2) ;
	for (uint32_t ii = 0 ; ii < 2 * EE24C_PAGESIZE (eeDesc.eeType) ; ii++)
	{
		eeBuffer [ii] = ii ;
	}
	sts = ee24cWrite (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("Write page 4 OK sts = %d\n", sts) ;
	}
	else
	{
		aaPrintf ("Write page 4 error sts = %d\n", sts) ;
	}

	// Read data from page 4
	aaMEMSET (eeBuffer, 0, sizeof (eeBuffer)) ;
	sts = ee24cRead (& eeDesc, offset, eeBuffer, size) ;
	if (sts == I2C_EOK)
	{
		aaPrintf ("Read page 4 OK sts = %d\n", sts) ;
		aaDump (eeBuffer, size) ;
	}
	else
	{
		aaPrintf ("Read page 4 error sts = %d\n", sts) ;
		return ;
	}
}

//----------------------------------------------------------------------

