/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	gpiobasic.c	Some routine to manage basic GPIO and RCC features

	When		Who	What
	17/03/18	ac	Creation
	28/07/22	ac	Add return value to gpioConfigurePin() and gpioConfigureAndSetPin()

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"
#include	"gpiobasic.h"

#include	"stm32f1xx_ll_bus.h"
#include	"stm32f1xx_ll_rcc.h"

//----------------------------------------------------------------------

#define	OFFSET_MASK		0x0000FFFFu

//----------------------------------------------------------------------
// Peripheral clock enable
// Example:
// rccEnableClock (gpioPortFromName (pPinDesc->port), 1u << ((pPinDesc->port - GPIO_PORTA) + RCC_APB2ENR_IOPAEN_Pos) ;
// rccEnableClock (GPIOA, RCC_APB2ENR_IOPAEN) ;
// rccEnableClock (I2C3, RCC_APB1ENR_I2C3EN) ;

void	rccEnableClock (void * deviceAddress, uint32_t enableBitMask)
{
	switch ((uintptr_t) deviceAddress & ~OFFSET_MASK)
	{
		case APB1PERIPH_BASE:
			LL_APB1_GRP1_EnableClock (enableBitMask) ;	// e.g. LL_APB1_GRP1_PERIPH_I2C2
			break ;

		case APB2PERIPH_BASE:
			LL_APB2_GRP1_EnableClock (enableBitMask) ;	// e.g. LL_APB2_GRP1_PERIPH_I2C2
			break ;

#if defined(AHB1PERIPH_BASE)
		case AHB1PERIPH_BASE:
			LL_AHB1_GRP1_EnableClock (enableBitMask) ;
			break ;
#endif

#if defined(RCC_AHB2_SUPPORT)
		case AHB2PERIPH_BASE:
			LL_AHB2_GRP1_EnableClock (enableBitMask) ;
			break ;
#endif
			
		default:
			AA_ASSERT (0) ;
			break ;
	}
}

//----------------------------------------------------------------------
void	rccDisableClock (void * deviceAddress, uint32_t enableBitMask)
{
	switch ((uintptr_t) deviceAddress & ~OFFSET_MASK)
	{
		case APB1PERIPH_BASE:
			LL_APB1_GRP1_EnableClock (enableBitMask) ;	// LL_APB1_GRP1_PERIPH_I2C2
			break ;

		case APB2PERIPH_BASE:
			LL_APB2_GRP1_EnableClock (enableBitMask) ;	// LL_APB1_GRP1_PERIPH_I2C2
			break ;

#if defined(AHB1PERIPH_BASE)
		case AHB1PERIPH_BASE:
			LL_AHB1_GRP1_EnableClock (enableBitMask) ;
			break ;
#endif

#if defined(RCC_AHB2_SUPPORT)
		case AHB2PERIPH_BASE:
			LL_AHB2_GRP1_EnableClock (enableBitMask) ;
			break ;
#endif
			
		default:
			AA_ASSERT (0) ;
			break ;
	}
}

//----------------------------------------------------------------------
//	All devices except timer

uint32_t	rccGetClockFrequency (void * deviceAddress)
{
	LL_RCC_ClocksTypeDef	clocks ;
	uint32_t				devClock ;
	
	LL_RCC_GetSystemClocksFreq (& clocks) ;
	
	switch ((uintptr_t)  deviceAddress & ~OFFSET_MASK)
	{
		case APB1PERIPH_BASE:
			devClock = clocks.PCLK1_Frequency ;
			break ;

		case APB2PERIPH_BASE:
			devClock = clocks.PCLK2_Frequency ;
			break ;

#if defined(AHB1PERIPH_BASE)
		case AHB1PERIPH_BASE:
			devClock = clocks.HCLK_Frequency ;
			break ;
#endif

#if defined(RCC_AHB2_SUPPORT)
		case AHB2PERIPH_BASE:
			devClock = clocks.HCLK_Frequency ;
			break ;
#endif
			
		default:
			AA_ASSERT (0) ;
			devClock = 0u ;
			break ;
	}
	return devClock ;
}

//----------------------------------------------------------------------
//	Timer clock depends on APB bus prescaler value
//	Timer clock may be APBx peripherals frequency x2

uint32_t	rccGetTimerClockFrequency (void * deviceAddress)
{
	LL_RCC_ClocksTypeDef	clocks ;
	uint32_t				prescaler ;
	uint32_t				timClock ;
	
	LL_RCC_GetSystemClocksFreq (& clocks) ;
	
	switch ((uintptr_t)  deviceAddress & ~OFFSET_MASK)
	{
		case APB1PERIPH_BASE:
			timClock = clocks.PCLK1_Frequency ;
			prescaler = LL_RCC_GetAPB1Prescaler () ;
			if (prescaler >= RCC_CFGR_PPRE1_DIV2)
			{
				timClock *= 2u ;
			}
			break ;
			
		case APB2PERIPH_BASE:
			timClock = clocks.PCLK2_Frequency ;
			prescaler = LL_RCC_GetAPB2Prescaler () ;
			if (prescaler >= RCC_CFGR_PPRE2_DIV2)
			{
				timClock *= 2u ;
			}
			timClock = clocks.PCLK2_Frequency ;
			break ;
			
		default:
			AA_ASSERT (0) ;
			timClock = 0u ;
			break ;
	}
	return timClock ;
}

//----------------------------------------------------------------------

void	rccResetPeriph (void * deviceAddress, uint32_t enableBitMask)
{
	switch ((uintptr_t) deviceAddress & ~OFFSET_MASK)
	{
		case APB1PERIPH_BASE:
			LL_APB1_GRP1_ForceReset   (enableBitMask) ;
			LL_APB1_GRP1_ReleaseReset (enableBitMask) ;
			break ;

		case APB2PERIPH_BASE:
			LL_APB2_GRP1_ForceReset   (enableBitMask) ;
			LL_APB2_GRP1_ReleaseReset (enableBitMask) ;
			break ;

#if defined(AHB1PERIPH_BASE)
		case AHB1PERIPH_BASE:
			LL_AHB1_GRP1_ForceReset   (enableBitMask) ;
			LL_AHB1_GRP1_ReleaseReset (enableBitMask) ;
			break ;
#endif

#if defined(RCC_AHB2_SUPPORT)
		case AHB2PERIPH_BASE:
			LL_AHB2_GRP1_ForceReset   (enableBitMask) ;
			LL_AHB2_GRP1_ReleaseReset (enableBitMask) ;
			break ;
#endif
			
		default:
			AA_ASSERT (0) ;
			break ;
	}
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// Remap peripheral pins
// if remap contain one of AA_REMAP_xxx then the remap for this peripheral is done.
// If the low bits of remap are 0, the remap is reset, else it is set to the low bits value.

static void	gpioRemap (uint32_t remap)
{
	uint32_t	reg = AFIO->MAPR ;
	uint32_t	bits = remap & 0x03u ;		// Up to 2 bits per peripheral

	switch (remap & 0x0F0u)
	{
		case AA_REMAP_SPI1:
			reg &= ~AFIO_MAPR_SPI1_REMAP ;
			if (bits != 0u)
			{
				reg |= AFIO_MAPR_SPI1_REMAP ;
			}
			break ;

		case AA_REMAP_I2C1:
			reg &= ~AFIO_MAPR_I2C1_REMAP ;
			if (bits != 0u)
			{
				reg |= AFIO_MAPR_I2C1_REMAP ;
			}
			break ;

		case AA_REMAP_USART1:
			reg &= ~AFIO_MAPR_USART1_REMAP ;
			if (bits != 0u)
			{
				reg |= AFIO_MAPR_USART1_REMAP ;
			}
			break ;

		case AA_REMAP_USART2:
			reg &= ~AFIO_MAPR_USART2_REMAP ;
			if (bits != 0u)
			{
				reg |= AFIO_MAPR_USART2_REMAP ;
			}
			break ;

#if (defined AFIO_MAPR_USART3_REMAP)
		case AA_REMAP_USART3:
			reg &= ~AFIO_MAPR_USART3_REMAP ;
			if (bits != 0u)
			{
				reg |= bits << AFIO_MAPR_USART3_REMAP_Pos ;
			}
			break ;
#endif

		case AA_REMAP_TIM1:
			reg &= ~AFIO_MAPR_TIM1_REMAP ;
			if (bits != 0u)
			{
				reg |= bits << AFIO_MAPR_TIM1_REMAP_Pos ;
			}
			break ;

		case AA_REMAP_TIM2:
			reg &= ~AFIO_MAPR_TIM2_REMAP ;
			if (bits != 0u)
			{
				reg |= bits << AFIO_MAPR_TIM2_REMAP_Pos ;
			}
			break ;

		case AA_REMAP_TIM3:
			reg &= ~AFIO_MAPR_TIM3_REMAP ;
			if (bits != 0u)
			{
				reg |= bits << AFIO_MAPR_TIM3_REMAP_Pos ;
			}
			break ;

#if (defined AFIO_MAPR_TIM4_REMAP)
		case AA_REMAP_TIM4:
			reg &= ~AFIO_MAPR_TIM4_REMAP ;
			if (bits != 0u)
			{
				reg |= AFIO_MAPR_TIM4_REMAP ;
			}
			break ;
#endif

		case AA_REMAP_CAN:
			reg &= ~AFIO_MAPR_CAN_REMAP ;
			if (bits != 0u)
			{
				reg |= bits << AFIO_MAPR_CAN_REMAP_Pos ;
			}
			break ;

		case AA_REMAP_PD01:
			reg &= ~AFIO_MAPR_PD01_REMAP ;
			if (bits != 0u)
			{
				reg |= AFIO_MAPR_PD01_REMAP ;
			}
			break ;

#if (defined AFIO_MAPR_TIM5CH4_IREMAP)
		case AA_REMAP_TIM5:
			reg &= ~AFIO_MAPR_TIM5CH4_IREMAP ;
			if (bits != 0u)
			{
				reg |= AFIO_MAPR_TIM5CH4_IREMAP ;
			}
			break ;
#endif

#if (defined AFIO_MAPR_ADC1_ETRGINJ_REMAP)
		case AA_REMAP_ADC1:
			reg &= ~(AFIO_MAPR_ADC1_ETRGINJ_REMAP | AFIO_MAPR_ADC1_ETRGREG_REMAP) ;
			if (bits != 0u)
			{
				reg |= bits << AFIO_MAPR_ADC1_ETRGINJ_REMAP_Pos ;
			}
			break ;
#endif

#if (defined AFIO_MAPR_ADC2_ETRGINJ_REMAP)
		case AA_REMAP_ADC2:
			reg &= ~(AFIO_MAPR_ADC2_ETRGINJ_REMAP | AFIO_MAPR_ADC2_ETRGREG_REMAP) ;
			if (bits != 0u)
			{
				reg |= bits << AFIO_MAPR_ADC2_ETRGINJ_REMAP_Pos ;
			}
			break ;
#endif

		default:
			break ;

	}
	AFIO->MAPR = reg ;
}

//----------------------------------------------------------------------
// Configure one pin using its pin descriptor
// Uses a 4-byte descriptor instead of 24 bytes in the stack with LL

	uint32_t	gpioConfigureAndSetPin_ (const gpioPinDesc_t * pPinDesc, uint32_t value)
{
	GPIO_TypeDef		* pGpio ;
	uint32_t			reg ;
	
	if (pPinDesc->port == 0u || pPinDesc->port == '0')
	{
		// Pin not used
		return AA_EFAIL ;
	}

	pGpio = gpioPortFromName (pPinDesc->port) ;

	// Enable port clock
	rccEnableClock (pGpio, 1u << ((uint32_t)(pPinDesc->port - AA_GPIO_PORTA) + RCC_APB2ENR_IOPAEN_Pos)) ;

	// Must remap the pins of the peripheral ?
	if (pPinDesc->af != AA_REMAP_NONE)
	{
		gpioRemap (pPinDesc->af) ;
	}

	reg = pPinDesc->flags ;
	if ((reg & 0x10u) != 0u)
	{
		// Input, clear MODE bits
		reg &= GPIO_CRL_CNF0 ;
	}
	else
	{
		// Output: check speed, must be != 0
		if ((reg & GPIO_CRL_MODE0) == 0u)
		{
			reg |= 1u ;		// Force medium speed
		}
	}

	if (value != 0u)
	{
		// Set output value before direction mode
		if ((value & 1u) == 0u)
		{
			gpioPortPinReset (pGpio, 1u << pPinDesc->pin) ;
		}
		else
		{
			gpioPortPinSet (pGpio, 1u << pPinDesc->pin) ;
		}
	}

	if (pPinDesc->pin < 8u)
	{
		pGpio->CRL &= ~((GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << (pPinDesc->pin << 2)) ;
		pGpio->CRL |= reg << (pPinDesc->pin << 2) ;
	}
	else
	{
		pGpio->CRH &= ~((GPIO_CRL_CNF0 | GPIO_CRL_MODE0) << ((pPinDesc->pin - 8u) << 2)) ;
		pGpio->CRH |= reg << ((pPinDesc->pin - 8u) << 2) ;
	}
	return AA_ENONE ;
}

//----------------------------------------------------------------------
//	Set the value of the pin using its descriptor
//	Atomic

void	gpioPinSet		(const gpioPinDesc_t * pPinDesc, uint32_t bValue)
{
	GPIO_TypeDef	* pPort ;
	uint32_t		pinMask ;

	AA_ASSERT (pPinDesc->port != 0u  &&  pPinDesc->port != '0') ;

	pPort = gpioPortFromName (pPinDesc->port) ;
	pinMask = 1u << pPinDesc->pin ;

	if (bValue == 0u)
	{
		#if (BSP_HAS_BRR != 0)
			pPort->BRR = pinMask ;			// Reset bit
		#else
			pPort->BSRR = pinMask << 16u ;	// Reset bit
		#endif
	}
	else
	{
		pPort->BSRR = pinMask ;				// Set bit
	}
}

//----------------------------------------------------------------------
//	Get the value of the pin using its descriptor

uint32_t	gpioPinGet		(const gpioPinDesc_t * pPinDesc)
{
	GPIO_TypeDef	* pPort ;

	AA_ASSERT (pPinDesc->port != 0u  &&  pPinDesc->port != '0') ;

	pPort = gpioPortFromName (pPinDesc->port) ;
	return (pPort->IDR & (1u << pPinDesc->pin)) == 0u ? 0u : 1u ;
}

//----------------------------------------------------------------------
//	Toggle the value of the pin using its descriptor (atomic)

void	gpioPinToggle		(const gpioPinDesc_t * pPinDesc)
{
	GPIO_TypeDef	* pPort ;
	uint32_t		pinMask ;
	uint32_t		odr ;

	AA_ASSERT (pPinDesc->port != 0u  &&  pPinDesc->port != '0') ;

	pPort = gpioPortFromName (pPinDesc->port) ;
	pinMask	= 1u << pPinDesc->pin ;
	odr = pPort->ODR ;

	pPort->BSRR = ((odr & pinMask) << 16u) | (~odr & pinMask) ;
}

//----------------------------------------------------------------------
