/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	gpiobasic.h	Some routine to manage basic GPIO and RCC features

	When		Who	What
	17/03/18	ac	Creation
	20/08/18	ac	Port for STL32F1xx
	28/07/22	ac	Add return value to gpioConfigurePin() and gpioConfigureAndSetPin()

	For STL32F1xx 'af' field is used for pin mode GPIO_MODE_xxx
----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined GPIOBASIC_H_
#define	GPIOBASIC_H_
//----------------------------------------------------------------------

typedef struct
{
	uint8_t		port ;		//	'A', 'B', ... Unused pin: 0 or '0'
	uint8_t		pin ;		//	0 to 15
	uint8_t		af ;		//	Remap config
	uint8_t		flags ;		//	Mode, Speed
	
} gpioPinDesc_t ;

//	To specify PGIO port
#define AA_GPIO_PORTA				((uint8_t)'A')
#define AA_GPIO_PORTB				((uint8_t)'B')
#define AA_GPIO_PORTC				((uint8_t)'C')
#define AA_GPIO_PORTD				((uint8_t)'D')
#define AA_GPIO_PORTE				((uint8_t)'E')
#define AA_GPIO_PORTF				((uint8_t)'F')
#define AA_GPIO_PORTG				((uint8_t)'G')
#define AA_GPIO_PORTH				((uint8_t)'H')
#define AA_GPIO_PORTI				((uint8_t)'I')
#define AA_GPIO_PORTJ				((uint8_t)'J')
#define AA_GPIO_PORTK				((uint8_t)'K')

// gpioPinDesc_t.flags bits definitions
// Pull-up or pull-down only for input on this MCU family

#define	AA_GPIO_MODE_OUTPUT_PP		0x00u	// Output Push-Pull
#define	AA_GPIO_MODE_OUTPUT_OD		0x04u	// Output Open-Drain
#define	AA_GPIO_MODE_ALT_PP			0x08u	// Alternate Output Push-Pull
#define	AA_GPIO_MODE_ALT_OD			0x0Cu	// Alternate Output Open-Drain
#define	AA_GPIO_MODE_ANALOG			0x10u	// Analog
#define	AA_GPIO_MODE_INPUT			0x14u	// Input float (no pull up/down)
#define	AA_GPIO_MODE_INPUT_PU		0x18u	// Input pull up
#define	AA_GPIO_MODE_INPUT_PD		0x1Cu	// Input pull down

#define AA_GPIO_SPEED_LOW			0x02u	// Select I/O low output speed     2 MHz
#define AA_GPIO_SPEED_MEDIUM		0x01u	// Select I/O medium output speed 10 MHz
#define AA_GPIO_SPEED_HIGH			0x03u	// Select I/O fast output speed   50 MHz

#define AA_GPIO_MODE_ADC			(AA_GPIO_MODE_ANALOG)	//	Configure for ADC input

// For the remap field of gpioPinDesc_t
// Add the bits value AA_REMAP_xxx to the gpioPinDesc_t.af field
#define	AA_REMAP_NONE				((uint8_t) 0x00u)

#define	AA_REMAP_SPI1				((uint8_t) 0x10u)
#define	AA_REMAP_I2C1				((uint8_t) 0x20u)
#define	AA_REMAP_USART1				((uint8_t) 0x30u)
#define	AA_REMAP_USART2				((uint8_t) 0x40u)
#define	AA_REMAP_USART3				((uint8_t) 0x50u)
#define	AA_REMAP_TIM1				((uint8_t) 0x60u)
#define	AA_REMAP_TIM2				((uint8_t) 0x70u)
#define	AA_REMAP_TIM3				((uint8_t) 0x80u)
#define	AA_REMAP_TIM4				((uint8_t) 0x90u)
#define	AA_REMAP_CAN				((uint8_t) 0xA0u)
#define	AA_REMAP_PD01				((uint8_t) 0xB0u)
#define	AA_REMAP_TIM5				((uint8_t) 0xC0u)
#define	AA_REMAP_ADC1				((uint8_t) 0xD0u)	// Add 2 bits for injected and regular
#define	AA_REMAP_ADC2				((uint8_t) 0xE0u)	// Add 2 bits for injected and regular

//----------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

// Get a pointer to port registers from port name ('A', 'B', ..)
#define gpioPortFromName(portName)		((GPIO_TypeDef *)(GPIOA_BASE + (GPIOB_BASE-GPIOA_BASE)*((uint32_t)portName-(uint32_t)AA_GPIO_PORTA)))

// Get a pointer to port registers from port number (0, 1, ..)
#define gpioPortFromNum(portNum)		((GPIO_TypeDef *)(GPIOA_BASE + (GPIOB_BASE-GPIOA_BASE)*((uint32_t)portNum)))

uint32_t	gpioConfigureAndSetPin_	(const gpioPinDesc_t * pPinDesc, uint32_t value) ;		// Libray Internal use only

// Set the pin value before set pin mode: avoid glitch if there is external pull up/down
// The pin value must be set after the port clock is enabled and before the mode is set.
__ALWAYS_STATIC_INLINE uint32_t	gpioConfigureAndSetPin	(const gpioPinDesc_t * pPinDesc, uint32_t value)
{
	return gpioConfigureAndSetPin_ (pPinDesc, 0x8000u | value) ;
}

__ALWAYS_STATIC_INLINE uint32_t	gpioConfigurePin		(const gpioPinDesc_t * pPinDesc)
{
	return gpioConfigureAndSetPin_ (pPinDesc, 0) ;
}

void		gpioPinSet					(const gpioPinDesc_t * pPinDesc, uint32_t bValue) ;
uint32_t	gpioPinGet					(const gpioPinDesc_t * pPinDesc) ;
void		gpioPinToggle				(const gpioPinDesc_t * pPinDesc) ;

// To set multiple bits at the same time. Pins parameter is a bit mask of the pins to set.
__ALWAYS_STATIC_INLINE void	gpioPortPinSet			(GPIO_TypeDef * pPort, uint32_t pins)
{
	pPort->BSRR = pins ;
}

// To clear multiple bits at the same time. Pins parameter is a bit mask of the pins to reset.
__ALWAYS_STATIC_INLINE void	gpioPortPinReset		(GPIO_TypeDef * pPort, uint32_t pins)
{
	#if (BSP_HAS_BRR != 0)
		pPort->BRR = pins ;
	#else
		pPort->BSRR = pins << 16u ;
	#endif
}

// To toggle multiple bits at the same time. Pins parameter is a bit mask of the pins to toggle.
__ALWAYS_STATIC_INLINE void	gpioPortPinToggle		(GPIO_TypeDef * pPort, uint32_t pins)
{
	uint32_t		odr ;

	odr = pPort->ODR ;
	pPort->BSRR = ((odr & pins) << 16u) | (~odr & pins) ;
}

void		rccEnableClock				(void * deviceAddress, uint32_t enableBitMask) ;
void		rccDisableClock				(void * deviceAddress, uint32_t enableBitMask) ;
void		rccResetPeriph 				(void * deviceAddress, uint32_t enableBitMask) ;
uint32_t	rccGetClockFrequency		(void * deviceAddress) ;
uint32_t	rccGetTimerClockFrequency	(void * deviceAddress) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// GPIOBASIC_H_
