/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	i2cbasic.c	Master and Slave basic routines:
				Slave mode interrupt,
				Master mode interrupt or polling
				7 bits address only (no 10 bits address)
				Slave have only one address, no general call

	When		Who	What
	01/03/18	ac	Creation

	This code is a rework of STMicroelectronics application note AN2824

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2020 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"

#include	"i2cbasic.h"

#include	"stm32f1xx_ll_i2c.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------

#define I2C_DIRECTION_TX 		0
#define I2C_DIRECTION_RX 		1

#define I2C_DUTYCYCLE			LL_I2C_DUTYCYCLE_2

//----------------------------------------------------------------------
// Transaction descriptor, useful for i2c functions using interrupts

typedef struct
{
	uint8_t		* pBuffer ;
	uint32_t	size ;
	uint32_t	idx ;
	uint16_t	direction ;
	uint16_t	address ;
	i2cStatus_t	status ;

} i2cIo_t ;

//----------------------------------------------------------------------

#if (WITH_I2C_X == 1)

	#if (defined STM32F1)
		#include "i2cF1xx.h"		//	I2C descriptors for STM32F4xx
	#endif

	STATIC const uint32_t		i2cArrayCount = sizeof (i2cArray) / sizeof (i2cDesc_t) ;

#endif

//----------------------------------------------------------------------
//	One I/O descriptor for each physical I2C peripheral

#if (WITH_I2C_MASTER_INTR == 1) || (WITH_I2C_SLAVE == 1)

	#if (WITH_I2C_1 == 1)
		i2cIo_t		i2cIo1 ;
	#endif	// WITH_I2C_1

	#if (WITH_I2C_2 == 1)
		i2cIo_t		i2cIo2 ;
	#endif	// WITH_I2C_2

	#if (WITH_I2C_3 == 1)
		i2cIo_t		i2cIo3 ;
	#endif	// WITH_I2C_3

	#if (WITH_I2C_4 == 1  &&  defined I2C4)
		i2cIo_t		i2cIo4 ;
	#endif	// WITH_I2C_4

#endif	// WITH_I2C_MASTER_INTR  ||  WITH_I2C_SLAVE

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Get the I/O descriptor address for a I2C peripheral

static	i2cIo_t *	i2cGetIoDesc (I2C_TypeDef * pI2C) ;

static	i2cIo_t *	i2cGetIoDesc (I2C_TypeDef * pI2C)
{
	#if (WITH_I2C_X == 0)
	{
		(void) pI2C ;	// To avoid compiler warning when none I2C is supported
	}
	#endif

	#if (WITH_I2C_1 == 1)
	{
		if (pI2C == I2C1)
		{
			return & i2cIo1 ;
		}
	}
	#endif	// WITH_I2C_1

	#if (WITH_I2C_2 == 1)
	{
		if (pI2C == I2C2)
		{
			return & i2cIo2 ;
		}
	}
	#endif	// WITH_I2C_2

	#if (WITH_I2C_3 == 1)
	{
		if (pI2C == I2C2)
		{
			return & i2cIo3 ;
		}
	}
	#endif	// WITH_I2C_3

	#if (WITH_I2C_4 == 1  &&  defined I2C4)
	{
		if (pI2C == I2C4)
		{
			return & i2cIo4 ;
		}
	}
	#endif // WITH_I2C_4

	AA_ASSERT (0) ;
	return NULL ;
}

//----------------------------------------------------------------------
//	Get an I2C descriptor from its identifier

const i2cDesc_t *	i2cGetDesc	(uint32_t i2cId)
{
	#if (WITH_I2C_X == 1)
	{
		uint32_t	ii ;

		for (ii = 0 ; ii < i2cArrayCount ; ii++)
		{
			if (i2cArray [ii].id == i2cId)
			{
				return & i2cArray [ii] ;
			}
		}
	}
	#else
	{
		(void) i2cId ;
	}
	#endif

	return NULL ;	// Identifier not found
}

//----------------------------------------------------------------------
//	Configure I2C master: polling or interrupt

//	Enable clock stretching
//	7 bits address
//	Own address: 0
//	No own address 2
//	No general call

void	i2cConfigureMaster (const i2cDesc_t * pI2cDesc, uint32_t speed)
{
	uint32_t				i2cClock ;

	gpioConfigurePin (& pI2cDesc->scl) ;
	gpioConfigurePin (& pI2cDesc->sda) ;

	// Enable the peripheral clock for I2C
	rccEnableClock (pI2cDesc->pI2c, 1u << pI2cDesc->periphEnPos) ;

	// Reset peripheral
	rccResetPeriph (pI2cDesc->pI2c, 1u << pI2cDesc->periphRstPos) ;

	// Configure Event IT:
	// - Set priority for I2Cx_EV_IRQn
	// - Enable I2Cx_EV_IRQn
	NVIC_SetPriority (pI2cDesc->evIrqn, pI2cDesc->irqPriority);
	NVIC_EnableIRQ   (pI2cDesc->evIrqn) ;

	// Configure Error IT:
	// - Set priority for I2Cx_ER_IRQn
	// - Enable I2Cx_ER_IRQn
	NVIC_SetPriority (pI2cDesc->erIrqn, pI2cDesc->irqPriority);
	NVIC_EnableIRQ   (pI2cDesc->erIrqn) ;

	// Disable I2Cx prior modifying configuration registers
	LL_I2C_Disable (pI2cDesc->pI2c);

	// Retrieve Clock frequencies
	i2cClock = rccGetClockFrequency (pI2cDesc->pI2c) ;

	// Configure the SCL Clock Speed
	LL_I2C_ConfigSpeed (pI2cDesc->pI2c, i2cClock, speed, I2C_DUTYCYCLE) ;

	// Configure the Own Address1
	// Reset Values of :
	//    - OwnAddress1 is 0x00
	//    - OwnAddrSize is LL_I2C_OWNADDRESS1_7BIT
	//LL_I2C_SetOwnAddress1(pI2cDesc->pI2c, 0x00, LL_I2C_OWNADDRESS1_7BIT);

	// Enable Clock stretching
	// Reset Value is Clock stretching enabled
	//LL_I2C_EnableClockStretching (pI2cDesc->pI2c) ;

	// Enable General Call
	// Reset Value is General Call disabled
	//LL_I2C_EnableGeneralCall (pI2cDesc->pI2c) ;

	// Configure the 7bits Own Address2
	// Reset Values of :
	//    - OwnAddress2 is 0x00
	//    - Own Address2 is disabled
	//LL_I2C_SetOwnAddress2 (pI2cDesc->pI2c, 0x00) ;
	//LL_I2C_DisableOwnAddress2 (pI2cDesc->pI2c) ;

	// Enable Peripheral in I2C mode
	// Reset Value is I2C mode
	//LL_I2C_SetMode (pI2cDesc->pI2c, LL_I2C_MODE_I2C) ;

	// Activate  I2C
	LL_I2C_Enable (pI2cDesc->pI2c) ;

	// Prepare acknowledge for next reception
	LL_I2C_AcknowledgeNextData (pI2cDesc->pI2c, LL_I2C_ACK) ;
}

//----------------------------------------------------------------------
//	Configure slave, interrupt

//	Enable clock stretching
//	7 bits address
//	Own address: 0
//	No own address 2
//	No general call


void	i2cConfigureSlave (const i2cDesc_t * pI2cDesc, uint32_t ownAddress1)
{
	I2C_TypeDef		* pI2c = pI2cDesc->pI2c ;

	gpioConfigurePin (& pI2cDesc->scl) ;
	gpioConfigurePin (& pI2cDesc->sda) ;

	// Enable the peripheral clock for I2C
	rccEnableClock (pI2c, 1u << pI2cDesc->periphEnPos) ;

	// Reset peripheral
	rccResetPeriph (pI2c, 1u << pI2cDesc->periphRstPos) ;

	// Configure Event IT:
	// - Set priority for I2Cx_EV_IRQn
	// - Enable I2Cx_EV_IRQn
	NVIC_SetPriority (pI2cDesc->evIrqn, pI2cDesc->irqPriority);
	NVIC_EnableIRQ   (pI2cDesc->evIrqn) ;

	// Configure Error IT:
	// - Set priority for I2Cx_ER_IRQn
	// - Enable I2Cx_ER_IRQn
	NVIC_SetPriority (pI2cDesc->erIrqn, pI2cDesc->irqPriority);
	NVIC_EnableIRQ   (pI2cDesc->erIrqn) ;

	// Disable I2C prior modifying configuration registers
	LL_I2C_Disable (pI2c) ;

	// Configure the Own Address1 :
	LL_I2C_SetOwnAddress1 (pI2c, ownAddress1, LL_I2C_OWNADDRESS1_7BIT) ;

	// Enable Clock stretching. Reset Value is Clock stretching enabled
	LL_I2C_EnableClockStretching (pI2c) ;

	// Enable General Call. Reset Value is General Call disabled
	//LL_I2C_EnableGeneralCall (pI2cDesc->pI2c) ;

	// Configure the 7bits Own Address2
	// Reset Values of :
	//    - OwnAddress2 is 0x00
	//    - Own Address2 is disabled
	//LL_I2C_SetOwnAddress2     (pI2cDesc->pI2c, ownAddress2);
	//LL_I2C_DisableOwnAddress2 (pI2cDesc->pI2c) ;

	// Enable Peripheral in I2C mode. Reset Value is I2C mode
	LL_I2C_SetMode (pI2c, LL_I2C_MODE_I2C) ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//	Wait for ADDR flag, or an error as AF, ARLO, TIMEOUT, too long delay

static	i2cStatus_t	i2cWaitAddr (I2C_TypeDef * pI2c)
{
	uint32_t	timeout ;
	uint32_t	reg ;

	// Wait until ADDR or error flag is set: EV6
	timeout = 0xFFFF ;
	while (1)
	{
		reg = pI2c->SR1 ;
		if ((reg & I2C_SR1_ADDR) != 0)
		{
			// Don't clear ADDR
			return I2C_EOK ;
		}
		if ((reg & I2C_SR1_AF) != 0)
		{
			// No slave acknowledge
			// Program the STOP
			pI2c->CR1 |= I2C_CR1_STOP;
			while ((pI2c->CR1 & I2C_CR1_STOP) == I2C_CR1_STOP)
			{
			}
			return I2C_EAF ;
		}
		if ((reg & I2C_SR1_ARLO) != 0)
		{
			return I2C_EARLO ;	// Arbitration lost
		}
		if ((reg & I2C_SR1_TIMEOUT) != 0)
		{
			return I2C_ETMO ;
		}
		if (timeout-- == 0)
		{
			return I2C_ETMO ;
		}
	}
}


//----------------------------------------------------------------------
//	Send START condition and slave 7 bits address
//	Direction : I2C_TX or I2C_RX
//	Returns I2C_EOK in case of success, I2C_EAF if the slave doesn't acknowledge the address,
//	else returns another error value

i2cStatus_t i2cMasterStart (const i2cDesc_t * pI2cDesc, uint32_t slaveAddress, i2cMode_t direction)
{
	__IO uint32_t	timeout = 0;
	I2C_TypeDef		* pI2c = pI2cDesc->pI2c ;
	i2cStatus_t		status ;

	// Disable I2C interrupts
	pI2c->CR2 &= ~(I2C_CR2_ITERREN | I2C_CR2_ITEVTEN | I2C_CR2_ITBUFEN) ;

	// Clear previous AF or ARLO
	pI2c->SR1 &= ~(I2C_SR1_AF | I2C_SR1_ARLO | I2C_SR1_TIMEOUT) ;

	// Enable ACK
	pI2c->CR1 |= I2C_CR1_ACK ;

	// Send START condition
	pI2c->CR1 |= I2C_CR1_START;
	// Wait until SB flag is set: EV5
	timeout = 0xFFFF;
	while ((pI2c->SR1 & I2C_SR1_SB) == 0)
	{
		if (timeout-- == 0)
		{
			return I2C_ETMO;
		}
	}

	// Send slave address
	if (direction == I2C_TX)
	{
		pI2c->DR = slaveAddress & ~I2C_OAR1_ADD0 ;	// Reset the address bit 0 for write
	}
	else
	{
		pI2c->DR = slaveAddress | I2C_OAR1_ADD0 ;	// Set the address bit 0 for read
	}

	// Wait ADDR flag or error
	status =  i2cWaitAddr (pI2c) ;
	if (status == I2C_EOK)
	{
		// Clear ADDR flag
		__IO uint32_t temp = pI2c->SR2 ;
		(void) temp ;
	}
	return status ;
}

//----------------------------------------------------------------------
// Send STOP condition

void		i2cMasterStop (const i2cDesc_t * pI2cDesc)
{
	I2C_TypeDef		* pI2c = pI2cDesc->pI2c ;

	// Send STOP condition only if I2C is busy
	if ((pI2c->SR2 & I2C_SR2_BUSY) == I2C_SR2_BUSY)
	{
		pI2c->CR1 |= I2C_CR1_STOP ;
		// Make sure that the STOP bit is cleared by Hardware
		while ((pI2c->CR1 & I2C_CR1_STOP) == I2C_CR1_STOP)
		{
		}
	}
}

//----------------------------------------------------------------------
//	Check slave availability. Master should be configured before
//	Returns I2C_EOK if slave answered ACK
//	Returns I2C_EAF if slave doesn't answered

i2cStatus_t		i2cCheckDevice (const i2cDesc_t * pI2cDesc, uint32_t slaveAddress)
{
	i2cStatus_t	status ;

	status = i2cMasterStart (pI2cDesc, slaveAddress, I2C_TX) ;	// TX mandatory
	i2cMasterStop (pI2cDesc) ;
	return status ;
}

//----------------------------------------------------------------------
//	Send data so slave
//	Returns I2C_EOK if all bytes sent
//	Returns I2C_EAF if slave doesn't acknowledge before all bytes are sent
//	No STOP or RESTART at the end

i2cStatus_t i2cMasterWrite (const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToWrite)
{
	I2C_TypeDef		* pI2c = pI2cDesc->pI2c ;
	uint8_t 		* pByte = pBuffer ;
	uint32_t		size = numByteToWrite ;
	i2cStatus_t		status = I2C_ENONE ;

	// Write the first data in DR register (EV8_1)
	pI2c->DR = * pByte++ ;
	size-- ;

	// While there is data to be written
	while (size != 0)
	{
		// Poll on BTF to send data because in polling mode we can not guarantee the
		// EV8 software sequence is managed before the current byte transfer completes
		// BTF not set if AF
		while ((pI2c->SR1 & (I2C_SR1_BTF | I2C_SR1_AF)) == 0)
		{
		}
		if ((pI2c->SR1 & I2C_SR1_AF) != 0)
		{
			status = I2C_EAF ;
			break ;		// No acknowledge
		}

		// Send the current byte
		pI2c->DR = * pByte++ ;
		size-- ;
	}
	// EV8_2: Wait until BTF is set before return
	// BTF not set if AF
	while ((pI2c->SR1 & (I2C_SR1_BTF | I2C_SR1_AF)) == 0)
	{
	}

	if (size == 0)
	{
		status = I2C_EOK ;	// All bytes of message sent
	}
	return status;
}

//----------------------------------------------------------------------
//	Receive bytes with ACK
//	May be called more than once

i2cStatus_t	i2cMasterRead (const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToRead)
{
	I2C_TypeDef		* pI2c	= pI2cDesc->pI2c ;
	uint8_t 		* pByte	= pBuffer ;
	uint32_t		size	= numByteToRead ;
	i2cStatus_t		status	= I2C_EOK ;

	while (size != 0)
	{
		while ((pI2c->SR1 & I2C_SR1_RXNE) == 0)
		{
		}

		* pByte++ = (uint8_t) pI2c->DR ;
		size-- ;
	}

	return status ;
}

//----------------------------------------------------------------------
//	The last call to get message data, at least 3 bytes (ST I2C limitation)
//	Send NACK and STOP condition

i2cStatus_t	i2cMasterReadLast (const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToRead)
{
	I2C_TypeDef		* pI2c	= pI2cDesc->pI2c ;
	uint8_t 		* pByte	= pBuffer ;
	uint32_t		size	= numByteToRead ;
	i2cStatus_t		status	= I2C_EOK ;

	// Enable ACK
	pI2c->CR1 |= I2C_CR1_ACK ;

	AA_ASSERT (size >= 3) ;

	while (size > 0)
	{
		if (size > 3)
		{
			// Poll on BTF to receive data because in polling mode we can not guarantee the
			// EV7 software sequence is managed before the current byte transfer completes
			while ((pI2c->SR1 & I2C_SR1_BTF) != I2C_SR1_BTF)
			{
			}
			// Read data
			* pByte++ = (uint8_t) pI2c->DR ;
			size-- ;
		}
		else
		{
			// It remains to read three data: data N-2, data N-1, Data N

			// Wait until BTF is set: Data N-2 in DR and data N-1 in shift register
			while ((pI2c->SR1 & I2C_SR1_BTF) != I2C_SR1_BTF)
			{
			}
			// Clear ACK
			pI2c->CR1 &= ~I2C_CR1_ACK ;

			// Disable IRQs around data reading and STOP programming because of the limitation
			__disable_irq () ;
			// Read Data N-2
			* pByte++ = (uint8_t) pI2c->DR ;		// N-2
			// Program the STOP
			pI2c->CR1 |= I2C_CR1_STOP ;
			// Read Data N-1
			* pByte++ = (uint8_t) pI2c->DR ;		// N-1
			// Re-enable IRQs
			__enable_irq () ;

			// Wait until RXNE is set (DR contains the last data)
			while ((pI2c->SR1 & I2C_SR1_RXNE) != I2C_SR1_RXNE)
			{
			}
			// Read Data N
			* pByte++ = (uint8_t) pI2c->DR ;		// N
			// Reset the number of bytes to be read by master
			size = 0 ;
		}
	}
	// Make sure that the STOP bit is cleared by Hardware before CR1 write access
	while ((pI2c->CR1 & I2C_CR1_STOP) == I2C_CR1_STOP)
	{
	}

	return status ;
}

//----------------------------------------------------------------------
//	Read message of any size, in one time

i2cStatus_t	i2cMasterReadStraight (const i2cDesc_t * pI2cDesc, uint32_t slaveAddress, uint8_t * pBuffer, uint32_t numByteToRead)
{
	I2C_TypeDef		* pI2c	= pI2cDesc->pI2c ;
	uint8_t 		* pByte	= pBuffer ;
	uint32_t		size	= numByteToRead ;
	i2cStatus_t		status	= I2C_EOK ;
	uint32_t		timeout ;
	__IO uint32_t	temp = 0;

	(void) temp ;

	if (size == 1)
	{
		// Send START condition
		pI2c->CR1 |= I2C_CR1_START;
		// Wait until SB flag is set: EV5
		timeout = 0xFFFF;
		while ((pI2c->SR1 & I2C_SR1_SB) != I2C_SR1_SB)
        {
			if (timeout-- == 0)
				return I2C_ETMO;
        }
		// Send slave address
		pI2c->DR = slaveAddress | I2C_OAR1_ADD0 ;	// Set the address bit0 for read
		// Wait until ADDR is set: EV6_3, then program ACK = 0, clear ADDR
		// and program the STOP just after ADDR is cleared. The EV6_3
		// software sequence must complete before the current byte end of transfer.
		// Wait until ADDR is set

		// Wait until ADDR or AF is set: EV6
		status = i2cWaitAddr (pI2c) ;
		if (status == I2C_EOK)
		{
			// ADDR flag set

			// Clear ACK bit
			pI2c->CR1 &= ~I2C_CR1_ACK;
			// Disable all active IRQs around ADDR clearing and STOP programming because the EV6_3
			// software sequence must complete before the current byte end of transfer
			__disable_irq();
			// Clear ADDR flag
			temp = pI2c->SR2;
			// Program the STOP
			pI2c->CR1 |= I2C_CR1_STOP;
			// Re-enable IRQs
			__enable_irq();
			// Wait until a data is received in DR register (RXNE = 1) EV7
			while ((pI2c->SR1 & I2C_SR1_RXNE) != I2C_SR1_RXNE)
			{
			}
			// Read the data
			* pByte++ = (uint8_t) pI2c->DR;
			// Make sure that the STOP bit is cleared by Hardware before CR1 write access
			while ((pI2c->CR1 & I2C_CR1_STOP) == I2C_CR1_STOP)
			{
			}
			status = I2C_EOK ;	// Receive ended without error
		}
	}
	else if (size == 2)
	{
		// Set POS bit
		pI2c->CR1 |= I2C_CR1_POS;
		// Send START condition
		pI2c->CR1 |= I2C_CR1_START;
		// Wait until SB flag is set: EV5
		timeout = 0xFFFF;
		while ((pI2c->SR1 & I2C_SR1_SB) != I2C_SR1_SB)
		{
			if (timeout-- == 0)
				return I2C_ETMO;
		}
		// Send the slave address
		pI2c->DR = slaveAddress | I2C_OAR1_ADD0 ;	// Set the address bit0 for read
		// Wait until ADDR or AF is set: EV6
		status = i2cWaitAddr (pI2c) ;
		if (status == I2C_EOK)
		{
			// ADDR flag set

			// EV6_1: The acknowledge disable should be done just after EV6,
			// that is after ADDR is cleared, so disable all active IRQs around ADDR clearing and
			// ACK clearing
			__disable_irq();
			// Clear ADDR by reading SR2 register
			temp = pI2c->SR2;
			// Clear ACK
			pI2c->CR1 &= ~I2C_CR1_ACK;
			//Re-enable IRQs
			__enable_irq();
			// Wait until BTF is set
			while ((pI2c->SR1 & I2C_SR1_BTF) != I2C_SR1_BTF)
			{
			}
			// Disable IRQs around STOP programming and data reading because of the limitation ?
			__disable_irq();
			// Program the STOP
			pI2c->CR1 |= I2C_CR1_STOP;
			// Read first data
			* pByte++ = (uint8_t) pI2c->DR;
			// Re-enable IRQs
			__enable_irq();
			// Read second data
			* pByte++ = (uint8_t) pI2c->DR;
			// Make sure that the STOP bit is cleared by Hardware before CR1 write access
			while ((pI2c->CR1 & I2C_CR1_STOP) == I2C_CR1_STOP)
			{
			}
			// Clear POS bit
			pI2c->CR1  &= ~I2C_CR1_POS;
			status = I2C_EOK ;	// Receive ended without error
		}
	}
    else	// pI2cIo->size > 2
    {
    	status = i2cMasterStart (pI2cDesc, slaveAddress, I2C_RX) ;
		if (status == I2C_EAF)
		{
			i2cMasterStop (pI2cDesc) ;
	  	}
		if (status != I2C_EOK)
		{
			return status ;
		}
		status = i2cMasterReadLast (pI2cDesc, pBuffer, numByteToRead) ;
	}

	return status ;
}

//----------------------------------------------------------------------

void i2cDeactivate (const i2cDesc_t * pI2cDesc)
{
	LL_I2C_Disable (pI2cDesc->pI2c) ;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------

#if (WITH_I2C_MASTER_INTR == 1)

/**
  * @brief  Reads buffer of bytes  from the slave.
  * @param pI2cDesc: I2C peripheral descriptor.
  * @param pBuffer: Buffer of bytes to be read from the slave.
  * @param NumByteToRead: Number of bytes to be read by the Master.
  * @param SlaveAddress: The address of the slave to be addressed by the Master (left aligned).
  * @retval : i2cStatus_t value.
  */

i2cStatus_t i2cMasterReadIt (const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToRead, uint8_t slaveAddress)
{
	__IO uint32_t	temp = 0;
	I2C_TypeDef		* pI2c = pI2cDesc->pI2c ;
	i2cIo_t			* pI2cIo = i2cGetIoDesc (pI2c) ;

	// Initialize IO descriptor
	pI2cIo->pBuffer 	= pBuffer ;
	pI2cIo->size		= numByteToRead ;
	pI2cIo->address		= slaveAddress ;
	pI2cIo->direction	= I2C_DIRECTION_RX ;
	pI2cIo->idx			= 0 ;
	pI2cIo->status		= I2C_ENONE ;
	(void) temp ;

	// Enable I2C errors interrupts (used in all modes: Polling and Interrupts
	pI2c->CR2 |= I2C_CR2_ITERREN ;

	// Enable Acknowledge
	pI2c->CR1  |= I2C_CR1_ACK;

	// Enable EVT and BUF IT*/
	pI2c->CR2 |= I2C_CR2_ITEVTEN;
	pI2c->CR2 |= I2C_CR2_ITBUFEN;
	// Set the I2C direction to reception
	pI2cIo->direction = I2C_DIRECTION_RX;
	pI2cIo->address |= I2C_OAR1_ADD0 ;	// Set the address bit0 for write
	// Send START condition
	pI2c->CR1 |= I2C_CR1_START;
	// Wait until the START condition is generated on the bus: START bit is cleared by hardware
	while ((pI2c->CR1 & I2C_CR1_START) == I2C_CR1_START)
	{
	}
	// Wait until BUSY flag is reset (until a STOP is generated)
	while ((pI2cIo->status == I2C_ENONE)  &&  (pI2c->SR2 & I2C_SR2_BUSY) == I2C_SR2_BUSY)
	{
	}
	if (pI2cIo->status != I2C_ENONE)
	{
		// Send STOP condition
		pI2c->CR1 |= I2C_CR1_STOP;
		// Make sure that the STOP bit is cleared by Hardware
		while ((pI2c->CR1 & I2C_CR1_STOP) == I2C_CR1_STOP)
		{
		}
	}
	else
	{
		pI2cIo->status = I2C_EOK ;	// Message received without error
	}

	return pI2cIo->status ;
}

//----------------------------------------------------------------------
// I2Cx Master Transmission using Interrupt [with highest priority in the application?]
/**
  * @brief  Writes a buffer of bytes to the slave.
  * @param pI2cDesc: I2C peripheral descriptor.
  * @param pBuffer: Buffer of bytes to be sent to the slave.
  * @param numByteToWrite: Number of bytes to be sent by the Master.
  * @param slaveAddress: The address of the slave to be addressed by the Master (left aligned).
  * @retval : i2cStatus_t value.
  */
i2cStatus_t i2cMasterWriteIt (const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToWrite, uint8_t slaveAddress)
{
	__IO uint32_t	temp = 0;
	I2C_TypeDef		* pI2c = pI2cDesc->pI2c ;
	i2cIo_t			* pI2cIo = i2cGetIoDesc (pI2c) ;

	// Initialize IO descriptor
	pI2cIo->pBuffer 	= pBuffer ;
	pI2cIo->size		= numByteToWrite ;
	pI2cIo->address		= slaveAddress ;
	pI2cIo->direction	= I2C_DIRECTION_TX ;
	pI2cIo->idx			= 0 ;
	pI2cIo->status		= I2C_ENONE ;
	(void) temp ;

	// Enable Error IT
	pI2c->CR2 |= I2C_CR2_ITERREN;

	// Enable EVT IT
	pI2c->CR2 |= I2C_CR2_ITEVTEN ;
	// Enable BUF IT
	pI2c->CR2 |= I2C_CR2_ITBUFEN ;
	// Set the I2C direction to Transmission
	pI2cIo->direction = I2C_DIRECTION_TX;
	pI2cIo->address &= (uint16_t) ~I2C_OAR1_ADD0 ;
	// Send START condition
	pI2c->CR1 |= I2C_CR1_START;
	// Wait until the START condition is generated on the bus: the START bit is cleared by hardware
	while ((pI2c->CR1 & I2C_CR1_START) == I2C_CR1_START)
	{
	}
	// Wait until BUSY flag is reset: a STOP has been generated on the bus signaling the end of transmission
	// Or an error has occurred if status != I2C_ENONE: so send stop condition
	while ((pI2cIo->status == I2C_ENONE)  &&  (pI2c->SR2 & I2C_SR2_BUSY) == I2C_SR2_BUSY)
	{
	}
	if (pI2cIo->status != I2C_ENONE)
	{
		// Busy flag set so send STOP condition
		pI2c->CR1 |= I2C_CR1_STOP;
		// Make sure that the STOP bit is cleared by Hardware
		while ((pI2c->CR1 & I2C_CR1_STOP) == I2C_CR1_STOP)
		{
		}
	}
	if (pI2cIo->size == 0)
	{
		pI2cIo->status = I2C_EOK ;	// Message transmitted without error
	}

	return pI2cIo->status ;
}

#endif	// WITH_I2C_MASTER_INTR

//----------------------------------------------------------------------
//----------------------------------------------------------------------

#if (WITH_I2C_SLAVE == 1)

/**
  * @brief Prepares the I2Cx slave for transmission using interrupt with highest priority in the application
  * @param pI2cDesc: I2C peripheral descriptor.
  * @param pBuffer: Buffer of bytes to be sent/received from the slave.
  * @param maxSize: the size of pBuffer.
  * @retval : None.
  */

void i2cSlaveReadWrite (const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t maxSize)
{
	I2C_TypeDef		* pI2c = pI2cDesc->pI2c ;
	i2cIo_t			* pI2cIo = i2cGetIoDesc (pI2c) ;

	// Initialize IO descriptor
	pI2cIo->pBuffer 	= pBuffer ;
	pI2cIo->size		= maxSize ;
	pI2cIo->status		= I2C_ENONE ;

	// Activate  slave I2C
	LL_I2C_Enable (pI2c) ;

	// Enable Event IT needed for ADDR and STOPF events ITs
	pI2c->CR2 |= I2C_CR2_ITEVTEN ;

	// Enable Error IT
	pI2c->CR2 |= I2C_CR2_ITERREN ;

	// Enable Buffer IT (TXE and RXNE ITs)
	pI2c->CR2 |= I2C_CR2_ITBUFEN ;

	// Prepare acknowledge for Slave address reception
	pI2c->CR1 |= I2C_CR1_ACK ;
}

//----------------------------------------------------------------------
//	If transfer is not terminated, returns I2C_ENONE
//	If transfer is terminated without error, returns I2C_EOK and transmitted/received size
//	else returns another error value

i2cStatus_t		i2cSlaveResult		(const i2cDesc_t * pI2cDesc, uint32_t * pSize)
{
	i2cIo_t		* pI2cIo = i2cGetIoDesc (pI2cDesc->pI2c) ;

	if (pSize != NULL)
	{
		* pSize = pI2cIo->idx ;
	}
	return pI2cIo->status ;
}

#endif	// WITH_I2C_SLAVE

//----------------------------------------------------------------------
//----------------------------------------------------------------------

#if (WITH_I2C_MASTER_INTR == 1) || (WITH_I2C_SLAVE == 1)

void I2Cx_EV_IRQHandler (I2C_TypeDef * pI2c, i2cIo_t * pI2cIo) ;

void I2Cx_EV_IRQHandler (I2C_TypeDef * pI2c, i2cIo_t * pI2cIo)
{
	__IO uint32_t SR1Register ;
	__IO uint32_t SR2Register ;

	aaIntEnter () ;

	// Read the I2C SR1 and SR2 status registers
	// This clears the ADDR flags, so should terminate before end of word transfer!
	SR1Register = pI2c->SR1 ;
	SR2Register = pI2c->SR2 ;

	// If I2C is slave (MSL flag = 0)
	if ((SR2Register & I2C_SR2_MSL) != I2C_SR2_MSL)
	{
		// If ADDR = 1: EV1
		if ((SR1Register & I2C_SR1_ADDR) == I2C_SR1_ADDR)
		{
			if ((SR2Register & I2C_SR2_TRA) != 0)
			{
				pI2cIo->direction = I2C_DIRECTION_TX ;
			}
			else
			{
				pI2cIo->direction = I2C_DIRECTION_RX ;
			}
			// Clear SR1Register and SR2Register variables to prepare for next IT
			SR1Register = 0;
			SR2Register = 0;
			// Initialize the transmit/receive counters for next transmission/reception using Interrupt
			pI2cIo->idx = 0 ;
		}
		// If TXE = 1: EV3
		if ((SR1Register & I2C_SR1_TXE) == I2C_SR1_TXE)
		{
			// Write data in I2C data register
			pI2c->DR = pI2cIo->pBuffer[pI2cIo->idx++];
			if (pI2cIo->idx > pI2cIo->size)
			{
				// Master requests more data than slave buffer size
				pI2cIo->status = I2C_ESIZE ;
			}
			SR1Register = 0;
			SR2Register = 0;
		}
		// If RXNE = 1: EV2
		if ((SR1Register & I2C_SR1_RXNE) == I2C_SR1_RXNE)
		{
			if (pI2cIo->idx == pI2cIo->size-2)
			{
				// To receive last byte
				// Disable acknowledgement
				pI2c->CR1  &= ~I2C_CR1_ACK ;
			}
			// Read data from I2C data register
			if (pI2cIo->idx < pI2cIo->size)
			{
				pI2cIo->pBuffer[pI2cIo->idx++] = (uint8_t) pI2c->DR ;
			}
			else
			{
				// Master sends more data than slave buffer size
				// Should never go here if the master takes into account the NACK
				pI2cIo->status = I2C_ESIZE ;
				SR1Register = pI2c->DR ;	// Read and trash
			}
			SR1Register = 0;
			SR2Register = 0;
   		}
		// If STOPF = 1: EV4 (Slave has detected a STOP condition on the bus
		if (( SR1Register & I2C_SR1_STOPF) == I2C_SR1_STOPF)
		{
			pI2c->CR1 &= ~I2C_CR1_PE ;	// De activate peripheral
			SR1Register = 0;
			SR2Register = 0;
			pI2cIo->status = I2C_EOK ;	// End of transfer without error
		}
	} // End slave mode

	// If SB = 1, I2C master sent a START on the bus: EV5)
	if ((SR1Register & I2C_SR1_SB) == I2C_SR1_SB)
	{
		// Send the slave address for transmission or for reception
		// according to the configured value in the master write routine
		pI2c->DR = pI2cIo->address ;
		SR1Register = 0;
		SR2Register = 0;
	}

	// If I2C is Master (MSL flag = 1)
    if ((SR2Register & I2C_SR2_MSL) == I2C_SR2_MSL)
    {
        // If ADDR = 1, EV6
        if ((SR1Register & I2C_SR1_ADDR) == I2C_SR1_ADDR)
        {
			if (pI2cIo->direction == I2C_DIRECTION_TX)
			{
				if (pI2cIo->size == 0)
				{
					// This is a device scan
					// Program the STOP
					pI2c->CR1 |= I2C_CR1_STOP ;
				}
				else
				{
					// Write the first data
					pI2cIo->idx = 0 ;
					// If no further data to be sent, disable the I2C BUF IT
					// in order to not have a TxE interrupt
					if (pI2cIo->size == 1)
					{
						pI2c->CR2 &= (uint16_t) ~I2C_CR2_ITBUFEN;
					}
					// Write the first data in the data register
					pI2c->DR = pI2cIo->pBuffer [pI2cIo->idx++];
					pI2cIo->size-- ;
				}
			}
			else
			{
				// Master Receiver
				// Initialize Receive counter
				pI2cIo->idx = 0 ;
				// At this stage, ADDR is cleared because both SR1 and SR2 were read.
				// EV6_1: used for single byte reception. The ACK disable and the STOP
				// Programming should be done just after ADDR is cleared.
				if (pI2cIo->size == 1)
				{
					// Clear ACK
					pI2c->CR1 &= ~I2C_CR1_ACK;
					// Program the STOP
					pI2c->CR1 |= I2C_CR1_STOP;
				}
			}
			SR1Register = 0;
			SR2Register = 0;
		}

		// Master transmits the remaining data: from data 2 until the last one.
		if (pI2cIo->size != 0  &&  (SR1Register & I2C_SR1_TXE) == I2C_SR1_TXE)
		{
			// TxE flag and there is still data to write
			// If only 1 data remains to write, disable the BUF IT in order
			// to not have again a TxE interrupt.
			if (pI2cIo->size == 1)
			{
				// Disable the BUF IT
				pI2c->CR2 &= (uint16_t) ~I2C_CR2_ITBUFEN ;
			}
			// Write the data in DR register
			pI2c->DR = pI2cIo->pBuffer [pI2cIo->idx++] ;
			pI2cIo->size-- ;
			SR1Register = 0 ;
			SR2Register = 0 ;
		}
// this is the reason why the interrupt level must be the highest ?
/*
		// If TXE is set and not BTF
		if ((SR1Register & 0x0084) == 0x0080)
		{
			// If there is still data to write
			if (pI2cIo->size != 0)
			{
				// If 1 data remains to write, disable the BUF IT in order
				// to not have again a TxE interrupt.
				if (pI2cIo->size == 1)
				{
					// Disable the BUF IT
					pI2c->CR2 &= (uint16_t) ~I2C_CR2_ITBUFEN;
				}
				// Write the data in DR register
				pI2c->DR = pI2cIo->pBuffer [pI2cIo->idx++];
				pI2cIo->size--;
			}
			SR1Register = 0;
			SR2Register = 0;
		}
*/
		// If BTF and TXE are set (EV8_2), program the STOP
		if ((SR1Register & (I2C_SR1_TXE | I2C_SR1_BTF)) == (I2C_SR1_TXE | I2C_SR1_BTF))
		{
			// Program the STOP
			pI2c->CR1 |= I2C_CR1_STOP ;
			// Disable EVT IT In order to not have again a BTF IT
			pI2c->CR2 &= (uint16_t) ~I2C_CR2_ITEVTEN ;
			SR1Register = 0 ;
			SR2Register = 0 ;
		}

		// If RXNE is set
		if ((SR1Register & I2C_SR1_RXNE) == I2C_SR1_RXNE)
		{
			// Read the data register
			pI2cIo->pBuffer [pI2cIo->idx++] = (uint8_t) pI2c->DR ;		// ??? To do after Clear ACK and Program the STOP?
			pI2cIo->size-- ;
			// If it remains only one byte to read, disable ACK and program the STOP (EV7_1)
			if (pI2cIo->size == 1)
			{
				// Clear ACK
				pI2c->CR1 &= ~I2C_CR1_ACK ;
				// Program the STOP
				pI2c->CR1 |= I2C_CR1_STOP ;
			}
			SR1Register = 0 ;
			SR2Register = 0 ;
		}
	}

	aaIntExit () ;
}

//----------------------------------------------------------------------

#if (WITH_I2C_1 == 1)
void I2C1_EV_IRQHandler (void) ;
void I2C1_EV_IRQHandler (void)
{
	I2Cx_EV_IRQHandler (I2C1, & i2cIo1) ;
}
#endif	// WITH_I2C_1

#if (WITH_I2C_2 == 1)
void I2C2_EV_IRQHandler (void) ;
void I2C2_EV_IRQHandler (void)
{
	I2Cx_EV_IRQHandler (I2C2, & i2cIo2) ;
}
#endif	// WITH_I2C_2

#if (WITH_I2C_3 == 1)
void I2C3_EV_IRQHandler (void) ;
void I2C3_EV_IRQHandler (void)
{
	I2Cx_EV_IRQHandler (I2C3, & i2cIo3) ;
}
#endif	// WITH_I2C_3

#if (WITH_I2C_4 == 1  &&  defined I2C4)
void I2C4_EV_IRQHandler (void) ;
void I2C4_EV_IRQHandler (void)
{
	I2Cx_EV_IRQHandler (I2C4, & i2cIo4) ;
}
#endif	// WITH_I2C_4

//----------------------------------------------------------------------

void I2Cx_ER_IRQHandler (I2C_TypeDef * pI2c, i2cIo_t * pI2cIo) ;

void I2Cx_ER_IRQHandler (I2C_TypeDef * pI2c, i2cIo_t * pI2cIo)
{
	__IO uint32_t SR1Register ;
	__IO uint32_t SR2Register ;

	aaIntEnter () ;

	// Read the I2C1 status register
	SR1Register = pI2c->SR1 ;

	// If AF = 1
	if ((SR1Register & I2C_SR1_AF) == I2C_SR1_AF)
	{
		pI2c->SR1 &= ~I2C_SR1_AF ;	// Clear AF
		SR1Register = 0 ;
	    SR2Register = pI2c->SR2 ;
		if ((SR2Register & I2C_SR2_MSL) != I2C_SR2_MSL)
		{
			// I2C is slave (MSL flag = 0)
			if (pI2cIo->idx == (pI2cIo->size+1))
			{
				// Size+1 bytes was transfered, and one remain in DR
				// So the transfer size is OK
				pI2cIo->idx-- ;
				pI2cIo->status = I2C_EOK ;
			}
			else
			{
				// The master stopped receive before end of buffer
				pI2cIo->status = I2C_EAF ;
			}
		}
		else
		{
			// I2C is master. The slave doesn't acknowledge
			pI2cIo->status = I2C_EAF ;
		}
	}
	// If ARLO = 1
	if ((SR1Register & I2C_SR1_ARLO) == I2C_SR1_ARLO)
	{
		pI2c->SR1 &= ~I2C_SR1_ARLO ;
		SR1Register = 0 ;
		pI2cIo->status = I2C_EARLO ;
	}
	// If BERR = 1
	if ((SR1Register & I2C_SR1_BERR) == I2C_SR1_BERR)
	{
		pI2c->SR1 &= ~I2C_SR1_BERR ;
		SR1Register = 0 ;
		pI2cIo->status = I2C_EBERR ;
	}

	// If OVR = 1
	if ((SR1Register & I2C_SR1_OVR) == I2C_SR1_OVR)
	{
		pI2c->SR1 &= ~I2C_SR1_OVR ;
		SR1Register = 0 ;
		pI2cIo->status = I2C_EOVR ;
	}

	aaIntExit () ;
}

//----------------------------------------------------------------------

#if (WITH_I2C_1 == 1)
void I2C1_ER_IRQHandler(void) ;
void I2C1_ER_IRQHandler(void)
{
	I2Cx_ER_IRQHandler (I2C1, & i2cIo1) ;
}
#endif	// WITH_I2C_1

#if (WITH_I2C_2 == 1)
void I2C2_ER_IRQHandler (void) ;
void I2C2_ER_IRQHandler (void)
{
	I2Cx_ER_IRQHandler (I2C2, & i2cIo2) ;
}
#endif	// WITH_I2C_2

#if (WITH_I2C_3 == 1)
void I2C3_ER_IRQHandler (void) ;
void I2C3_ER_IRQHandler (void)
{
	I2Cx_ER_IRQHandler (I2C3, & i2cIo3) ;
}
#endif	// WITH_I2C_3

#if (WITH_I2C_4 == 1  &&  defined I2C4)
void I2C4_ER_IRQHandler (void) ;
void I2C4_ER_IRQHandler (void)
{
	I2Cx_ER_IRQHandler (I2C4, & i2cIo4) ;
}
#endif	// WITH_I2C_4

#endif	// WITH_I2C_MASTER_INTR  ||  WITH_I2C_SLAVE

//----------------------------------------------------------------------
