/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	i2cbasic.h	Master and Slave basic routines
				Header for applications
				For STM32 F1 F2 F4 L1

	When		Who	What
	01/03/18	ac	Creation

	This code is a rework of STMicroelectronics application note AN2824

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined I2CBASIC_H_
#define I2CBASIC_H_
//----------------------------------------------------------------------
#include	"gpiobasic.h"

//----------------------------------------------------------------------
//	I2C API configuration

// To define the supported I2C devices

#define	WITH_I2C_1		0
#define	WITH_I2C_1R		0	// To define if I2C1 remap is wanted
#define	WITH_I2C_2		0

// If one WITH_I2C_x is 1 then i2cArray in i2cbasic.c is not empty and must be defined
#if (WITH_I2C_1 == 1  ||  WITH_I2C_2 == 1)
	#define	WITH_I2C_X			1
#endif

#if (WITH_I2C_2 == 1  &&  ! defined I2C2)
	#error	"I2C not existing"
#endif

#if (WITH_I2C_1 != 1  &&  WITH_I2C_1R == 1)
	#error	"Remap withour I2C"
#endif

// Allows to include/exclude the I2C MASTER API using I2C interrupts
#define	WITH_I2C_MASTER_INTR	1

// Allows to include the I2C slave API. The slave API uses I2C interrupts
#define	WITH_I2C_SLAVE			1

// I2C interrupt priority
#define	I2C_INT_PRIORITY		BSP_IRQPRIOMAX_MINUS(2)

//----------------------------------------------------------------------

typedef enum
{
	I2C_TX		= 1,
	I2C_RX		= 2

} i2cMode_t ;

typedef enum
{
	I2C_ENONE	= 0,	// No error, idle state
	I2C_EFAIL	= 1,	// Some error
	I2C_EOK		= 2,	// Ended without error
	I2C_EAF		= 3,	// Acknowledge fail
	I2C_EARLO	= 4,	// Arbitration lost
	I2C_ETMO	= 5,	// Timeout
	I2C_EBERR	= 6,	// Bus error
	I2C_EOVR	= 7,	// I2C Overflow
	I2C_ESIZE	= 8		// Slave / master buffer size non-coherent:
						// Master stopped receive before slave buffer size
						// Master sent more data than the slave buffer size
} i2cStatus_t ;


//----------------------------------------------------------------------
//	I2C device descriptor

typedef struct
{
	uint16_t		id ;			// Unique descriptor identifier
	uint8_t			periphRstPos ;	// e.g. RCC_APB1RSTR_I2C1RST_Pos
	uint8_t			periphEnPos ;	// e.g. RCC_APB1ENR_I2C1EN_Pos
	gpioPinDesc_t	sda ;			// SDA gpio descriptor
	gpioPinDesc_t	scl ;			// SCL gpio descriptor
	I2C_TypeDef		* pI2c ;		// e.g. I2C1

	IRQn_Type		evIrqn ;		// Event interrupt
	IRQn_Type		erIrqn ;		// Error interrupt
	uint32_t		irqPriority ;	// Same interrupt priority for both interrupts

}  i2cDesc_t ;

//----------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

const i2cDesc_t *	i2cGetDesc			(uint32_t i2cId) ;
void				i2cConfigureMaster	(const i2cDesc_t * pI2cDesc, uint32_t speed) ;
void 				i2cDeactivate 		(const i2cDesc_t * pI2cDesc) ;

// Master polling functions
i2cStatus_t			i2cCheckDevice		(const i2cDesc_t * pI2cDesc, uint32_t slaveAddress) ;
i2cStatus_t 		i2cMasterStart		(const i2cDesc_t * pI2cDesc, uint32_t slaveAddress, i2cMode_t direction) ;
void				i2cMasterStop		(const i2cDesc_t * pI2cDesc) ;
i2cStatus_t			i2cMasterWrite		(const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToWrite) ;
i2cStatus_t			i2cMasterRead		(const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToRead) ;
i2cStatus_t			i2cMasterReadLast	(const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToRead) ;
i2cStatus_t			i2cMasterReadStraight (const i2cDesc_t * pI2cDesc, uint32_t slaveAddress, uint8_t * pBuffer, uint32_t numByteToRead) ;

#if defined WITH_I2C_MASTER_INTR  &&  WITH_I2C_MASTER_INTR == 1
	// MASTER API using I2C Interrupts
	i2cStatus_t		i2cMasterReadIt		(const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToRead, uint8_t slaveAddress) ;
	i2cStatus_t		i2cMasterWriteIt	(const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t numByteToWrite, uint8_t slaveAddress ) ;
#endif	// WITH_I2C_MASTER_INTERRUPT


#if defined WITH_I2C_SLAVE  &&  WITH_I2C_SLAVE == 1
	// SLAVE API using I2C Interrupts
	void			i2cConfigureSlave	(const i2cDesc_t * pI2cDesc, uint32_t ownAddress1) ;
	void			i2cSlaveReadWrite	(const i2cDesc_t * pI2cDesc, uint8_t * pBuffer, uint32_t maxSize) ;
	i2cStatus_t		i2cSlaveResult		(const i2cDesc_t * pI2cDesc, uint32_t * pSize) ;
#endif	// WITH_I2C_SLAVE


#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// I2CBASIC_H_
