/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	spiF429ZI.c		STM32F103 LQFP144 SPI descriptors for spibasic.c

	When		Who	What
	12/04/18	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2020 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined SPIF1xx_H_
#define SPIF1xx_H_
//------------------------------------------------------------------------------

/*
STM32F103xx

		MOSI	MISO	SCK		NSS		APB

SPI_1	PA7		PA6		PA5		PA4		2
		PB5		PB4		PB3=SWO	PA15

SPI_2	PB15	PB14	PB13	PA15	1	8/B/C/D/E/F/G only

SPI_3	PB5		PB4		PB3=SWO	PA15	1	C/D/E/F/G only
*/

/*
	Mode	CPOL	CPHA
	0		0		0
	1		0		1
	2		1		0
	3		1		1

	CPOL	Clock polarity controls the steady state value of the clock when no data is being transferred
			0: Low level when idle state

	CPHA	Controls which edge of the clock locks the data.
			0:first edge, 1:second edge
*/

static	const spiDesc_t	spiDescArray [] =
{
	#if (WITH_SPI_1 == 1)
	{
		10,								// id
		RCC_APB2ENR_SPI1EN_Pos,			// SPI periph pos
		SPI1,							// Pointer to SPI registers
		{	'A',	7,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// MOSI
		{	'A',	6,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// MISO
		{	'A',	5,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// SCK
		{	'A',	4,	AA_REMAP_NONE,	AA_GPIO_MODE_OUTPUT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS select USER/SOFT
//		{	'A',	4,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_MEDIUM },		// NSS select AUTO/HARD
		{	// DMA Tx
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			3,									//	Channel number (1..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			SPI_INT_PRIORITY,					//	Interrupt priority
		},
		{	// DMA Rx
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			2,									//	Channel number (1..7)
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			SPI_INT_PRIORITY,					//	Interrupt priority
		},
	},
	#endif	// WITH_SPI_1

	#if (WITH_SPI_2 == 1  &&  defined SPI2)
	{
		20,
		RCC_APB1ENR_SPI2EN_Pos,
		SPI2,
		{	'B',	15,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// MOSI
		{	'B',	14,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// MISO
		{	'B',	13,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// SCK
		{	'B',	12,	AA_REMAP_NONE,	AA_GPIO_MODE_OUTPUT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			5,									//	Channel number (1..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			SPI_INT_PRIORITY,					//	Interrupt priority
		},
		{	// DMA Rx
			1,									//	DMA number 1 or 2, 0 => unused descriptor
			4,									//	Channel number (1..7)
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			SPI_INT_PRIORITY,					//	Interrupt priority
		},
	},
	#endif	// WITH_SPI_2

	#if (WITH_SPI_3 == 1  &&  defined SPI3)
	{
		30,
		RCC_APB1ENR_SPI3EN_Pos,
		SPI3,
		{	'B',	5,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// MOSI
		{	'B',	4,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// MISO
		{	'B',	3,	AA_REMAP_NONE,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_HIGH },			// SCK
		{	'A',	15,	AA_REMAP_NONE,	AA_GPIO_MODE_OUTPUT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS
		{	// DMA Tx
			2,									//	DMA number 1 or 2, 0 => unused descriptor
			1,									//	Channel number (1..7)
			LL_DMA_DIRECTION_MEMORY_TO_PERIPH,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			SPI_INT_PRIORITY,					//	Interrupt priority
		},
		{	// DMA Rx
			2,									//	DMA number 1 or 2, 0 => unused descriptor
			2,									//	Channel number (1..7)
			LL_DMA_DIRECTION_PERIPH_TO_MEMORY,	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
			SPI_INT_PRIORITY,					//	Interrupt priority
		},
	},
	#endif	// WITH_SPI_3

} ;

//------------------------------------------------------------------------------
#endif	// SPIF1xx_H_
