/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	spibasic.c	Master only, software or hardware SS management, no CRC
				STM32F4xx

	When		Who	What
	12/04/18	ac	Creation
	06/04/20	ac	Add 16 bits data word management

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2020 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"
#include	"gpiobasic.h"
#include	"spibasic.h"

#if defined (STM32F4)
	#include	"stm32f4xx_ll_spi.h"
#elif  defined (STM32F1)
	#include	"stm32f1xx_ll_spi.h"
#elif
	#error "MCU not defined"
#endif


// Used for timing measure with an oscilloscope.
//#define dbgPin1(v)		bspOutput (BSP_OUTPUT1, v) ;	// Debug only
#define dbgPin1(v)											// No debug pulse

//----------------------------------------------------------------------

#if (WITH_SPI_X == 1)
	// Include the proper SPI descriptors
	#if defined (STM32F4)
		#if (defined STM32F405xx || defined STM32F415xx || defined STM32F407xx || defined STM32F417xx ||	\
			 defined STM32F427xx || defined STM32F437xx || defined STM32F429xx || defined STM32F439xx ||	\
			 defined STM32F469xx || defined STM32F479xx || defined STM32F446xx || defined STM32F411xE ||	\
			 defined STM32F401xC || defined STM32F401xE)
			#include "spiF4xx.h"
		#endif
	#else
		// STM32F1
		#if (defined STM32F103x6 || defined STM32F103xB || defined STM32F103xE || defined STM32F103xG)
			#include "spiF1xx.h"
		#endif
	#endif

	// Count of SPI devices in spiDescArray
	STATIC	const uint32_t	spiArrayCount = sizeof (spiDescArray) / sizeof (spiDesc_t) ;
#endif

#define	SPI_DEF_TIMEOUT		10u			// Timeout in kernel tick

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SPI handler structure. The SPI handle is a pointer to this structure.

typedef struct
{
	const spiDesc_t	* pSpiDesc ;	// The descriptor from spiMasterConfigure()
	SPI_TypeDef		* pSpi ;		// SPI1, SPI2...
	spiStatus_t		result ;		// Result of the operation (DMA only)
	aaTaskId_t		taskId ;		// The task to signal at end of operation (DMA only)
	uint16_t		timeout ;		// Transfer timeout (system tick)

} spiIo_t ;

//	One handler structure for each physical SPI peripheral

#if (WITH_SPI_1 == 1)
	spiIo_t		spiIo1 ;
#endif	// WITH_SPI_1

#if (WITH_SPI_2 == 1)
	spiIo_t		spiIo2 ;
#endif	// WITH_SPI_2

#if (WITH_SPI_3 == 1)
	spiIo_t		spiIo3 ;
#endif	// WITH_SPI_3

#if (WITH_SPI_4 == 1  &&  defined SPI4)
	spiIo_t		spiIo4 ;
#endif	// WITH_SPI_4

#if (WITH_SPI_5 == 1  &&  defined SPI5)
	spiIo_t		spiIo5 ;
#endif	// WITH_SPI_5

#if (WITH_SPI_6 == 1  &&  defined SPI6)
	spiIo_t		spiIo6 ;
#endif	// WITH_SPI_6

//----------------------------------------------------------------------
//	Get the I/O descriptor address for an SPI peripheral

static	spiIo_t *	spiGetIoDesc (SPI_TypeDef * pSpi) ;

static	spiIo_t *	spiGetIoDesc (SPI_TypeDef * pSpi)
{
	#if (WITH_SPI_X == 0)
	{
		(void) pSpi ;	// To avoid compiler warning when none SPI is supported
	}
	#endif

	#if (WITH_SPI_1 == 1)
	{
		if (pSpi == SPI1)
		{
			return & spiIo1 ;
		}
	}
	#endif	// WITH_SPI_1

	#if (WITH_SPI_2 == 1)
	{
		if (pSpi == SPI2)
		{
			return & spiIo2 ;
		}
	}
	#endif	// WITH_SPI_2

	#if (WITH_SPI_3 == 1)
	{
		if (pSpi == SPI3)
		{
			return & spiIo3 ;
		}
	}
	#endif	// WITH_SPI_3

	#if (WITH_SPI_4 == 1  &&  defined SPI4)
	{
		if (pSpi == SPI4)
		{
			return & spiIo4 ;
		}
	}
	#endif	// WITH_SPI_4

	#if (WITH_SPI_5 == 1  &&  defined SPI5)
	{
		if (pSpi == SPI5)
		{
			return & spiIo5 ;
		}
	}
	#endif	// WITH_SPI_5

	#if (WITH_SPI_6 == 1  &&  defined SPI6)
	{
		if (pSpi == SPI6)
		{
			return & spiIo6 ;
		}
	}
	#endif	// WITH_SPI_6

	AA_ASSERT (0) ;
	return NULL ;
}

//----------------------------------------------------------------------
//	Return SPI descriptor address from an index
//	The return value is an opaque handle for the application

spiStatus_t		spiGetDesc	(uint32_t id, const spiDesc_t ** ppSpiDesc)
{
	#if (WITH_SPI_X == 1)
		uint32_t	ii ;

		for (ii = 0 ; ii < spiArrayCount ; ii++)
		{
			if (id == spiDescArray [ii].id)
			{
				* ppSpiDesc = & spiDescArray [ii] ;
				return SPI_ENONE ;
			}
		}
	#else
		(void) id ;
		(void) ppSpiDesc ;
	#endif

	return SPI_EFAIL ;
}

//----------------------------------------------------------------------
//	Set / Reset Slave Select pin
//	Use NSS pin of SPI descriptor
//	SPI slave select is active low: so bSelect 1 => NSS 0

void	spiSlaveSelect (spiHandle_t hdl, uint32_t bSelect)
{
	spiIo_t		* pSpiIo = (spiIo_t *) hdl ;

	if ((pSpiIo->pSpiDesc->pinNss.port != 0u)  &&  (pSpiIo->pSpiDesc->pinNss.port != '0'))
	{
		gpioPinSet (& pSpiIo->pSpiDesc->pinNss, (bSelect & 1u) ^ 1u) ;
	}
}

//----------------------------------------------------------------------
//	Find baud rate control value from bits/s rate
//	The return value is the SPI_CR1 BR[2:0] field value

uint32_t	spiSpeedToBr (const spiDesc_t * pSpiDesc, uint32_t speed)
{
	uint32_t	br ;
	uint32_t	clock ;

	// Max allowed clock is Fpclk / 2
	clock = rccGetClockFrequency (pSpiDesc->pSpi) / 2 ;

	for (br = 0u ; br < 7u ; br++)
	{
		if (speed >= clock)
		{
			break ;
		}
		clock /= 2u ;
	}

	return br ;
}

//----------------------------------------------------------------------
//	Find bit/s rate from BR control register value

uint32_t	spiBrToSpeed (const spiDesc_t * pSpiDesc, uint32_t br)
{
	uint32_t	clock ;

	clock = rccGetClockFrequency (pSpiDesc->pSpi) ;
	return clock >> ((br & 0x7u) + 1u) ;
}

//----------------------------------------------------------------------
// Set some options for master SPI
// To use after spiMasterConfigure()

spiStatus_t		spiMasterSetOption		(spiHandle_t hdl, uint32_t option, uintptr_t value)
{
	spiIo_t			* pSpiIo = (spiIo_t *) hdl ;
//	SPI_TypeDef		* pSpi = pSpiIo->pSpi ;

	switch (option)
	{
		case SPI_OPT_TIMEOUT:
			pSpiIo->timeout = (value > 0xFFFFu) ? (uint8_t) 0xFFFFu : (uint8_t) value ;
			break ;

		default:
			return SPI_EARG ;
	}
	return SPI_ENONE ;
}

//----------------------------------------------------------------------
// DMA callback to manage SPI end of DMA, TC or error

static	void spiCallback (uint32_t event, uint32_t dir, uintptr_t arg) ;

static	void spiCallback (uint32_t event, uint32_t dir, uintptr_t arg)
{
	spiIo_t		* pSpiIo = (spiIo_t *) arg ;

dbgPin1 (1) ;
	if (event == DMA_EV_ERROR)
	{
		if (pSpiIo->taskId != AA_INVALID_TASK)
		{
			pSpiIo->result = SPI_EDMA ;
			aaSignalSend (pSpiIo->taskId, AA_SIGNAL_IO) ;
			pSpiIo->taskId = AA_INVALID_TASK ;	// Prevent multiple wake up
		}
	}
	else if (event == DMA_EV_COMPLETE)
	{
		if (pSpiIo->taskId != AA_INVALID_TASK)
		{
			pSpiIo->result = SPI_ENONE ;
dbgPin1 (0) ;
			aaSignalSend (pSpiIo->taskId, AA_SIGNAL_IO) ;
			pSpiIo->taskId = AA_INVALID_TASK ;	// Prevent multiple wake up
dbgPin1 (1) ;
		}
	}
	else
	{
	}

	(void) dir ;
}

//----------------------------------------------------------------------
//	Configure one DMA stream for SPI

static	void	spiDmaConfigure (const dmaDesc_t * pDmaDesc, spiIo_t * pSpiIo) ;

static	void	spiDmaConfigure (const dmaDesc_t * pDmaDesc, spiIo_t * pSpiIo)
{
	dma_t				* pDma ;
	dmaChannel_t		* pChannel ;
	SPI_TypeDef 		* pSpi = pSpiIo->pSpi ;
	uint32_t			chanNum ;
	uint32_t			reg ;

	if (pDmaDesc->dmaNum == 0u)
	{
		return ;	// No DMA to configure
	}

	pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
	chanNum		= DMA_GETCHANNEL (pDmaDesc) ;
	pChannel	= dmaChannelAddressRaw (pDma, chanNum) ;

	// Enable DMA clock
	dmaEnableClock (pDma) ;

	// Disable channel
	dmaChannelResetRaw (pDma, chanNum) ;

	// Configure the DMA/channel functional parameters
	reg = pChannel->CCR ;
	if ((pSpi->CR1 & SPI_CR1_DFF) == 0u)
	{
		reg |= LL_DMA_PDATAALIGN_BYTE | LL_DMA_MDATAALIGN_BYTE ;
	}
	else
	{
		reg |= LL_DMA_PDATAALIGN_HALFWORD | LL_DMA_MDATAALIGN_HALFWORD ;
	}
	pChannel->CCR = reg | (pDmaDesc->config       |
						LL_DMA_PRIORITY_HIGH      |
						LL_DMA_PERIPH_NOINCREMENT |
						LL_DMA_MEMORY_INCREMENT) ;

	// Set peripheral address
	pChannel->CPAR = (uint32_t) & pSpi->DR ;

	// Configure NVIC for DMA transfer complete/error interrupts
	dmaSetIrq (pDmaDesc) ;
	dmaSetCallback (pDmaDesc, spiCallback, (uintptr_t) pSpiIo) ;

	// Enable DMA error interrupt
	pChannel->CCR |= DMA_CCR_TEIE ;
}

//----------------------------------------------------------------------
//	Initialize SPI device: SPI master mode, polling or DMA, no CRC
//	Speed is a wish, because this SPI only allows a very limited number of speeds
//	Default exchange timeout is SPI_DEF_TIMEOUT ticks

spiStatus_t		spiMasterConfigure	(const spiDesc_t * pSpiDesc, uint32_t speed, uint32_t flags, spiHandle_t * pHdl)
{
	LL_SPI_InitTypeDef	SPI_InitStruct ;
	spiIo_t				* pSpiIo ;
	SPI_TypeDef			* pSpi = pSpiDesc->pSpi ;

	pSpiIo = spiGetIoDesc (pSpi) ;
	if (pSpiIo == NULL)
	{
		return SPI_EARG ;
	}
	pSpiIo->pSpi		= pSpi ;
	pSpiIo->pSpiDesc	= pSpiDesc ;
	pSpiIo->taskId		= AA_INVALID_TASK ;
	pSpiIo->timeout		= SPI_DEF_TIMEOUT ;

	// Enable the peripheral clock for SPI
	rccEnableClock (pSpi, 1u << pSpiDesc->periphPos) ;

	// Reset peripheral
	rccResetPeriph (pSpi, 1u << pSpiDesc->periphPos) ;

	// Configure pins
	gpioConfigurePin (& pSpiDesc->pinMosi) ;
	gpioConfigurePin (& pSpiDesc->pinMiso) ;

	// Set SCK value before pin init: avoid glitch if there is external pull up/down
	if ((flags & SPI_MODE_2) == 0)
	{
		gpioConfigureAndSetPin (& pSpiDesc->pinSck, 0) ;	// SPI_MODE_0 or SPI_MODE_1 : Low level when idle state
	}
	else
	{
		gpioConfigureAndSetPin (& pSpiDesc->pinSck, 1) ;	// SPI_MODE_2 or SPI_MODE_3 : High level when idle state
	}

	// Set NSS value before pin init: avoid glitch if there is external pull up
	gpioConfigureAndSetPin (& pSpiDesc->pinNss, 1) ;

	SPI_InitStruct.Mode				= LL_SPI_MODE_MASTER ;
	SPI_InitStruct.NSS				= ((flags & SPI_NSS_MASK) == SPI_NSS_USER) ? LL_SPI_NSS_SOFT : LL_SPI_NSS_HARD_OUTPUT ;
	SPI_InitStruct.BaudRate			= spiSpeedToBr (pSpiDesc, speed) << SPI_CR1_BR_Pos ;
	SPI_InitStruct.CRCCalculation	= LL_SPI_CRCCALCULATION_DISABLE ;
	SPI_InitStruct.CRCPoly			= 10u ;

	// Set SPI polarity / phase
	switch (flags & SPI_MODE_MASK)
	{
		default:
		case SPI_MODE_0:		// Default
			SPI_InitStruct.ClockPolarity	= LL_SPI_POLARITY_LOW ;
			SPI_InitStruct.ClockPhase		= LL_SPI_PHASE_1EDGE ;
			break ;

		case SPI_MODE_1:
			SPI_InitStruct.ClockPolarity	= LL_SPI_POLARITY_LOW ;
			SPI_InitStruct.ClockPhase		= LL_SPI_PHASE_2EDGE ;
			break ;

		case SPI_MODE_2:
			SPI_InitStruct.ClockPolarity	= LL_SPI_POLARITY_HIGH ;
			SPI_InitStruct.ClockPhase		= LL_SPI_PHASE_1EDGE ;
			break ;

		case SPI_MODE_3:
			SPI_InitStruct.ClockPolarity	= LL_SPI_POLARITY_HIGH ;
			SPI_InitStruct.ClockPhase		= LL_SPI_PHASE_2EDGE ;
			break ;
	}

	// Set transfer mode
	switch (flags & SPI_DIR_MASK)
	{
		default:
		case SPI_DIR_FULL_DUPLEX:		// Default
			SPI_InitStruct.TransferDirection	= LL_SPI_FULL_DUPLEX ;
			break ;
/*
		case SPI_DIR_RX_ONLY:
			SPI_InitStruct.TransferDirection	= LL_SPI_SIMPLEX_RX ;
			break ;

		case SPI_DIR_HALF_RX:
			SPI_InitStruct.TransferDirection	= LL_SPI_HALF_DUPLEX_RX ;
			break ;
*/
		case SPI_DIR_HALF_TX:
			SPI_InitStruct.TransferDirection	= LL_SPI_HALF_DUPLEX_TX ;
			break ;
	}

	// Bit order default: MSB
	SPI_InitStruct.BitOrder = ((flags & SPI_LSB_FIRST) != 0u) ? LL_SPI_LSB_FIRST : LL_SPI_MSB_FIRST ;

	// Data width default: 8
	SPI_InitStruct.DataWidth = (flags & SPI_BITS_16) != 0u ? LL_SPI_DATAWIDTH_16BIT : LL_SPI_DATAWIDTH_8BIT ;

	LL_SPI_Init (pSpi, & SPI_InitStruct) ;

	// Configure DMA
	spiDmaConfigure (& pSpiIo->pSpiDesc->dmaTx, pSpiIo) ;
	if (LL_SPI_GetTransferDirection (pSpi) == LL_SPI_FULL_DUPLEX)
	{
		// Initialize RX DMA only if full duplex mode
		spiDmaConfigure (& pSpiIo->pSpiDesc->dmaRx, pSpiIo) ;
	}

	* pHdl = (spiHandle_t) pSpiIo ;
	return SPI_ENONE ;
}

//----------------------------------------------------------------------
//	Data exchange with polling
//	Support only full duplex, or half duplex TX
//	pDataOut or pDataIn can be NULL if they are not necessary
//	Count is an item count (not a byte count)

spiStatus_t spiMasterExchange (spiHandle_t hdl, void * pDataOut, void * pDataIn, uint32_t count)
{
	uint32_t		ii ;
	spiIo_t			* pSpiIo = (spiIo_t *) hdl ;
	SPI_TypeDef		* pSpi = pSpiIo->pSpi ;
	
dbgPin1 (1) ;
	// Enable SPI
	pSpi->CR1 |= SPI_CR1_SPE ;

dbgPin1 (0) ;
	// Send/receive data
	if ((pSpi->CR1 & SPI_CR1_DFF) == 0u)
	{
		// 8 bits data
		uint8_t		data;
		uint8_t		* pOut = (uint8_t *) pDataOut ;
		uint8_t		* pIn  = (uint8_t *) pDataIn ;

		if ((pSpi->CR1 & (SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE)) == 0u)
		{
			// RX + TX
			for (ii = 0 ; ii < count ; ii++)
			{
				// Write data to send
				data = 0 ;
				if (pOut != NULL)
				{
					data = * pOut++ ;
				}
				pSpi->DR = data ;

				while ((pSpi->SR & SPI_SR_TXE) == 0)
				{
				}

				// Get received data
				while ((pSpi->SR & SPI_SR_RXNE) == 0)
				{
				}

				data = (uint8_t) pSpi->DR ;
				if (pIn != NULL)
				{
					* pIn++ = data ;
				}
			}
		}
		else
		{
			// TX only
			for (ii = 0 ; ii < count ; ii++)
			{
				// Write data to send
				pSpi->DR = * pOut++ ;

				while ((pSpi->SR & SPI_SR_TXE) == 0)
				{
				}
			}
		}

	}
	else
	{
		// 16 bits data
		uint16_t	data ;
		uint16_t	* pOut = (uint16_t *) pDataOut ;
		uint16_t	* pIn  = (uint16_t *) pDataIn ;

		if ((pSpi->CR1 & (SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE)) == 0u)
		{
			// RX + TX
			for (ii = 0 ; ii < count ; ii++)
			{
				// Write data to send
				data = 0 ;
				if (pOut != NULL)
				{
					data = * pOut++ ;
				}
				pSpi->DR = data ;

				while ((pSpi->SR & SPI_SR_TXE) == 0)
				{
				}

				// Get received data
				while ((pSpi->SR & SPI_SR_RXNE) == 0)
				{
				}

				data = (uint16_t) pSpi->DR ;
				if (pIn != NULL)
				{
					* pIn++ = data ;
				}
			}
		}
		else
		{
			// TX only
			for (ii = 0 ; ii < count ; ii++)
			{
				// Write data to send
				pSpi->DR = * pOut++ ;

				while ((pSpi->SR & SPI_SR_TXE) == 0)
				{
				}
			}
		}
	}
dbgPin1 (1) ;

	// Wait for end of transmission
	while ((pSpi->SR & SPI_SR_BSY) != 0u)
	{
	}
	
	// Disable SPI
	pSpi->CR1 &= ~SPI_CR1_SPE ;
dbgPin1 (0) ;

	return SPI_ENONE ;
}

//----------------------------------------------------------------------
//	pDataOut is a pointer to Tx data, can't be NULL
//	pDataIn  is a pointer to Rx data:
//		Full duplex: can be NULL if there is nothing to receive
//		TX only: must be NULL
//	Count is an item count (not a byte count)

spiStatus_t	spiMasterExchangeDma (spiHandle_t hdl, void * pDataOut, void * pDataIn, uint32_t count)
{
	spiIo_t				* pSpiIo = (spiIo_t *) hdl ;
	SPI_TypeDef			* pSpi = pSpiIo->pSpi ;
	dma_t				* pDmaTx ;
	dma_t				* pDmaRx ;
	dmaChannel_t		* pChannelTx ;
	dmaChannel_t		* pChannelRx = NULL ;	// Avoid GCC false message "may be used uninitialized in this function"
	spiStatus_t			sts ;
	aaError_t			res ;

dbgPin1 (1) ;
	// pDataOut can't be NULL
	if (pDataOut == NULL)
	{
		AA_ASSERT (0) ;
		return SPI_EARG ;
	}

	// If TX only mode, then pDataIn must be NULL
	if (LL_SPI_GetTransferDirection (pSpi) != LL_SPI_FULL_DUPLEX)
	{
		if (pDataIn != NULL)
		{
			AA_ASSERT (0) ;
			return SPI_EARG ;
		}
	}

	pSpiIo->taskId = aaTaskSelfId () ;
	aaSignalClear (AA_SELFTASKID, AA_SIGNAL_IO) ;

	// Configure and start RX DMA
	if (pDataIn != NULL)
	{
		pDmaRx = DMA_GETDMA (pSpiIo->pSpiDesc->dmaRx.dmaNum) ;
		pChannelRx = dmaChannelAddressRaw (pDmaRx, pSpiIo->pSpiDesc->dmaRx.dmaChan)  ;
		pChannelRx->CCR	&= ~DMA_CCR_EN ;
		dmaClearFlagsRaw (pDmaRx, pSpiIo->pSpiDesc->dmaRx.dmaChan, DMA_FLAG_ALLIF) ;
		pChannelRx->CNDTR	= count ;
		pChannelRx->CMAR	= (uint32_t) pDataIn ;
		pChannelRx->CCR		|= DMA_CCR_TCIE ;	// Enable RX DMA IT transfer complete
		pChannelRx->CCR		|= DMA_CCR_EN ;

		LL_SPI_EnableDMAReq_RX (pSpi) ;
	}

	// Configure and start TX DMA
	pDmaTx = DMA_GETDMA (pSpiIo->pSpiDesc->dmaTx.dmaNum) ;
	pChannelTx = dmaChannelAddressRaw (pDmaTx, pSpiIo->pSpiDesc->dmaTx.dmaChan) ;
	pChannelTx->CCR	&= ~DMA_CCR_EN ;
	dmaClearFlagsRaw (pDmaTx, pSpiIo->pSpiDesc->dmaTx.dmaChan, DMA_FLAG_ALLIF) ;
	pChannelTx->CNDTR	= count ;
	pChannelTx->CMAR	= (uint32_t) pDataOut ;
	if (pDataIn == NULL)
	{
		// Tx only, enable TX DMA IT transfer complete
		pChannelTx->CCR |= DMA_CCR_TCIE ;
	}
	pChannelTx->CCR |= DMA_CCR_EN ;

	LL_SPI_EnableDMAReq_TX (pSpi) ;

	// Enable SPI
	pSpi->CR1 |= SPI_CR1_SPE ;

	// Wait end of DMA
dbgPin1 (0) ;
	res = aaSignalWait (AA_SIGNAL_IO, NULL, AA_SIGNAL_AND, pSpiIo->timeout) ;
dbgPin1 (0) ;
	if (res == AA_ENONE)
	{
		sts = SPI_ENONE ;
	}
	else if (res == AA_ETIMEOUT)
	{
		sts = SPI_ETIMEOUT ;
	}
	else
	{
		sts = SPI_EFAIL ;	// Other error
	}
	if (pSpiIo->result == SPI_EDMA)
	{
		sts = SPI_EDMA ;
	}
	pSpiIo->taskId = AA_INVALID_TASK ;

	// Wait for end of transmission
	if (sts == SPI_ENONE)
	{
		while ((pSpi->SR & SPI_SR_BSY) != 0u)
		{
		}
	}
dbgPin1 (1) ;

	// Disable DMA
	pChannelTx->CCR &= ~DMA_CCR_EN ;
	if (pDataIn != NULL)
	{
		pChannelRx->CCR &= ~DMA_CCR_EN ;
	}

	// Disable SPI
	pSpi->CR1 &= ~SPI_CR1_SPE ;

	LL_SPI_DisableDMAReq_TX (pSpi) ;
	LL_SPI_DisableDMAReq_RX (pSpi) ;

	// Clear SPI SR flags
	{
		volatile uint32_t	reg ;
		reg = pSpi->DR ;
		reg = pSpi->SR ;
		(void) reg ;
	}

dbgPin1 (0) ;
	return sts ;
}

//----------------------------------------------------------------------
