/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	spibasic.h	Master SPI polling or DMA (STM32F1)

	When		Who	What
	12/04/18	ac	Creation

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2020 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined SPIBASIC_H_
#define SPIBASIC_H_
//----------------------------------------------------------------------
#include	"gpiobasic.h"
#include	"dmabasic.h"

//	Set to 1 for supported SPI devices.
//	This allows to insert proper SPI descriptors from spiDescArray in spiFXxx.h

#define	WITH_SPI_1		1
#define	WITH_SPI_2		0
#define	WITH_SPI_3		0
#define	WITH_SPI_4		0
#define	WITH_SPI_5		0
#define	WITH_SPI_6		0

// If one WITH_SPI_x is 1 then spiDescArray in spiFxxx.h is not empty and must be defined
#if (WITH_SPI_1 == 1  ||  WITH_SPI_2 == 1  ||  WITH_SPI_3 == 1  ||  \
	 WITH_SPI_4 == 1  ||  WITH_SPI_5 == 1  ||  WITH_SPI_6 == 1  )
	#define	WITH_SPI_X		1
#endif

// Check if selected SPI are defined for this MCU
#if (WITH_SPI_2 == 1  &&  ! defined SPI2)	||	\
	(WITH_SPI_3 == 1  &&  ! defined SPI3)	||	\
	(WITH_SPI_4 == 1  &&  ! defined SPI4)	||	\
	(WITH_SPI_5 == 1  &&  ! defined SPI5)	||	\
	(WITH_SPI_6 == 1  &&  ! defined SPI6)
	#error	"SPI not existing"
#endif

//----------------------------------------------------------------------
// Flags for spiMasterConfigure()
// Choose one in each group

// Word size: flag value 8 or 16. A value of 0 is converted to 8.
#define	SPI_BITS_MASK			0x0000003F		// Internal use
#define	SPI_BITS_8				0x00000008		// 8 bits data word (default)
#define	SPI_BITS_16				0x00000010		// 16 bits data word

#define	SPI_MODE_MASK			0x00000300		// Internal use
#define	SPI_MODE_0				0x00000000		// CPOL:0	CPHA:0 (Default)
#define	SPI_MODE_1				0x00000100		// CPOL:0	CPHA:1
#define	SPI_MODE_2				0x00000200		// CPOL:1	CPHA:0
#define	SPI_MODE_3				0x00000300		// CPOL:1	CPHA:1

#define	SPI_NSS_MASK			0x00001000		// Internal use
#define	SPI_NSS_AUTO			0x00000000		// NSS pin managed automatically by hardware (default)
												// BEWARE: Automatic NSS output need a pullup, even if the pin is flagged push-pull
#define	SPI_NSS_USER			0x00001000		// NSS pin is user managed with spiSlaveSelect()(ignored by the hardware)

#define	SPI_MSB_MASK			0x00100000		// Internal use
#define	SPI_MSB_FIRST			0x00000000		// Data is transfered with the MSB first (default)
#define	SPI_LSB_FIRST			0x00100000		// Data is transfered with the LSB first

#define	SPI_DIR_MASK			0x03000000		// Internal use
#define	SPI_DIR_FULL_DUPLEX		0x00000000		// Full-Duplex mode. Rx and Tx transfer on 2 lines (default)
//#define	SPI_DIR_RX_ONLY		0x01000000		// Receive only transfer on MISO
//#define	SPI_DIR_HALF_RX		0x02000000		// Half-Duplex Rx mode. Rx transfer on MOSI
#define	SPI_DIR_HALF_TX			0x03000000		// Half-Duplex Tx mode. Tx transfer on MOSI

//----------------------------------------------------------------------
//	Options to use with spiMasterOption()

#define	SPI_OPT_TIMEOUT			0x03			// Timeout of exchange, the value is a number of tick

//----------------------------------------------------------------------
//	Default priority for DMA interrupt, used in SPI descriptors

#define	SPI_INT_PRIORITY		(BSP_MAX_INT_PRIO+2)

typedef enum
{
	SPI_ENONE		= 0,		// No error
	SPI_EARG		= 1,
	SPI_EFAIL		= 2,
//	SPI_ENOTINIT	= 3,
	SPI_ETIMEOUT	= 4,
	SPI_EDMA		= 5

} spiStatus_t ;

typedef struct spiDesc_t
{
	uint16_t		id ;			// Descriptor id
	uint16_t		periphPos ;		// SPI periph pos
	SPI_TypeDef		* pSpi ;		// Pointer to SPI registers
	gpioPinDesc_t	pinMosi ;		// MOSI pin I/O descriptor
	gpioPinDesc_t	pinMiso ;		// MISO pin I/O descriptor
	gpioPinDesc_t	pinSck ;		// Clock pin I/O descriptor
	gpioPinDesc_t	pinNss ;		// Slave select pin I/O descriptor
	dmaDesc_t		dmaTx ;
	dmaDesc_t		dmaRx ;

} spiDesc_t ;

// The SPI handle returned by spiMasterConfigure()
typedef void * spiHandle_t ;


#ifdef __cplusplus
extern "C" {
#endif

spiStatus_t			spiGetDesc				(uint32_t id, const spiDesc_t ** ppSpiDesc) ;
spiStatus_t			spiMasterConfigure		(const spiDesc_t * pSpiDesc, uint32_t speed, uint32_t flags, spiHandle_t * pHdl) ;
spiStatus_t			spiMasterSetOption		(spiHandle_t hdl, uint32_t option, uintptr_t value) ;
void				spiSlaveSelect			(spiHandle_t hdl, uint32_t bSelect) ;

spiStatus_t 		spiMasterExchange		(spiHandle_t hdl, void * pDataOut, void * pDataIn, uint32_t count) ;

// The function spiMasterExchangeDma() use the AA_SIGNAL_IO task signal
spiStatus_t			spiMasterExchangeDma	(spiHandle_t hdl, void * pDataOut, void * pDataIn, uint32_t count) ;

uint32_t			spiSpeedToBr			(const spiDesc_t * pSpiDesc, uint32_t speed) ;
uint32_t			spiBrToSpeed			(const spiDesc_t * pSpiDesc, uint32_t br) ;

#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// SPIBASIC_H_
