/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	timbasic.c	Basic routines for timers

	When		Who	What
	08/03/18	ac	Creation
	10/05/20	ac	Updated for the full F103 family
	12/07/21	ac	Add TIM_SR_IMASK in interrupt handlers
					Add input capture management
	26/07/21	ac	Change timPwmReset() to timReset()
					Add slave modes (Reset Gated, Trigger...)
					Add "pTim->SR = 0" to timStart() to clear interrupt flags
					Add timClearDmaRequest()
					Use rccEnableTimClock(), rccResetTim()
	12/10/21	ac	Better prescaler computing in timTimeBaseInit() for low frequencies

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aakernel.h"
#include	"timbasic.h"
#include	"gpiobasic.h"


//------------------------------------------------------------------------------
//	Descriptor array for this processor

// Include the proper timer descriptors
#if (WITH_TIM_X == 1)
	#if (defined STM32F1)
		#include "timF1xx.h"
	#endif

	// Count of timers in timDescArray
	STATIC const	uint32_t	timAarrayCount = sizeof (timDescArray) / sizeof (timDesc_t) ;
#endif

//------------------------------------------------------------------------------
//	Every timer have a descriptor for interrupt callback management
//	Only 1 timer interrupt descriptor in use at any time for each timer

// Number of the highest timers in the MCU		THIS MUST BE UPDATED IF YOU CHANGE MCU
#if defined (STM32F103x6)
	#define	MAX_TIMER	3
#elif defined (STM32F103xB)
	#define	MAX_TIMER	4
#elif defined (STM32F103xE)
	#define	MAX_TIMER	8
#elif defined (STM32F103xG)
	#define	MAX_TIMER	14
#endif

typedef struct timCallback_s
{
	const timDesc_t	* pTimDesc ;
	fnCallback_t	pCallback ;
	uintptr_t		arg ;

} timCallback_t ;

// Do not define timCallbackArray if none timer is used
#if (WITH_TIM_X	== 1)

timCallback_t	timCallbackArray [MAX_TIMER] = {0} ;
#endif

//------------------------------------------------------------------------------
//	Convert channel number 1..4  and complementary channels to LL number LL_TIM_CHANNEL_CH1..LL_TIM_CHANNEL_CH4
//	LL_TIM_CHANNEL_CH1  == TIM_CCER_CC1E  == (0x1U << TIM_CCER_CC1E_Pos)
//	LL_TIM_CHANNEL_CH1N == TIM_CCER_CC1NE == (0x1U << TIM_CCER_CC1NE_Pos)

static	uint32_t	timGetLlChannel (const timDesc_t * pTimDesc, uint32_t channel)
{
	if (channel == 1u  &&  (pTimDesc->capabilities & TIM_CAP_CHAN_1) != 0u)
	{
		return LL_TIM_CHANNEL_CH1 ;
	}

	if (channel == 2u  &&  (pTimDesc->capabilities & TIM_CAP_CHAN_2) != 0u)
	{
		return LL_TIM_CHANNEL_CH2 ;
	}

	if (channel == 3u  &&  (pTimDesc->capabilities & TIM_CAP_CHAN_3) != 0u)
	{
		return LL_TIM_CHANNEL_CH3 ;
	}

	if (channel == 4u  &&  (pTimDesc->capabilities & TIM_CAP_CHAN_4) != 0u)
	{
		return LL_TIM_CHANNEL_CH4 ;
	}

	// Complementary channels
	if (channel == (1u + TIM_CHxN_OFFSET))
	{
		return LL_TIM_CHANNEL_CH1N ;
	}

	if (channel == (2u + TIM_CHxN_OFFSET))
	{
		return LL_TIM_CHANNEL_CH2N ;
	}

	if (channel == (3u + TIM_CHxN_OFFSET))
	{
		return LL_TIM_CHANNEL_CH3N ;
	}


	return 0u ;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//	Return timer descriptor address from timer descriptor id

timError_t		timGetDesc (uint32_t id, const timDesc_t ** ppTimDesc)
{
	#if (WITH_TIM_X == 1)
		uint32_t	ii ;
		const timDesc_t	* pTimDesc = & timDescArray [0] ;

		for (ii = 0u ; ii < timAarrayCount ; ii++)
		{
			if (id == pTimDesc->id)
			{
				* ppTimDesc = pTimDesc ;
				return TIM_ENONE ;
			}
			pTimDesc++ ;
		}
	#else
		(void) id ;
		(void) ppTimDesc ;
	#endif

	return TIM_EFAIL ;	// Id not found
}

//------------------------------------------------------------------------------
//	Configure a timer for external clock, and/or slave modes
//	External clock mode 1: The clock is TRGI => not compatible with slaves modes (Reset, Gated, Trigger)
//	External clock mode 2: The clock is ETR  => compatible with slave modes (Reset, Gated, Trigger)
//	Does not set the TIM_SMCR_TS field (must use timSetTriggerInput())
//	Does not set the ARR register (must use timSetReload())

//	The input parameters is used to configure the required clock input I/O.
// 	input is one of:
//		0					=> no external clock input to configure
//		TIM_ECLK_TI1F_ED	|
//		TIM_ECLK_TI1FP1	 	 > imply External mode 1 or slave mode
//		TIM_ECLK_TI2FP2		|
//		TIM_ECLK_ETRF		=> imply External Clock Mode 2, possibly combined with slave mode

//	Allowed modes combinations:
//							ECE		SMS
//	ETR						1		0	External Clock Mode 2
//	ETR+SLV	mode			1		xxx	External Clock Mode 2 + Slave mode
//	SLV mode				0		xxx Slave mode
//	TI only (!ETR+!SLV)		0		111	External Clock Mode 1

//	prescaler is the real prescaler value (not the value -1)

timError_t	timExtClockInit (const timDesc_t * pTimDesc, uint32_t input, uint32_t prescaler, uint32_t flags)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;
	uint32_t		llChannel ;

	// Enable timer clock
	rccEnableClock (pTim, 1u << pTimDesc->timEnPos) ;

	// Reset timer
	rccResetPeriph (pTim, 1u << pTimDesc->timRstPos) ;

// TODO: This does not take into account the TIM_ICMODE_TISWAP flag
	// Configure input pin
	if (input == TIM_ECLK_TI1FP1  ||  input == TIM_ECLK_TI1F_ED)
	{
		llChannel = LL_TIM_CHANNEL_CH1 ;
		gpioConfigurePin (& pTimDesc->pins [TIM_PIN_CH1]) ;
	}

	if (input == TIM_ECLK_TI2FP2)
	{
		llChannel = LL_TIM_CHANNEL_CH2 ;
		gpioConfigurePin (& pTimDesc->pins [TIM_PIN_CH2]) ;
	}

	if (input == TIM_ECLK_ETRF)
	{
		gpioConfigurePin (& pTimDesc->pins [TIM_PIN_ETR]) ;
	}

	// Configure input channel
	if (input == TIM_ECLK_TI1FP1  ||  input == TIM_ECLK_TI2FP2)
	{
		// Configure the channel as input in CCMR1_CCxS : DIRECTTI(01) / INDIRECTTI(10) / TRC(11)
		if ((flags & TIM_ICMODE_TRC) != 0)
		{
			LL_TIM_IC_SetActiveInput (pTim, llChannel, LL_TIM_ACTIVEINPUT_TRC) ;
		}
		else if ((flags & TIM_ICMODE_TISWAP) != 0)
		{
			LL_TIM_IC_SetActiveInput (pTim, llChannel, LL_TIM_ACTIVEINPUT_INDIRECTTI) ;
		}
		else
		{
			LL_TIM_IC_SetActiveInput (pTim, llChannel, LL_TIM_ACTIVEINPUT_DIRECTTI) ;
		}

		// CCER_CCxP : Reset value is RISING
		#if defined (LL_TIM_IC_POLARITY_BOTHEDGE)
			if ((flags & (TIM_ECLK_EDGE_RISING | TIM_ECLK_EDGE_FALLING)) == (TIM_ECLK_EDGE_RISING | TIM_ECLK_EDGE_FALLING))
			{
				LL_TIM_IC_SetPolarity (pTim, llChannel, LL_TIM_IC_POLARITY_BOTHEDGE) ;
			}
			else
		#endif
		if ((flags & TIM_ECLK_EDGE_FALLING) != 0u)
		{
			LL_TIM_IC_SetPolarity (pTim, llChannel, LL_TIM_IC_POLARITY_FALLING) ;
		}
		else
		{
			// Nothing, Reset value is RISING
		}
	}

	// CCMR1_1C1F : Input capture filter. Reset value is LL_TIM_IC_FILTER_FDIV1
	//LL_TIM_IC_SetFilter (pTim, llChannel, LL_TIM_IC_FILTER_FDIV1) ;

	// Configure the external clock mode and/or slave mode
	if (input == TIM_ECLK_ETRF)
	{
		// This is reset configuration
		LL_TIM_ConfigETR (pTim, LL_TIM_ETR_POLARITY_NONINVERTED, LL_TIM_ETR_PRESCALER_DIV1, LL_TIM_ETR_FILTER_FDIV1);
// TODO  Allows to configure ETR : prescaler, filter

		// Select SMCR_SMS : external clock mode 2, set SMS to 0 and ECE to 1
		LL_TIM_EnableExternalClock (pTim) ;
	}
	if ((flags & TIM_SLV_MASK_) != 0)
	{
		// Slave mode required, set SMS with slave mode
		LL_TIM_SetSlaveMode (pTim, (flags & TIM_SLV_MASK_) >> 8);
	}
	else if (input != TIM_ECLK_ETRF)
	{
		// No ETRF and no slave mode, so external clock mode 1
		LL_TIM_SetClockSource (pTim, LL_TIM_CLOCKSOURCE_EXT_MODE1) ;
	}
	else
	{
		// Nothing to do
	}

	LL_TIM_SetPrescaler (pTim, prescaler - 1u) ;

	// DIER_TIE : reset value is Disabled trigger interrupt (TIE)
	//LL_TIM_DisableIT_TRIG (pTim) ;

	// Reset value is Disabled trigger DMA request (TDE)
	//LL_TIM_DisableDMAReq_TRIG (pTim) ;

	// Reset value is LL_TIM_TRGO_RESET
	//LL_TIM_SetTriggerOutput (pTim, LL_TIM_TRGO_RESET) ;

	// Reset value is Disabled Master/Slave mode
	//LL_TIM_DisableMasterSlaveMode (pTim) ;

	return TIM_ENONE ;
}

//------------------------------------------------------------------------------
//	Initialize a timer for time base or PWM
//	prescaler is the PSC+1 value to get the timer clock frequency. if 0 then timTimeBaseInit() compute the prescaler value.
//	if the flag TIM_INIT_FORCE_ARR is specified
//		timeBaseFrequency is exact ARR value (the value is not decremented before writing to register)
//	else
//		timeBaseFrequency is the desired time base or PWM frequency (in Hz), which is used to compute ARR value

//	The prescaler computing may be lengthy.
//	If you cannot afford this time, specify the PSC and ARR values in the parameters and use the TIM_INIT_FORCE_ARR flag

void  timTimeBaseInit (const timDesc_t * pTimDesc, uint32_t prescaler, uint32_t timeBaseFrequency, uint32_t flags)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;
	uint32_t		clock = rccGetTimerClockFrequency (pTim) ;

	// Enable timer clock
	rccEnableClock (pTimDesc->pTim, 1u << pTimDesc->timEnPos) ;
	
	// Reset timer
	rccResetPeriph (pTimDesc->pTim, 1u << pTimDesc->timRstPos) ;

	// Time base configuration
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Set counter mode. Reset value is LL_TIM_COUNTERMODE_UP
	if ((flags & (TIM_CR1_CMS | TIM_CR1_DIR)) != 0u)
	{
		LL_TIM_SetCounterMode (pTim, flags & (TIM_CR1_CMS | TIM_CR1_DIR)) ;
	}

	// Set ARR Auto reload, default value is ARR not buffered
	if ((flags & TIM_INIT_ARR_PRELOADEN) != 0u)
	{
		LL_TIM_EnableARRPreload (pTim) ;
	}

	// Set UDIS bit: Update event disabled
	if ((flags & TIM_INIT_UPDATEDIS) != 0u)
	{
		LL_TIM_EnableUpdateEvent (pTim) ;
	}

	// Set Division ratio for digital filter of ETR or TIx, default is DIV1
	LL_TIM_SetClockDivision (pTim, flags & (TIM_CR1_CKD_1 | TIM_CR1_CKD_0)) ;

	LL_TIM_SetOnePulseMode (pTim, flags & LL_TIM_ONEPULSEMODE_SINGLE) ;
	LL_TIM_SetUpdateSource (pTim, flags & LL_TIM_UPDATESOURCE_COUNTER) ;

	// Compute and set prescaler value
	if (prescaler != 0u)
	{
		LL_TIM_SetPrescaler (pTim, prescaler - 1u) ;
	}
	else
	{
		// Compute the prescaler value
		uint32_t	div = (clock + timeBaseFrequency / 2u) / timeBaseFrequency ;
		if ((pTimDesc->capabilities & TIM_CAP_32_BITS) != 0u)
		{
			// 32 bit CNT: No prescaler
			// TODO : compute prescaler with 64 bits values ?
			LL_TIM_SetPrescaler (pTim, 0u) ;	// Divide by 1
		}
		else
		{
			// 16 bit CNT
			if (div > 65536u)
			{
				// Need prescaler > 1
				// Compute PSC and ARR: this is the well known integer factorization problem.
				// Use the trivial "Trial division" algorithm
				uint32_t	psc, rem, minRem, minPsc ;

				psc = (div + 65535) / 65536 ;
				if (psc == 1u)
				{
					psc = 2u ;	// Min PSC value
				}
				minRem = div ;
				minPsc = psc ;
				for ( ; psc < 65536 ; psc++)
				{
					rem = div - ((div / psc) * psc) ;
					if (rem == 0u)
					{
						// Find an even divider
						minRem = 0 ;
						minPsc = psc ;
						break ;
					}
					if (rem < minRem)
					{
						minRem = rem ;
						minPsc = psc ;
					}
				}
				LL_TIM_SetPrescaler (pTim, minPsc - 1u) ;
			}
			else
			{
				LL_TIM_SetPrescaler (pTim, 0u) ;	// Divide by 1
			}
		}
	}

	// Set ARR value
	if ((flags & TIM_INIT_FORCE_ARR) == 0u)
	{
		// Set ARR register to get the timer frequency specified by timeBaseFrequency
		LL_TIM_SetAutoReload (pTim, __LL_TIM_CALC_ARR (clock, LL_TIM_GetPrescaler (pTim), timeBaseFrequency)) ;
	}
	else
	{
		// TIM_INIT_FORCE_ARR specified: timeBaseFrequency is the ARR exact value
		LL_TIM_SetAutoReload (pTim, timeBaseFrequency) ;
	}
}

//------------------------------------------------------------------------------
//	Set the source of the timer TRGO output
//	flags is one of LL_TIM_TRGO_xxx

void	timSetTriggerOutput		(const timDesc_t * pTimDesc, uint32_t flags)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;

	LL_TIM_SetTriggerOutput (pTim, flags);
}

//------------------------------------------------------------------------------
//	Set the source of the timer TRGI input
//	flags is one of LL_TIM_TS_xxx

void	timSetTriggerInput		(const timDesc_t * pTimDesc, uint32_t flags)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;

	LL_TIM_SetTriggerInput (pTim, flags);
}

//------------------------------------------------------------------------------
//	Initialize timer as encoder mode

// Flags can contain a filter value : LL_TIM_IC_FILTER_FDIVx_Ny defined by xxx_ll_tim.h
// Default value is no filter

void		timEncoderInit (const timDesc_t * pTimDesc, uint32_t flags)
{
	TIM_TypeDef 	* pTim = pTimDesc->pTim ;
	uint32_t		temp ;

	// Check if encoder is available on this timer
	if (! IS_TIM_ENCODER_INTERFACE_INSTANCE (pTim))
	{
		AA_ASSERT (0)	;
		return ;
	}

	// Enable timer clock
	rccEnableClock (pTim, 1u << pTimDesc->timEnPos) ;

	// Reset timer
	rccResetPeriph (pTim, 1u << pTimDesc->timRstPos) ;

	// Enable pins
	gpioConfigurePin (& pTimDesc->pins [TIM_PIN_CH1]) ;
	gpioConfigurePin (& pTimDesc->pins [TIM_PIN_CH2]) ;

	// Select encoder interface mode
	if ((flags & TIM_ENC_X4) != 0u)
	{
		temp = 0x0003u ;		// X4 : Ti2 X2 + T12 X2
	}
	else
	{
		temp = 0x000Eu ;						// Default: Ti1 X1
		if ((flags & TIM_ENC_X2) != 0u)
		{
			// X2 mode selected
			temp ^= 0x000Fu ;					// Ti1 X2
		}
		if ((flags & TIM_ENC_SWITCH) != 0u)
		{
			// Counting on Ti2
			temp = (temp + 1u) & 0x000Fu ;	// Ti2 X1 or X2
		}
	}
	temp = ((temp & 0x0008u) << 13) | (temp & 0x0007u) ;
	pTim->SMCR = (pTim->SMCR & ~TIM_SMCR_SMS) | temp ;

	// Set edge polarity
	if ((flags & TIM_ENC_INVCH1) != 0u)
	{
		LL_TIM_IC_SetPolarity (pTim, LL_TIM_CHANNEL_CH1, LL_TIM_IC_POLARITY_FALLING);
	}
	if ((flags & TIM_ENC_INVCH2) != 0u)
	{
		LL_TIM_IC_SetPolarity (pTim, LL_TIM_CHANNEL_CH2, LL_TIM_IC_POLARITY_FALLING);
	}

	// Activate inputs, e.g. IC1 on TI1, IC2 on TI2
	temp = LL_TIM_ACTIVEINPUT_DIRECTTI ;
	if ((flags & TIM_ENC_IC1_INDIRECTTI) != 0u)
	{
		temp = LL_TIM_ACTIVEINPUT_INDIRECTTI ;
	}
	LL_TIM_IC_SetActiveInput (pTim, LL_TIM_CHANNEL_CH1, temp) ;

	temp = LL_TIM_ACTIVEINPUT_DIRECTTI ;
	if ((flags & TIM_ENC_IC2_INDIRECTTI) != 0u)
	{
		temp = LL_TIM_ACTIVEINPUT_INDIRECTTI ;
	}
	LL_TIM_IC_SetActiveInput (pTim, LL_TIM_CHANNEL_CH2, temp) ;

	// Set filter on IC1 and IC2
	LL_TIM_IC_SetFilter (pTim, LL_TIM_CHANNEL_CH1, flags & 0x000F0000);
	LL_TIM_IC_SetFilter (pTim, LL_TIM_CHANNEL_CH2, flags & 0x000F0000);

	// Set ARR to simulate free running counter
	if ((pTimDesc->capabilities & TIM_CAP_32_BITS) == 0u)
	{
		pTim->ARR = 0xFFFFu ;
	}
	else
	{
		pTim->ARR = 0xFFFFFFFFu ;
	}
}

//------------------------------------------------------------------------------
//	Returns the native pointer to timer registers. Allows application to configure the timer

void		timGetRegPtr	(const timDesc_t * pTimDesc, TIM_TypeDef ** ppTimPtr)
{
	* ppTimPtr = pTimDesc->pTim ;
}

//------------------------------------------------------------------------------
//	Set the timer counter value

void		timSetCounter	(const timDesc_t * pTimDesc, uint32_t value)
{
	LL_TIM_SetCounter (pTimDesc->pTim, value) ;
}

//------------------------------------------------------------------------------
//	Get the current timer counter value

uint32_t	timGetCounter	(const timDesc_t * pTimDesc)
{
	return LL_TIM_GetCounter (pTimDesc->pTim) ;
}

//------------------------------------------------------------------------------
//	Set timer reload counter value. This value is the real value (not the value -1)

void		timSetReload	(const timDesc_t * pTimDesc, uint32_t reload)
{
	pTimDesc->pTim->ARR = reload - 1u ;
}

//------------------------------------------------------------------------------
//	Get timer reload counter value. This value is directly read from the register

uint32_t	timGetReload (const timDesc_t * pTimDesc)
{
	 return (uint32_t)(pTimDesc->pTim->ARR) ;
}

//------------------------------------------------------------------------------
//	Configure timer interrupt
//	Flags contain the interrupt flag mask of the interrupt to enable
//	If TIM_INTR_DISABLE is used all interrupts are disabled
//	To enable DMA interrupts use timDmaChannelInit()
//	COMIE and COMDE not managed

// Warning: some timers (as TIM1 and TIM8) can use more than 1 interrupt (up to 4)
// TODO : this function manage only 1 IRQn per timer

void	timConfigureIntr (const timDesc_t * pTimDesc, uint32_t priority, fnCallback_t pCallback, uintptr_t arg, uint32_t flags)
{
	TIM_TypeDef * pTim = pTimDesc->pTim ;

	if ((flags & TIM_INTR_DISABLE) != 0u)
	{
		// Disable all interrupts of this timer
		pTim->DIER = 0u ;
		NVIC_DisableIRQ (pTimDesc->timIrqn) ;
		return ;
	}

	// Configure the NVIC to handle the TIM interrupt
	NVIC_SetPriority (pTimDesc->timIrqn, priority) ;
	NVIC_EnableIRQ (pTimDesc->timIrqn) ;

	// Do not use timCallbackArray if none timer is used
	#if (WITH_TIM_X	== 1)
	{
		timCallbackArray [pTimDesc->timNumber-1u].pTimDesc	= pTimDesc ;
		timCallbackArray [pTimDesc->timNumber-1u].pCallback	= pCallback ;
		timCallbackArray [pTimDesc->timNumber-1u].arg		= arg ;
	}
	#else
	{
		(void) pCallback ;
		(void) arg ;
	}
	#endif

	if ((flags & TIM_INTR_UPDATE) != 0u)
	{
		LL_TIM_EnableIT_UPDATE (pTim) ;
	}

	if ((flags & LL_TIM_DIER_COMIE) != 0u)
	{
		LL_TIM_EnableIT_COM (pTim) ;
	}

	if ((flags & LL_TIM_DIER_TIE) != 0u)
	{
		LL_TIM_EnableIT_TRIG (pTim) ;
	}

	if ((flags & LL_TIM_DIER_BIE) != 0u)
	{
		LL_TIM_EnableIT_BRK (pTim) ;
	}

	// Enable the capture/compare interrupt for channel
	if ((flags & LL_TIM_DIER_CC1IE) != 0u)
	{
		LL_TIM_EnableIT_CC1 (pTim) ;
	}

	if ((flags & LL_TIM_DIER_CC2IE) != 0u)
	{
		LL_TIM_EnableIT_CC2 (pTim) ;
	}

	if ((flags & LL_TIM_DIER_CC3IE) != 0u)
	{
		LL_TIM_EnableIT_CC3 (pTim) ;
	}

	if ((flags & LL_TIM_DIER_CC4IE) != 0u)
	{
		LL_TIM_EnableIT_CC4 (pTim) ;
	}
}

//------------------------------------------------------------------------------
// Clear DMA pending requests

void		timClearDmaRequest			(const timDesc_t * pTimDesc, uint32_t channel)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;
	uint32_t		reg ;
	
	// Clear DMA pending requests
	reg = pTim->DIER  ;
	pTim->DIER = reg & ~(TIM_DIER_CC1DE << (channel - 1u)) ;
	pTim->DIER = reg ;
}

//------------------------------------------------------------------------------
//	Start timer

void	timStart (const timDesc_t * pTimDesc)
{
	TIM_TypeDef	* pTim = pTimDesc->pTim ;

	// Force update generation: Clear CNT and prescaler
	LL_TIM_GenerateEvent_UPDATE (pTim) ;

	pTim->SR = 0u ;	// Clear all flags, because LL_TIM_GenerateEvent_UPDATE sets UIF

	// Enable counter
	LL_TIM_EnableCounter (pTim) ;
}

//------------------------------------------------------------------------------
//	Stop timer

void	timStop (const timDesc_t * pTimDesc)
{
	// Disable counter
	LL_TIM_DisableCounter (pTimDesc->pTim) ;
}

//------------------------------------------------------------------------------
//	Disable the timer

void	timReset (const timDesc_t * pTimDesc)
{
	// Reset timer
	rccResetPeriph (pTimDesc->pTim, 1u << pTimDesc->timRstPos) ;
}

//------------------------------------------------------------------------------
//	Set channel PWM ratio. Channel 1..4

void	timOutputChannelSet (const timDesc_t * pTimDesc, uint32_t channel, uint32_t value)
{
	volatile uint32_t	* pReg ;
	
	if (channel > 0u  &&  channel < 5u)
	{
		pReg = & pTimDesc->pTim->CCR1 ;
		pReg [channel-1u] = value ;
		//LL_TIM_OC_SetCompareCH1 (pTimDesc->pTim, value);
	}
}

//------------------------------------------------------------------------------
//	Configure an output channel (e.g. PWM).
//	Does manage complementary channels of TIM1 or TIM8
//	Channel		1..4
//	Flags		Combination of bits of TIMx_CCMRx and TIMx_CCER bits in output mode
//	Set the same polarity for CHx and CHxN

//	Also configure the complementary output if required

void	timOutputChannelInit (const timDesc_t * pTimDesc, uint32_t channel, uint32_t flags)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;
	uint32_t		llChannel ;

	llChannel = timGetLlChannel (pTimDesc, channel) ;
	if (llChannel == 0u)
	{
		return ;
	}
	
	// Disable channel output
	LL_TIM_CC_DisableChannel (pTim, llChannel) ;	// CHx
	LL_TIM_CC_DisableChannel (pTim, timGetLlChannel (pTimDesc, channel + TIM_CHxN_OFFSET)) ;	// CHxN

	// Configure the GPIO pin, if it is required
	if ((flags & TIM_OCMODE_NOOUTPUTPIN) == 0u)
	{
		gpioConfigurePin (& pTimDesc->pins [channel - 1u]) ;	// CHx

		// Manage complementary GPIO
		// Complementary GPIO only for channel 1..3
		if (channel < 4u)
		{
			// If the complementary GPIO is defined in the descriptor, configure it.
			if (AA_ENONE  == gpioConfigurePin (& pTimDesc->pins [channel - 1 + TIM_CHxN_OFFSET]))	// CHxN
			{
				// This pin is configured, so enable it in CCR register
				pTim->CCER |= TIM_CCER_CC1NE << (4u * (channel - 1u))  ;
			}
		}
	}

	// Set output mode (OCxM and CCxS). Reset value is LL_TIM_OCMODE_FROZEN
	LL_TIM_OC_SetMode (pTim, llChannel, flags & TIM_CCMR1_OC1M) ;

	// Set TIMx_CCRx register preload (OCxPE).
	if (flags & TIM_OCMODE_PRELOADEN)
	{
		// TIM_CCRx preload value is loaded in the shadow register at each update event.
		LL_TIM_OC_EnablePreload (pTim, llChannel) ;
	}
	else
	{
		// TIM_CCRx preload value is loaded immediately in the shadow register.
		LL_TIM_OC_DisablePreload (pTim, llChannel) ;
	}

	// Set output compare fast
	if (flags & TIM_OCMODE_FASTEN)
	{
		// TIM_CCRx preload value is loaded in the shadow register at each update event.
		LL_TIM_OC_EnableFast (pTim, llChannel) ;
	}
	else
	{
		// TIM_CCRx preload value is loaded immediately in shadow register.
		LL_TIM_OC_DisableFast (pTim, llChannel) ;
	}

	// Set output compare clear on ETRF input high
	if (flags & TIM_OCMODE_CLEAREN)
	{
		// TIM_CCRx preload value is loaded in the shadow register at each update event.
		LL_TIM_OC_EnableClear (pTim, llChannel) ;
	}
	else
	{
		// TIM_CCRx preload value is loaded immediately in shadow register.
		LL_TIM_OC_DisableClear (pTim, llChannel) ;
	}

	// Set output channel polarity. Reset value is LL_TIM_OCPOLARITY_HIGH
	LL_TIM_OC_SetPolarity (pTim, llChannel,       (flags >> 16u) & TIM_CCER_CC1P) ;	// CHx
	LL_TIM_OC_SetPolarity (pTim, llChannel << 2u, (flags >> 16u) & TIM_CCER_CC1P) ;	// CHxN

	// Set compare value to default: half of the counter period (50% duty cycle)
	// This is useful to configure frequency output without managing PWM
	timOutputChannelSet (pTimDesc, channel, ((LL_TIM_GetAutoReload (pTim) + 1u ) / 2u)) ;

	// For some timers (e.g. TIM1 TIM8) need to set MOE
	if (IS_TIM_BREAK_INSTANCE (pTim))
	{
		pTim->BDTR |= TIM_BDTR_MOE ;
	}
}

//------------------------------------------------------------------------------
//	Enable a combination of multiple channels
//	channelMask is not a channel number but a bit mask of the channels to enable
//	Example: to enable channel 3 and channel 4 at the same time use the mask:
//		(1 << 3) | (1 << 4)

void	timChannelEnable	(const timDesc_t * pTimDesc, uint32_t channelMask)
{
	uint32_t		ccer ;

	ccer  = (channelMask & 0x0002u) >> 1u ;
	ccer |= (channelMask & 0x0004u) << 2u ;
	ccer |= (channelMask & 0x0008u) << 5u ;
	ccer |= (channelMask & 0x0010u) << 8u ;

	pTimDesc->pTim->CCER |= ccer ;
}

//------------------------------------------------------------------------------
//	Reset channel PWM configuration, and disable its output.
//	Channel		1..4

void	timChannelDisable (const timDesc_t * pTimDesc, uint32_t channel)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;
	uint32_t		llChannel ;

	llChannel = timGetLlChannel (pTimDesc, channel) ;
	if (llChannel == 0u)
	{
		return ;
	}

	// Disable the channel
	LL_TIM_CC_DisableChannel (pTim, llChannel | (llChannel << 2u)) ;	// CHx | CHxN
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//	Input capture management
//	The prescaler is the real prescaler value (not the value -1)

void	timInputInit		(const timDesc_t * pTimDesc, uint32_t prescaler)
{
	// Enable timer clock
	rccEnableClock (pTimDesc->pTim, 1 << pTimDesc->timEnPos) ;

	// Reset timer
	rccResetPeriph (pTimDesc->pTim, 1 << pTimDesc->timRstPos) ;

	pTimDesc->pTim->PSC = prescaler - 1 ;
}

//------------------------------------------------------------------------------
//	Configure an input capture channel

void	timInputChannelInit	(const timDesc_t * pTimDesc, uint32_t channel, uint32_t flags)
{
	TIM_TypeDef		* pTim = pTimDesc->pTim ;
	uint32_t		ccmr, ccer ;

	if (channel == 0u  ||  channel > 4u)
	{
		AA_ASSERT (0) ;
		return ;
	}

	// Configure GPIO pin, if it is required
	if ((flags & TIM_ICMODE_NOINPUTPIN) == 0u)
	{
		gpioConfigurePin (& pTimDesc->pins [channel - 1u]) ;
	}

	// Set channel in input mode
	// Set active polarity
	// Enable channel capture
	ccmr = flags & 0x000Fu ;				// IC1PSC + CC1S (prescaler + mapping TI1/TI2/TRC)
	ccmr |= (flags >> 16) & 0x00F0u ;		// + IC1F (filter)
	ccer = (flags >> 24) & (TIM_CCER_CC1P | TIM_CCER_CC1NP) ;	// Polarity: IC1P + CC1NP
	switch (channel)
	{
		case 1:
			pTim->CCER  &= ~TIM_CCER_CC1E ;		// Disable channel
			pTim->CCMR1 = (pTim->CCMR1 & ~0x00FFu) | ccmr ;
			pTim->CCER  = (pTim->CCER & 0x000Fu)   | ccer ;
			break ;

		case 2:
			pTim->CCER  &= ~TIM_CCER_CC2E ;		// Disable channel
			pTim->CCMR1 = (pTim->CCMR1 & ~0xFF00u) | (ccmr << 8u) ;
			pTim->CCER  = (pTim->CCER & 0x00F0u)   | (ccer << 4u) ;
			break ;

		case 3:
			pTim->CCER  &= ~TIM_CCER_CC3E ;		// Disable channel
			pTim->CCMR2 = (pTim->CCMR2 & ~0x00FFu) | ccmr ;
			pTim->CCER  = (pTim->CCER & 0x0F00u)   | (ccer << 8u) ;
			break ;

		case 4:
			pTim->CCER  &= ~TIM_CCER_CC4E ;		// Disable channel
			pTim->CCMR2 = (pTim->CCMR2 & ~0xFF00u) | (ccmr << 8u) ;
			pTim->CCER  = (pTim->CCER & 0xF000u)   | (ccer << 12u) ;
			break ;

		default:
			break ;
	}
}

//--------------------------------------------------------------------------------
//	Configure DMA for Input-Capture or Output-Compare
//	Channel		1..4

void	timDmaChannelInit (const dmaDesc_t * pDmaDesc, const timDesc_t * pTimDesc, uint32_t channel)
{
	dma_t			* pDma ;
	dmaChannel_t	* pChannel ;
	TIM_TypeDef 	* pTim = pTimDesc->pTim ;
	uint32_t		chanNum ;
	uint32_t		reg ;

	if (pDmaDesc->dmaNum == 0u)
	{
		return ;	// No DMA to configure
	}

	pDma		= DMA_GETDMA (pDmaDesc->dmaNum) ;
	chanNum		= DMA_GETCHANNEL (pDmaDesc) ;
	pChannel	= dmaChannelAddressRaw (pDma, chanNum) ;

	// Enable DMA clock
	dmaEnableClock (pDma) ;

	// Reset stream
	dmaChannelResetRaw (pDma, chanNum) ;

	// Configure the DMA/Stream functional parameters
	// 16 or 32 bits timer ?
#if (defined STM32F1 || defined STM32F4 || defined STM32G0 || defined STM32G4 || defined STM32L4)
	// F4: with TIM2, some with TIM5
	// F4: with TIM5, some without TIM2
	// G0: some with TIM2 TIM5, some without TIM2 TIM5
	// G4: with TIM2, some without TIM5
	// L4: with TIM2, some without TIM5
	if (
		#if (defined TIM2)
			pTim == TIM2 ||
		#endif
		#if (defined TIM5)
			pTim == TIM5 ||
		#endif
		0
		)
#elif (defined STM32H7)
	if (pTim == TIM2  ||  pTim == TIM5
		#if (defined TIM23)
			||  pTim == TIM23
		#endif
		#if (defined TIM24)
			||  pTim == TIM24
		#endif
		)
#else
	#error "STM32 not defined"
#endif
	{
		// 32 bits timer
		reg = LL_DMA_PDATAALIGN_WORD | LL_DMA_MDATAALIGN_WORD ;
	}
	else
	{
		// 16 bits timer
		reg = LL_DMA_PDATAALIGN_HALFWORD | LL_DMA_MDATAALIGN_HALFWORD ;
	}
	reg |= (pDmaDesc->config          |
			LL_DMA_PERIPH_NOINCREMENT |
			LL_DMA_MEMORY_INCREMENT) ;

	pChannel->CCR &= ~(DMA_CCR_DIR | DMA_CCR_CIRC | DMA_CCR_PINC | DMA_CCR_MINC | DMA_CCR_PSIZE | DMA_CCR_MSIZE | DMA_CCR_PL) ;
	pChannel->CCR |= reg ;

	// Set the DMA multiplexer (transfer request)
//	((dmaMux_t *) DMAMUX1)->CCR [streamNum + ((pDma == (dma_t *) DMA1) ? 0u : 7u)] = pDmaDesc->muxInput ;

	// Set the peripheral address
	switch (channel)
	{
		case 1:
			pChannel->CPAR = (uint32_t) & pTim->CCR1 ;
			pTim->DIER |= TIM_DIER_CC1DE ;
			break ;

		case 2:
			pChannel->CPAR = (uint32_t) & pTim->CCR2 ;
			pTim->DIER |= TIM_DIER_CC2DE ;
			break ;

		case 3:
			pChannel->CPAR = (uint32_t) & pTim->CCR3 ;
			pTim->DIER |= TIM_DIER_CC3DE ;
			break ;

		case 4:
			pChannel->CPAR = (uint32_t) & pTim->CCR4 ;
			pTim->DIER |= TIM_DIER_CC4DE ;
			break ;

		default:
			AA_ASSERT (0) ;
			break ;
	}
}

//--------------------------------------------------------------------------------
// Configure the DMA with the required interrupts
// flags is a combination of: DMA_FLAG_TCIF, DMA_FLAG_HTIF, DMA_FLAG_TEIF

void	timDmaInterruptConfigure (const dmaDesc_t * pDmaDesc, dmaCallback_t pCb, uint32_t arg, uint32_t flags)
{
	dmaChannel_t	* pChannel ;
	uint32_t	reg ;

	dmaSetIrq (pDmaDesc) ;
	dmaSetCallback (pDmaDesc, pCb, arg) ;

	// Enable DMA interrupts
	pChannel = dmaChannelAddress (pDmaDesc)  ;
	reg = pChannel->CCR & ~(DMA_CCR_TCIE | DMA_CCR_HTIE | DMA_CCR_TEIE) ;
	pChannel->CCR = reg | flags ;
}

//--------------------------------------------------------------------------------
// If the timer is 16 bits, pData is a cast to uint32_t
// count is the count of words or half words to receive in pData

void	timDmaStart (const dmaDesc_t * pDmaDesc, uint32_t * pData, uint32_t count)
{
	dma_t			* pDma ;
	dmaChannel_t	* pChannel ;

	AA_ASSERT (pDmaDesc->dmaNum != DMA_NONE) ;
	if (pDmaDesc->dmaNum == DMA_NONE)
	{
		return ;
	}
	pDma    = DMA_GETDMA (pDmaDesc->dmaNum) ;
	pChannel = dmaChannelAddress (pDmaDesc)  ;
	pChannel->CCR		&= ~DMA_CCR_EN ;
	dmaClearFlagsRaw	(pDma, DMA_GETCHANNEL (pDmaDesc), DMA_FLAG_ALLIF) ;
	pChannel->CNDTR		= count ;
	pChannel->CMAR		= (uint32_t) pData ;
	pChannel->CCR		|= DMA_CCR_EN ;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

#define	TIM_SR_IMASK	(TIM_SR_UIF | TIM_SR_BIF | TIM_SR_TIF | TIM_SR_COMIF | TIM_SR_CC4IF |  TIM_SR_CC3IF |  TIM_SR_CC2IF |  TIM_SR_CC1IF)

__STATIC_INLINE void callCallback (TIM_TypeDef * pTim, timCallback_t * pCbDesc)
{
	uint32_t	sr = pTim->SR ;

	if (pCbDesc->pCallback != NULL)
	{
		(* pCbDesc->pCallback) (pCbDesc->pTimDesc, pCbDesc->arg, sr) ;
	}
	if ((sr & TIM_SR_UIF) != 0u)
	{
		pTim->SR = ~TIM_SR_UIF ;	// clear UIF flag
	}
}

//------------------------------------------------------------------------------

#if (WITH_TIM_1 == 1)  ||  (WITH_TIM_9 == 1)
void TIM1_BRK_TIM9_IRQHandler (void)
{
	aaIntEnter () ;

	// Check interrupt source
	#if (WITH_TIM_1 == 1)
	{
		if (((TIM1->SR & TIM_SR_BIF) & TIM1->DIER) != 0u)	// Break Interrupt Flag
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM1, & timCallbackArray [1u-1u]) ;
		}
	}
	#endif	// WITH_TIM1

	#if (WITH_TIM_9 == 1)
	{
		if (((TIM9->DIER & TIM_SR_IMASK) & TIM9->SR) != 0u)
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM9, & timCallbackArray [9u-1u]) ;
		}
	}
	#endif	// WITH_TIM9

	aaIntExit () ;
}
#endif	// WITH_TIM1 || WITH_TIM9

//------------------------------------------------------------------------------

#if (WITH_TIM_1 == 1)  ||  (WITH_TIM_10 == 1)
void TIM1_UP_TIM10_IRQHandler (void)
{
	aaIntEnter () ;

	// Check interrupt source
	#if (WITH_TIM_1 == 1)
	{
		if (((TIM1->SR & TIM_SR_UIF) & TIM1->DIER) != 0u)
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM1, & timCallbackArray [1u-1u]) ;
		}
	}
	#endif	// WITH_TIM1

	#if (WITH_TIM_10 == 1)
	{
		if (((TIM10->SR & TIM_SR_IMASK) & TIM10->DIER) != 0u)
		{
			// Some TIM10 interrupt flag is set
			// User must clear interrupt flags other than UIF
			callCallback (TIM10, & timCallbackArray [10u-1u]) ;
		}
	}
	#endif	// WITH_TIM10

	aaIntExit () ;
}
#endif	// WITH_TIM1 || WITH_TIM10

//------------------------------------------------------------------------------

#if (WITH_TIM_1 == 1)  ||  (WITH_TIM_11 == 1)
void	TIM1_TRG_COM_TIM11_IRQHandler (void)
{
	aaIntEnter () ;

	// Check interrupt source
	#if (WITH_TIM_1 == 1)
	{
		if (((TIM1->SR & (TIM_SR_TIF | TIM_SR_COMIF)) & TIM1->DIER) != 0u)	// Trig Interrupt Flag
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM1, & timCallbackArray [1u-1u]) ;
		}
	}
	#endif	// WITH_TIM1

	#if (WITH_TIM_11 == 1)
	{
		if (((TIM11->SR & TIM_SR_IMASK) & TIM11->DIER) != 0u)
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM11, & timCallbackArray [11u-1u]) ;
		}
	}
	#endif	// WITH_TIM11

	aaIntExit () ;
}
#endif	// WITH_TIM1 || WITH_TIM9

//------------------------------------------------------------------------------

#if (WITH_TIM_1 == 1)
void	TIM1_CC_IRQHandler (void) ;
void	TIM1_CC_IRQHandler (void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM1, & timCallbackArray [1u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM1

//------------------------------------------------------------------------------

#if (WITH_TIM_2 == 1)
void	TIM2_IRQHandler(void) ;
void	TIM2_IRQHandler(void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM2, & timCallbackArray [2u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM2

//------------------------------------------------------------------------------

#if (WITH_TIM_3 == 1)
void	TIM3_IRQHandler(void) ;
void	TIM3_IRQHandler(void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM3, & timCallbackArray [3u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM3

//------------------------------------------------------------------------------

#if (WITH_TIM_4 == 1)
void	TIM4_IRQHandler(void) ;
void	TIM4_IRQHandler (void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM4, & timCallbackArray [4u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM4

//------------------------------------------------------------------------------

#if (WITH_TIM_5 == 1)
void	TIM5_IRQHandler(void) ;
void	TIM5_IRQHandler (void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM5, & timCallbackArray [5u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM5

//------------------------------------------------------------------------------
//	Common interrupt handler for TIM6 and DAC
//	Manage DAC1

#if (WITH_TIM_6 == 1)
void TIM6_IRQHandler (void) ;
void TIM6_IRQHandler (void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM6, & timCallbackArray [6u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM6

//------------------------------------------------------------------------------

#if (WITH_TIM_7 == 1)
void	TIM7_IRQHandler (void) ;
void	TIM7_IRQHandler (void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM7, & timCallbackArray [7u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM7

//------------------------------------------------------------------------------

#if (WITH_TIM_8 == 1)  ||  (WITH_TIM_12 == 1)
void	TIM8_BRK_TIM12_IRQHandler (void) ;
void	TIM8_BRK_TIM12_IRQHandler (void)
{
	aaIntEnter () ;

	// Check interrupt source
	#if (WITH_TIM_8 == 1)
	{
		if (((TIM8->SR & TIM_SR_BIF) & TIM8->DIER) != 0u)	// Break Interrupt Flag
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM8, & timCallbackArray [8u-1u]) ;
		}
	}
	#endif	// WITH_TIM8

	#if (WITH_TIM_12 == 1)
	{
		if (((TIM12->SR & TIM_SR_IMASK) & TIM12->DIER) != 0u)
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM12, & timCallbackArray [12u-1u]) ;
		}
	}
	#endif	// WITH_TIM12

	aaIntExit () ;
}
#endif	// WITH_TIM8 || WITH_TIM12

//------------------------------------------------------------------------------

#if (WITH_TIM_8 == 1)  ||  (WITH_TIM_13 == 1)
void	TIM8_UP_TIM13_IRQHandler (void) ;
void	TIM8_UP_TIM13_IRQHandler (void)
{
	aaIntEnter () ;

	// Check interrupt source
	#if (WITH_TIM_8 == 1)
	{
		if (((TIM8->SR & TIM_SR_UIF) & TIM8->DIER) != 0u)	// Update Interrupt Flag
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM8, & timCallbackArray [8u-1u]) ;
		}
	}
	#endif	// WITH_TIM8

	#if (WITH_TIM_13 == 1)
	{
		if (((TIM13->SR & TIM_SR_IMASK) & TIM13->DIER) != 0u)
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM13, & timCallbackArray [13u-1u]) ;
		}
	}
	#endif	// WITH_TIM13

	aaIntExit () ;
}
#endif	// WITH_TIM8 || WITH_TIM13

//------------------------------------------------------------------------------

#if (WITH_TIM_8 == 1)  ||  (WITH_TIM_14 == 1)
void	TIM8_TRG_COM_TIM14_IRQHandler (void) ;
void	TIM8_TRG_COM_TIM14_IRQHandler (void)
{
	aaIntEnter () ;

	// Check interrupt source
	#if (WITH_TIM_8 == 1)
	{
		if (((TIM8->SR & (TIM_SR_TIF | TIM_SR_COMIF)) & TIM8->DIER) != 0u)	// Trig Interrupt Flag
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM8, & timCallbackArray [8u-1u]) ;
		}
	}
	#endif	// WITH_TIM8

	#if (WITH_TIM_14 == 1)
	{
		if (((TIM14->SR & TIM_SR_IMASK) & TIM14->DIER) != 0u)
		{
			// User must clear interrupt flags other than UIF
			callCallback (TIM14, & timCallbackArray [14u-1u]) ;
		}
	}
	#endif	// WITH_TIM14

	aaIntExit () ;
}
#endif	// WITH_TIM8 || WITH_TIM14

//------------------------------------------------------------------------------

#if (WITH_TIM_8 == 1)
void	TIM8_CC_IRQHandler (void) ;
void	TIM8_CC_IRQHandler (void)
{
	aaIntEnter () ;

	// User must clear interrupt flags other than UIF
	callCallback (TIM8, & timCallbackArray [8u-1u]) ;

	aaIntExit () ;
}
#endif	// WITH_TIM8

//------------------------------------------------------------------------------
