/*
	Generic linker script for Cortex-M STM32F1xx
	This file is included by the specific MCU linker script
 
	Use multi-region initializations for code, data, bss and noinit

 */
 
 /*
 */

/*--------------------------------------------------------------------------
/* The main (privileged) stack definition, default heap definition */

/* Required amount of RAM for main stack */
Main_Stack_Size_ = 1024 ;

_estack_ = ORIGIN(RAM) + LENGTH(RAM);		/* End of main stack pointer just beyond the RAM */
_sstack_ = _estack_  - Main_Stack_Size_ ;	/* Start of main stack	*/

/* Default heap definitions.
 * The heap start immediately after the last statically allocated 
 * .bss/.noinit section, and extends up to the main stack start.
 */
Min_Heap_Size_ = 1024 ;					/* For RAM size check only */
PROVIDE ( _heap_begin_ = _end_noinit ) ;
PROVIDE ( _heap_end_   = _sstack_ ) ;

/* The entry point is informative since the Cortex-M reset vector points to it anyway */
ENTRY(start_)

/*--------------------------------------------------------------------------
 * Sections Definitions
 */

SECTIONS
{
    /*
     * The main stack address, IRQ vectors, the default IRQ handlers and the startup code
     * are stored in the .isr_vectors section, which goes to beginning of FLASH. 
     */
    .isr_vectors : ALIGN(4)
    {
        FILL(0xFF)
        
        KEEP(*(.isr_stackaddress))			/* Interrupt stack address */
        KEEP(*(.isr_vectors))				/* Interrupt vectors */
        *(.isr_handlers .isr_handlers.*)	/* Interrupt handlers */
        *(.startup .startup.*)				/* Startup code*/
		*(.after_vectors*)

    } >FLASH

    .inits : ALIGN (4)
    {
        /* 
         * Memory regions initialization tables.
         *
         * There are two kinds of arrays for each RAM region: one for data and one for bss.
         * The startup code uses these tables to perform the initialization.
         * 
         * The data table includes:
         * - from (LOADADDR()) (in FLASH)
         * - region_begin (ADDR())
         * - region_end (ADDR()+SIZEOF())
         *
         * The bss table includes:
         * - region_begin (ADDR())
         * - region_end (ADDR()+SIZEOF())
         *
         * WARNING: It is mandatory that the regions are word aligned, 
         * since the initialization code works only on words.
         */
         
         /* The startup code uses only the variables defined in this section */
         
        __data_regions_array_start = . ;
        
        /* Initialized data in RAM */
        LONG(LOADADDR(.data)) ;
        LONG(ADDR(.data)) ;
        LONG(ADDR(.data)+SIZEOF(.data)) ;
      
        __data_regions_array_end = .;
        
        __bss_regions_array_start = .;
        
        LONG(ADDR(.bss)) ;
        LONG(ADDR(.bss)+SIZEOF(.bss)) ;
        
        __bss_regions_array_end = . ;

        /* End of memory regions initialization arrays. */
    
		/*--------------------------------------------------------------------------
         * The C++ preinit code, i.e. an array of pointers to initialization 
         * functions to be performed before constructors.
         */
		. = ALIGN (4) ;
		PROVIDE_HIDDEN (__preinit_array_start = .) ;
        
        /* The application inits. If you need to enforce some order in 
         * execution, create new sections before this.
         */
		KEEP (*(.preinit_array .preinit_array.*))
		PROVIDE_HIDDEN (__preinit_array_end = .) ;

		/*--------------------------------------------------------------------------
         * The C++ init code, i.e. an array of pointers to static constructors.
         */
		. = ALIGN (4) ;
		PROVIDE_HIDDEN (__init_array_start = .) ;
		KEEP (*(SORT(.init_array.*)))
		KEEP (*(.init_array*))
		PROVIDE_HIDDEN (__init_array_end = .) ;

		/*--------------------------------------------------------------------------
         * The C++ fini code, i.e. an array of pointers to static destructors.
         * .fini_array ommited because when would we run destructors?
         */
		. = ALIGN (4) ;
		PROVIDE_HIDDEN (__fini_array_start = .) ;
		KEEP (*(SORT(.fini_array.*)))
		KEEP (*(.fini_array))
		PROVIDE_HIDDEN (__fini_array_end = .) ;

    } >FLASH

    /*--------------------------------------------------------------------------
     * The program code is stored in the .text section, which goes to FLASH.
     */
	.text : ALIGN(4)
	{
		*(.text .text.*)			/* All the code */
 
		/* Read-only data (constants) */
		*(.rodata .rodata.* .constdata .constdata.*)

		*(vtable)					/* C++ virtual tables */

		KEEP(*(.eh_frame*))

	} >FLASH

	/* ARM magic sections */
	.ARM.extab : ALIGN(4)
	{
		*(.ARM.extab* .gnu.linkonce.armextab.*)
	} > FLASH

	. = ALIGN(4);
	.ARM.exidx : ALIGN(4)
	{
		__exidx_start = .;   	
		*(.ARM.exidx* .gnu.linkonce.armexidx.*)
		__exidx_end = .;
	} > FLASH
   	
	. = ALIGN(4);
	_etext = .;
    

	/* --------------------------------------------------------------------------
     * The initialized data sections.
     *
     * The program executes knowing that the data is in the RAM
     * but the compiler puts the initial values in the FLASH (inidata).
     * It is one task of the startup to copy the initial values from 
     * FLASH to RAM.
     */
	.data : ALIGN(4)
	{
		FILL(0xFF)

		*(.data)
		*(.data.*)

		. = ALIGN(4);
	} >RAM AT>FLASH

    /*--------------------------------------------------------------------------
     * The zeroized data sections. NOLOAD is used to avoid
     * the section type to be changed to PROGBITS
     */
     
	/* The primary uninitialised data section. */
	.bss (NOLOAD) : ALIGN(4)
	{
		*(.bss)
		*(.bss.*)
		*(COMMON)

		. = ALIGN(4);
	} >RAM

	/* This section MUST be the last one in RAM */
	.noinit (NOLOAD) : ALIGN(4)
	{
		_noinit = .;
		*(.noinit .noinit.*) 

		. = ALIGN(4) ;
		_end_noinit = . ;	/* Mandatory to be word aligned, _sbrk assumes this */
	} > RAM

	PROVIDE ( end  = _end_noinit );		/* For _sbrk, if used */
	PROVIDE ( _end = _end_noinit );
    
    /*--------------------------------------------------------------------------
     * Used for validation only, do not allocate anything in RAM from here
     *
     * This is to check that there is enough RAM left for the Main stack.
     * It generate an error if it's full.
     */
	._check_heap_stack : ALIGN(4)
	{
		. = . + Main_Stack_Size_ + Min_Heap_Size_ ;
	} >RAM

	/*--------------------------------------------------------------------------
	/*--------------------------------------------------------------------------
     * The FLASH Bank2
     * The C or assembly source must explicitly place the code 
     * or data there using the "section" attribute.
	 */
	.text_b2 : ALIGN(4)
	{
		*(.text_b2 .test_b2.*)			/* The code */
		*(.rodata_b2*)					/* Read-only data (constants) */
		*(.rodata_b2.*)
	} >FLASHB2
    
    /*--------------------------------------------------------------------------
    /*--------------------------------------------------------------------------
     * The EXTMEM.
     * The C or assembly source must explicitly place the code or data there
     * using the "section" attribute.
     */

    /* EXTMEM Bank0 */
    .eb0text : ALIGN(4)
    {
        *(.eb0text)                   /* remaining code */
        *(.eb0rodata)                 /* read-only data (constants) */
        *(.eb0rodata.*)
    } >EXTMEMB0
    
    /* EXTMEM Bank1 */
    .eb1text : ALIGN(4)
    {
        *(.eb1text)                   /* remaining code */
        *(.eb1rodata)                 /* read-only data (constants) */
        *(.eb1rodata.*)
    } >EXTMEMB1
    
    /* EXTMEM Bank2 */
    .eb2text : ALIGN(4)
    {
        *(.eb2text)                   /* remaining code */
        *(.eb2rodata)                 /* read-only data (constants) */
        *(.eb2rodata.*)
    } >EXTMEMB2
    
    /* EXTMEM Bank3 */
    .eb3text : ALIGN(4)
    {
        *(.eb3text)                   /* remaining code */
        *(.eb3rodata)                 /* read-only data (constants) */
        *(.eb3rodata.*)
    } >EXTMEMB3
   
	/*--------------------------------------------------------------------------*/
	/*--------------------------------------------------------------------------*/

	/* Remove the debugging information from the standard libraries */    
	DISCARD :
	{
		libc.a ( * )
		libm.a ( * )
		libgcc.a ( * )
	}
}
