/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testadc.c	ADC test for STM32F103RB
				From

	When		Who	What
	21/08/18	ac	Creation

	This test generate a signal for 8 RC servo from 8 analog inputs
	The signal is generated with a TIM PWM output

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aaprintf.h"

#include	"gpiobasic.h"
#include	"timbasic.h"

#include	"stm32f1xx_ll_gpio.h"
#include	"stm32f1xx_ll_adc.h"
#include	"stm32f1xx_ll_bus.h"

//------------------------------------------------------------------------------

/* Delay between ADC enable and ADC end of calibration.                     */
/* Delay estimation in CPU cycles: Case of ADC calibration done             */
/* immediately after ADC enable, ADC clock setting slow                     */
/* (LL_ADC_CLOCK_ASYNC_DIV32). Use a higher delay if ratio                  */
/* (CPU clock / ADC clock) is above 32.                                     */
#define ADC_DELAY_ENABLE_CALIB_CPU_CYCLES  (LL_ADC_DELAY_ENABLE_CALIB_ADC_CYCLES * 32)

//------------------------------------------------------------------------------

static void	adcInit (void)
{
	/* Enable ADC clock (core clock) */
	LL_APB2_GRP1_EnableClock (LL_APB2_GRP1_PERIPH_ADC1) ;

	/* Set ADC conversion data alignment */
//	LL_ADC_SetResolution(ADC1, LL_ADC_DATA_ALIGN_RIGHT) ;

	/* Set ADC group regular trigger source */
	LL_ADC_REG_SetTriggerSource (ADC1, LL_ADC_REG_TRIG_SOFTWARE) ;

	/* Set ADC group regular trigger polarity */
//	LL_ADC_REG_SetTriggerEdge (ADC1, LL_ADC_REG_TRIG_EXT_RISING) ;

	/* Set ADC group regular continuous mode */
	LL_ADC_REG_SetContinuousMode (ADC1, LL_ADC_REG_CONV_SINGLE) ;

	/* Set ADC group regular sequencer length and scan direction */
	LL_ADC_REG_SetSequencerLength (ADC1, LL_ADC_REG_SEQ_SCAN_DISABLE) ;

	/* Set ADC group regular sequencer discontinuous mode */
//	LL_ADC_REG_SetSequencerDiscont (ADC1, LL_ADC_REG_SEQ_DISCONT_DISABLE) ;
}

//------------------------------------------------------------------------------

static void	adcActivate ()
{
	__IO uint32_t wait_loop_index = 0 ;

	// Enable ADC
	LL_ADC_Enable(ADC1);

	// Delay between ADC enable and ADC start of calibration.
	// Note: Variable divided by 2 to compensate partially
	//       CPU processing cycles (depends on compilation optimization).
	wait_loop_index = (ADC_DELAY_ENABLE_CALIB_CPU_CYCLES >> 1);
	while (wait_loop_index != 0)
	{
		wait_loop_index--;
	}

	// Run ADC self calibration
	LL_ADC_StartCalibration(ADC1);

	// Poll for ADC effectively calibrated
	while (LL_ADC_IsCalibrationOnGoing (ADC1) != 0)
	{
	}
}

//------------------------------------------------------------------------------
// Conversion time ~12 us with ADC clock is 8 MHz and LL_ADC_SAMPLINGTIME_71CYCLES_5

static uint32_t adcStart (uint32_t llChannel)
{
	uint32_t	data ;

	// Set ADC group regular sequence: channel on the selected sequence rank.
	LL_ADC_REG_SetSequencerRanks (ADC1, LL_ADC_REG_RANK_1, llChannel);

	// Set ADC channels sampling time
	LL_ADC_SetChannelSamplingTime(ADC1, llChannel, LL_ADC_SAMPLINGTIME_71CYCLES_5);

	// Start conversion
	LL_ADC_REG_StartConversionSWStart (ADC1) ;

	// Wait end of conversion
    while (LL_ADC_IsActiveFlag_EOS (ADC1) == 0)
    {
    }

    LL_ADC_ClearFlag_EOS (ADC1) ;

    // Get data
    data = LL_ADC_REG_ReadConversionData12 (ADC1) ;
    return data ;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

#define	MAX_ADC_INPUT	8
#define	ADC_MEAN		8

const	gpioPinDesc_t	adcInputs [MAX_ADC_INPUT] =
{
	{	'A',	0,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_0
	{	'B',	0,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_8
	{	'A',	7,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_7
	{	'A',	6,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_6
	{	'C',	0,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_10
	{	'C',	1,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_11
	{	'A',	1,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_1
	{	'A',	4,	AA_GPIO_MODE_ANALOG,	0	},	// LL_ADC_CHANNEL_4
} ;

typedef struct
{
	uint32_t	adcLlChannelNumber ;
	uint32_t	offset ;
	uint32_t	bInvert ;
} anaConf_t ;

const	anaConf_t adcChannelConfig  [MAX_ADC_INPUT] =
{
	{	LL_ADC_CHANNEL_0,	1566,	0	},
	{	LL_ADC_CHANNEL_8,	1554,	0	},
	{	LL_ADC_CHANNEL_7,	1564,	1	},
	{	LL_ADC_CHANNEL_6,	1592,	1	},
	{	LL_ADC_CHANNEL_10,	0,		0	},
	{	LL_ADC_CHANNEL_11,	0,		0	},
	{	LL_ADC_CHANNEL_1,	0,		0	},
	{	LL_ADC_CHANNEL_4,	0,		0	},
} ;


uint32_t	analogValues [MAX_ADC_INPUT] ;
uint32_t	anaMin [MAX_ADC_INPUT] ;
uint32_t	anaMax [MAX_ADC_INPUT] ;

//------------------------------------------------------------------------------

void 	adcConvertChannels ()
{
	uint32_t	nn, ii ;
	uint32_t	datam, data1 ;

	for (nn = 0 ; nn < MAX_ADC_INPUT ; nn++)
	{
		datam = 0 ;
		for (ii = 0 ; ii < ADC_MEAN ; ii++)
		{
bspOutput (BSP_OUTPUT0, 1) ;
			data1 = adcStart (adcChannelConfig[nn].adcLlChannelNumber) ;
bspOutput (BSP_OUTPUT0, 0) ;
			datam += data1 ;
			if (data1 > anaMax [nn])
			{
				anaMax [nn] = data1 ;
			}
			if (data1 < anaMin [nn])
			{
				anaMin [nn] = data1 ;
			}
		}
		analogValues [nn] = datam / ADC_MEAN ;
	}
}

//------------------------------------------------------------------------------
//	Radio Controller signal generator

#define	RC_MIN			700			// Minimum pulse width (us)
#define	RC_GAP			300			// Gap between pulses
#define	RC_FRAMEGAP		12000		// Gap between frames
#define	RC_COUNT		4			// Count of pulses

#define	RC_ID			41			// Timer descriptor id for TIM4.
#define	RC_CHANNEL		1			// 1..4.  PWM is output on PB6

#define	TIM_PRIORITY	BSP_IRQPRIOMAX_MINUS(2)		// (time base test)

static	uint32_t	rcIndex ;		// Index of next value in rcValues

// The RC simulator timer use an interrupt callback
// The only cause is UPDATE
static	void rcCallback (const timDesc_t * pTimDesc, uintptr_t arg, uint32_t cause)
{
	// Check if current pulse is the last pulse of the frame
	if (rcIndex == RC_COUNT)
	{
		// Gap at end of frame
		timSetReload (pTimDesc, RC_MIN+ RC_FRAMEGAP + RC_GAP ) ;
		timOutputChannelSet (pTimDesc, RC_CHANNEL, RC_MIN + RC_FRAMEGAP) ;

		aaSignalSend (arg, 1) ;		// Send task signal to get analog values for next frame
	}
	else
	{
		// Set current pulse values (ARR and CC registers)
		timSetReload (pTimDesc, RC_MIN + RC_GAP + analogValues [rcIndex] - 1) ;
		timOutputChannelSet (pTimDesc, RC_CHANNEL, RC_MIN + analogValues [rcIndex]) ;
	}

	// Update index: RC_COUNT pulse + 1 large pulse at end of frame
	rcIndex ++ ;
	if (rcIndex == (RC_COUNT + 1))
	{
		rcIndex = 0 ;
	}

	(void) cause ;
	// UIF flag is cleared by interrupt handler
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

void	adcTest ()
{
	const timDesc_t		* pTimDesc ;
	uint32_t			ii, nn ;
	uint32_t			bDisplay = 1 ;
	int32_t				cc ;


	aaPrintf ("RC controller simulator\n") ;
/*
	while (1)
	{
bspOutput (BSP_OUTPUT0, 1) ;
		aaTaskDelay (250) ;
bspOutput (BSP_OUTPUT0, 0) ;
		aaTaskDelay (250) ;
		aaPrintf ("RC controller simulator\n") ;
	}
*/
	// Configure analog I/O and ADC
	for (ii = 0 ; ii < MAX_ADC_INPUT ; ii++)
	{
		gpioConfigurePin (& adcInputs [ii]) ;
		anaMin [ii] = 4095 ;
		anaMax [ii] = 0 ;
	}

	adcInit () ;
	adcActivate () ;

	// Configure timer
	if (timGetDesc (RC_ID, & pTimDesc) == TIM_EFAIL)
	{
		aaPrintf ("Unknown timer descriptor id: %u\n", RC_ID) ;
		return ;
	}

	// Configure time base for 1MHz timer clock and arbitrary RC period
	timTimeBaseInit  (pTimDesc, bspGetSysClock() / 1000000, RC_MIN + RC_GAP - 1, TIM_INIT_FORCE_ARR) ;
	timConfigureIntr (pTimDesc, TIM_PRIORITY, rcCallback, aaTaskSelfId (), TIM_INTR_UPDATE) ;

	// Configure channel. Use TIM_OCMODE_PWM1 to get positive pulses
	timOutputChannelInit   (pTimDesc, RC_CHANNEL, TIM_OCMODE_PWM1) ;
	timChannelEnable (pTimDesc, RC_CHANNEL) ;

	// Get initial analog values
	adcConvertChannels () ;

	// Initialize 1st timer pulse
	timSetReload (pTimDesc, RC_MIN + analogValues [0] + RC_GAP) ;
	timOutputChannelSet (pTimDesc, RC_CHANNEL, RC_MIN + analogValues [0]) ;
	rcIndex = 1 ;
	timStart (pTimDesc) ;

	ii = 0 ;
	while (1)
	{
		// Wait for end of frame
		aaSignalWait (1, NULL, AA_SIGNAL_AND, AA_INFINITE) ;

		// Get new analog values
		adcConvertChannels () ;

		// Convert analog values to pulses durations
		for (nn = 0 ; nn < MAX_ADC_INPUT ; nn++)
		{
			// Remove analog channel offset: pulse duration is 0 to ~1000
			cc = (int32_t) analogValues [nn] - adcChannelConfig[nn].offset ;
			if (cc < 0)
			{
				cc = 0 ;
			}

			// Some channels are inverted
			if (adcChannelConfig[nn].bInvert == 1)
			{
				if (cc > 1000)
				{
					cc = 1000 ;
				}
				cc = 1000 - cc ;
			}
			analogValues [nn] = cc ;	// This is the pulse duration less RC_MIN
		}

		// Display data every 10 loop
		ii++ ;
		if (ii == 10)
		{
			// Display analog values
			if (bDisplay == 1)
			{
				for (nn = 0 ; nn < MAX_ADC_INPUT ; nn++)
				{
					aaPrintf ("%4u ", analogValues [nn] ) ;
				}
				aaPrintf ("\n") ;
			}
			ii = 0 ;
		}

		// Check for keyboard command
		cc = aaCheckGetChar() ;
		if (cc != 0)
		{
			cc = aaGetChar () ;

			if (cc == 'd')
			{
				// On / OFF periodic display
				bDisplay ^= 1 ;
			}
			if (cc == 'm')
			{
				// Display min/max values and reset min/max
				aaPrintf ("min max\n") ;
				for (ii = 0 ; ii < MAX_ADC_INPUT ; ii++)
				{
					aaPrintf ("%4u ", anaMin [ii]) ;
				}
				aaPrintf ("\n") ;
				for (ii = 0 ; ii < MAX_ADC_INPUT ; ii++)
				{
					aaPrintf ("%4u ", anaMax [ii]) ;
				}
				aaPrintf ("\n") ;
				for (ii = 0 ; ii < MAX_ADC_INPUT ; ii++)
				{
					aaPrintf ("%4u ", anaMax [ii] - anaMin [ii]) ;
					anaMin [ii] = 4095 ;
					anaMax [ii] = 0 ;
				}
				aaPrintf ("\n\n") ;
			}
		}
	}
}

//------------------------------------------------------------------------------

