/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	testi2c.c	I2C basic API test

	When		Who	What
	01/03/18	ac	Creation

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"i2cbasic.h"
#include	"aaprintf.h"

// Variables related to slave receive process
#define SLAVE_OWN_ADDRESS	0x5A 	// Left shifted 7 bit address
									/* This value is a left shift of a real 7 bits of a slave address
									value which can find in a Datasheet as example: b0101101
									mean in uint8_t equivalent at 0x2D and this value can be
									seen in the OAR1 register in bits ADD[1:7] */
// Text values of i2cStatus_t enum
const char * i2cErrText [] =
{
	"I2C_ENONE",
	"I2C_EOK",
	"I2C_EAF",
	"I2C_EARLO",
	"I2C_TMO",
	"I2C_EBERR",
	"I2C_EOVR",
	"I2C_ESIZE",
} ;

uint8_t		aSlaveBuffer  [0xF] ;
uint8_t		aMasterBuffer [0xF] ;

void	i2cTest ()
{
	volatile i2cStatus_t	masterStatus ;
	volatile i2cStatus_t	slaveStatus ;
	uint32_t				slaveSize;

	const i2cDesc_t		* pI2cMaster = i2cGetDesc (2) ;
	const i2cDesc_t		* pI2cSlave  = i2cGetDesc (1) ;

	aaPrintf ("\nI2C tests\n") ;

	if (pI2cMaster ==  NULL  ||  pI2cSlave ==  NULL)
	{
		aaPrintf ("i2cGetDesc error: Master=%08X, Slave=%08X", pI2cMaster, pI2cSlave) ;
		return ;
	}

	i2cConfigureSlave (pI2cSlave, SLAVE_OWN_ADDRESS);

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	//	Master polling functions

	//	Check i2cMasterWrite()

	i2cConfigureMaster (pI2cMaster, 100000) ;
	i2cConfigureSlave (pI2cSlave, SLAVE_OWN_ADDRESS);

	aaPrintf ("\ni2cMasterWrite test:\n") ;
	aaPrintf ("Master send 4 bytes, slave accept only 3 bytes:\n") ;
	aaMEMSET (aSlaveBuffer, 0, sizeof (aSlaveBuffer));
	aaSTRCPY ((char *) aMasterBuffer, "4byte") ;
	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 3) ;
	masterStatus = i2cMasterStart (pI2cMaster, SLAVE_OWN_ADDRESS, I2C_TX) ;
	aaPrintf ("Master start %s\n", i2cErrText [masterStatus]) ;
	if (masterStatus == I2C_EOK)
	{
		masterStatus = i2cMasterWrite (pI2cMaster, aMasterBuffer, 4) ;
		i2cMasterStop (pI2cMaster) ;
		slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
		aaPrintf ("Master send 4 byte %s, Slave %s %u, Slave received:  %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aSlaveBuffer) ;
	}
	i2cMasterStop (pI2cMaster) ;
	aaTaskDelay (10) ;

	// Check slave NACK
	aaPrintf ("\ni2cCheckDevice test:\n") ;
	i2cDeactivate (pI2cSlave) ;
	masterStatus = i2cCheckDevice (pI2cMaster, SLAVE_OWN_ADDRESS) ;
	aaPrintf ("i2cCheckDevice KO: %s\n", i2cErrText [masterStatus]) ;

	// Check slave ACK
	i2cConfigureSlave (pI2cSlave, SLAVE_OWN_ADDRESS);
	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 3) ;
	masterStatus = i2cCheckDevice (pI2cMaster, SLAVE_OWN_ADDRESS) ;
	aaPrintf ("i2cCheckDevice OK: %s\n", i2cErrText [masterStatus]) ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Check i2cMasterRead()

	aaPrintf ("\ni2cMasterRead test:\n") ;
	aaSTRCPY ((char *) aSlaveBuffer, "Slavexx") ;
	aaMEMSET (aMasterBuffer, 0, sizeof (aMasterBuffer));

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, aaSTRLEN ((char *) aSlaveBuffer)) ;
	masterStatus = i2cMasterStart (pI2cMaster, SLAVE_OWN_ADDRESS, I2C_RX) ;
	aaPrintf ("Master start %s\n", i2cErrText [masterStatus]) ;
	if (masterStatus == I2C_EOK)
	{
		masterStatus = i2cMasterRead (pI2cMaster, aMasterBuffer, 2) ;
		aaPrintf ("i2cMasterRead     2 %s\n", i2cErrText [masterStatus]) ;
		masterStatus = i2cMasterReadLast (pI2cMaster, aMasterBuffer+2, aaSTRLEN ((char *) aSlaveBuffer)-2) ;
		aaPrintf ("i2cMasterReadLast %d %s\n", aaSTRLEN ((char *) aSlaveBuffer)-2, i2cErrText [masterStatus]) ;
		slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
		aaPrintf ("Slave %s sent %u bytes, Master received: %s\n", i2cErrText [slaveStatus], slaveSize, aMasterBuffer) ;
	}
	aaTaskDelay (10) ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Check i2cMasterReadStraight()
	aaPrintf ("\ni2cMasterReadStraight test:\n") ;

	aaSTRCPY ((char *) aSlaveBuffer, "Slavexx") ;

	// 1 byte
	aaMEMSET (aMasterBuffer, 0, sizeof (aMasterBuffer));
	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 1) ;
	masterStatus = i2cMasterReadStraight (pI2cMaster, SLAVE_OWN_ADDRESS, aMasterBuffer, 1) ;
	slaveStatus  = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("i2cMasterReadStraight 1 byte  %s, Slave %s %u, Master received: %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aMasterBuffer) ;
	aaTaskDelay (10) ;

	// 2 bytes
	aaMEMSET (aMasterBuffer, 0, sizeof (aMasterBuffer));
	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 2) ;
	masterStatus = i2cMasterReadStraight (pI2cMaster, SLAVE_OWN_ADDRESS, aMasterBuffer, 2) ;
	slaveStatus  = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("i2cMasterReadStraight 2 bytes %s, Slave %s %u, Master received: %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aMasterBuffer) ;
	aaTaskDelay (10) ;

	// 3 bytes
	aaMEMSET (aMasterBuffer, 0, sizeof (aMasterBuffer));
	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 3) ;
	masterStatus = i2cMasterReadStraight (pI2cMaster, SLAVE_OWN_ADDRESS, aMasterBuffer, 3) ;
	slaveStatus  = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("i2cMasterReadStraight 3 bytes %s, Slave %s %u, Master received: %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aMasterBuffer) ;
	aaTaskDelay (10) ;



	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#if defined WITH_I2C_MASTER_INTR  &&  WITH_I2C_MASTER_INTR == 1

	aaPrintf ("\nMaster interrupt tests:\n") ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// 1 or 2 byte transfer interrupt master -> slave
	i2cConfigureMaster (pI2cMaster, 100000) ;

	aaMEMSET (aSlaveBuffer, 0, sizeof (aSlaveBuffer));
	aaSTRCPY ((char *) aMasterBuffer, "1byte") ;

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 1) ;
	masterStatus = i2cMasterWriteIt (pI2cMaster, aMasterBuffer, 1, SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master 1 byte TX intr %s, Slave %s %u, Slave receive:  %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aSlaveBuffer) ;
	aaTaskDelay (10) ;


	aaMEMSET (aSlaveBuffer, 0, sizeof (aSlaveBuffer));
	aaSTRCPY ((char *) aMasterBuffer, "2byte") ;

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 3) ;
	masterStatus = i2cMasterWriteIt (pI2cMaster, aMasterBuffer, 2, SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master 2 byte TX intr %s, Slave %s %u, Slave receive:  %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aSlaveBuffer) ;
	aaTaskDelay (10) ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// 1 or 2 byte transfer interrupt slave -> master
	i2cConfigureMaster (pI2cMaster, 100000) ;

	aaMEMSET (aMasterBuffer, 0, sizeof (aMasterBuffer));
	aaSTRCPY ((char *) aSlaveBuffer, "1byte") ;

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 1) ;
	masterStatus = i2cMasterReadIt (pI2cMaster, aMasterBuffer, 1, SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master 1 byte RX intr %s, Slave %s %u, Master receive:  %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aMasterBuffer) ;
	aaTaskDelay (10) ;

	aaMEMSET (aMasterBuffer, 0, sizeof (aMasterBuffer));
	aaSTRCPY ((char *) aSlaveBuffer, "2byte") ;

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 2) ;
	masterStatus = i2cMasterReadIt (pI2cMaster, aMasterBuffer, 2, SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master 2 byte RX intr %s, Slave %s %u, Master receive:  %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aMasterBuffer) ;
	aaTaskDelay (10) ;

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	// Master transmit buffer larger/smaller than slave receive buffer
	i2cConfigureMaster (pI2cMaster, 100000) ;

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 4) ;
	masterStatus = i2cMasterWriteIt (pI2cMaster, aMasterBuffer, 6, SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master intr Tx too large 6/4 %s, Slave %s %u\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize) ;
	aaTaskDelay (10) ;

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, 6) ;
	masterStatus = i2cMasterWriteIt (pI2cMaster, aMasterBuffer, 4, SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master intr Tx too small 4/6 %s, Slave %s %u\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize) ;
	aaTaskDelay (10) ;

	// Master transmit interrupt
	i2cConfigureMaster (pI2cMaster, 100000) ;

	aaMEMSET (aSlaveBuffer, 0, sizeof (aSlaveBuffer));
	aaSTRCPY ((char *) aMasterBuffer, "Master") ;

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, aaSTRLEN ((char *) aMasterBuffer)) ;
	masterStatus = i2cMasterWriteIt (pI2cMaster, aMasterBuffer, aaSTRLEN ((char *) aMasterBuffer), SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master TX intr %s, Slave %s %u, Slave receive:  %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aSlaveBuffer) ;
	aaTaskDelay (10) ;


	// Master receive interrupt
	aaSTRCPY ((char *) aSlaveBuffer, "Slave") ;
	aaMEMSET (aMasterBuffer, 0, sizeof (aMasterBuffer));

	i2cSlaveReadWrite (pI2cSlave, aSlaveBuffer, aaSTRLEN ((char *) aSlaveBuffer)) ;
	masterStatus = i2cMasterReadIt (pI2cMaster, aMasterBuffer, aaSTRLEN ((char *) aSlaveBuffer), SLAVE_OWN_ADDRESS) ;
	slaveStatus = i2cSlaveResult (pI2cSlave, & slaveSize) ;
	aaPrintf ("Master RX intr %s, Slave %s %u, Master receive: %s\n", i2cErrText [masterStatus], i2cErrText [slaveStatus], slaveSize, aMasterBuffer) ;
	aaTaskDelay (10) ;

#endif	// WITH_I2C_MASTER_INTR
}
